//
//  AppDelegate.m
//  Messangi
//
//  Created by Oscar Anzola on 5/23/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "AppDelegate.h"
#import "User.h"
#import "LocationManager.h"
#import "DeviceSubscriber.h"
#import "CheckGeoFences.h"

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    
    [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[ NSString stringWithFormat:@"%@-Defaults", [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleName"]] ofType:@"plist"]]];

    [[UIApplication sharedApplication] registerForRemoteNotificationTypes:
     (UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound | UIRemoteNotificationTypeAlert)];
    
    _appUser = [[User alloc] init];

    /***Permit background fetch***/
    [[UIApplication sharedApplication] setMinimumBackgroundFetchInterval:UIApplicationBackgroundFetchIntervalMinimum];
    /*****/

    if (![[[LocationManager sharedInstance] delegate] isKindOfClass:[LocationManager class]]) {
            [[LocationManager sharedInstance] setDelegate:[LocationManager sharedInstance]];
    }

    // This is what determines if the authentication screen is shown or not
    _authenticated = NO;
    
    // Check to see if user is authenticated
    if (_authenticated) {
        // Set the root view controller to the tab bar controller
        self.window.rootViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]] instantiateInitialViewController];
    } else {
        // User isn't authenticated so load authentication controller
        UIViewController* rootController = [[UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]] instantiateViewControllerWithIdentifier:@"AuthViewController"];
        UINavigationController* navigation = [[UINavigationController alloc] initWithRootViewController:rootController];
        
        self.window.rootViewController = navigation;
    }
    
    
    return YES;
}

-(void)deviceSubscriberWithUniqueID
{
    DeviceSubscriber *ds = [[DeviceSubscriber alloc] init];
    ds.delegate = self;
    [ds registerDeviceWithUniqueID];
}
							
- (void)application:(UIApplication*)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    NSString *hexToken = [UtilitiesLibrary NSDataToNSString:deviceToken];
    FLog(@"Device Token:  %@", hexToken);
    
    /*
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    NSString *oldToken = [defaults objectForKey:@"devicePushID"];
    
    if (![oldToken isEqualToString:hexToken]){
        [defaults setObject:hexToken forKey:@"devicePushID"];
        [defaults synchronize];
    }
    
    
    BOOL isThisTheFirstTime = [defaults boolForKey:@"isFirstTime"];
     
    if (isThisTheFirstTime || ![oldToken isEqualToString:hexToken]){
        
        [self deviceSubscriberWithUniqueID];
    }
     */
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:hexToken forKey:@"devicePushID"];
    [defaults synchronize];
   [self deviceSubscriberWithUniqueID];

 }

- (void)application:(UIApplication*)application didFailToRegisterForRemoteNotificationsWithError:(NSError*)error
{
	FLog(@"Failed to get token, error: %@", error);
    UIAlertView * errorAlert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"ErrorAPNSTitle", nil)
                                                          message:NSLocalizedString(@"ErrorAPNS", nil)
                                                         delegate:self
                                                cancelButtonTitle:NSLocalizedString(@"OK", nil)
                                                otherButtonTitles:nil];
    [errorAlert show];
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:nil forKey:@"devicePushID"];
    [defaults synchronize];
}

/*
- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo {
    
    if(application.applicationIconBadgeNumber == 0)
        application.applicationIconBadgeNumber = 1;
    application.applicationIconBadgeNumber = 0;
    
    // OA: serious doubt this will work
    UITableViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]] instantiateViewControllerWithIdentifier:@"InboxTableViewController"];
    
    NSString *msgID = [userInfo objectForKey:@"msgId"];
    
    SingleMessageLoader *incomingMessage = [[SingleMessageLoader alloc] init];
    incomingMessage.delegate = vc;
    
    FLog(@"FUNCION CHIMBA PUSH: %@", userInfo);
    FLog(@"MsgID: %@", msgID);
    
    [incomingMessage startLoadingDataForMessageWithID:msgID];
    
    
    //[vc messageLoadFinishedReturningMessage:incomingMessage.currentMessage];
    
}
*/

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler
{
    FLog(@"Silent Notification Recieved");
    FLog(@"content: %@", userInfo);
    
    /*Unregister old regions*
     FLog(@"Background: Unregister old regions");
     [[LocationManager sharedInstance] unregisterOldRegions];*/
    
    
    NSString *msgID = [NSString stringWithFormat:@"%@", [userInfo objectForKey:@"msgId"]];
    NSString *msgType = [NSString stringWithFormat:@"%@", [userInfo objectForKey:@"type"]];
    
    if ([msgType isEqualToString:@"GET_LOC"]) {
        //TODO
        FLog(@"It is GET_LOC");
        // PUSH TYPE: GET_LOC
        // PUSH CONTENT: REQUEST_ID
        
        [[LocationManager sharedInstance] setTypeOfGeoPush:2];
        
        [[LocationManager sharedInstance] setRequestId:msgID];
        
        if ([CLLocationManager locationServicesEnabled] && ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusAuthorized) ) {
            
            FLog(@"getting location. background: %d", [[UIApplication sharedApplication] backgroundRefreshStatus]);
            [[LocationManager sharedInstance] startStandardUpdatesWithCompletionHandler:completionHandler];
            
        }else{
            
            FLog(@"Location Service problems. Enabled: %d Authorization: %d", [CLLocationManager locationServicesEnabled], [CLLocationManager authorizationStatus]);
            completionHandler(UIBackgroundFetchResultNoData);
        }
        
        
    } else if ([msgType isEqualToString:@"CRT_REGION"] || [msgType isEqualToString:@"UPDT_REGION"]) {
        
        FLog(@"It is %@", msgType);
        // PUSH TYPE: CRT_REGION || PUSH TYPE: UPDT_REGION
        // PUSH CONTENT: REGION_ID:LATITUDE:LONGITUDE:RADIUS:TRIGGER
        
        NSArray *data = [msgID componentsSeparatedByString:@":"];
        
        if ([CLLocationManager locationServicesEnabled] && ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusAuthorized) ) {
            
            CLLocation* coordinate = [[CLLocation alloc] initWithLatitude:[[data objectAtIndex:1] doubleValue] longitude:[[data objectAtIndex:2] doubleValue]];
            
            
            [[LocationManager sharedInstance] registerRegionWithID:[data objectAtIndex:0] Center:coordinate Radius:[[data objectAtIndex:3] doubleValue] andTrigger:[data objectAtIndex:4]];
            
            completionHandler(UIBackgroundFetchResultNewData);
            
        }else{
            
            FLog(@"Location Service problems. Enabled: %d Authorization: %d", [CLLocationManager locationServicesEnabled], [CLLocationManager authorizationStatus]);
            completionHandler(UIBackgroundFetchResultNoData);
        }
        
    } else if ([msgType isEqualToString:@"DLT_REGION"]) {
        //TODO
        FLog(@"It is DLT_REGION");
        // PUSH TYPE: DLT_REGION
        // PUSH CONTENT: REGION_ID
        
        
        [[LocationManager sharedInstance] unregisterRegionWithID:msgID];
        
        completionHandler(UIBackgroundFetchResultNewData);
        
    } else { //if ([msgType isEqualToString:@"TXT_MSG"])
        
        if ([msgID rangeOfString:@"GEOPUSH"].location != NSNotFound) {
            
            FLog(@"It is GEOPUSH");
            [[LocationManager sharedInstance] setTypeOfGeoPush:1];
            
            NSArray *data = [msgID componentsSeparatedByString:@":"];
            
            if ([CLLocationManager locationServicesEnabled] && ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusAuthorized) ) {
                
                if (![LocationManager sharedInstance].message) {
                    [LocationManager sharedInstance].message = [[UnstoredIncomingMessage alloc] init];
                }
                
                [[LocationManager sharedInstance].message setMessageID:[data objectAtIndex:1]];
                
                CLLocation* coordinate = [[CLLocation alloc] initWithLatitude:[[data objectAtIndex:2] doubleValue] longitude:[[data objectAtIndex:3] doubleValue]];
                
                [[LocationManager sharedInstance] setRadius: [[data objectAtIndex:4] doubleValue]];
                [[LocationManager sharedInstance] setCenter:coordinate];
                
                FLog(@"getting location. background: %d", [[UIApplication sharedApplication] backgroundRefreshStatus]);
                [[LocationManager sharedInstance] startStandardUpdatesWithCompletionHandler:completionHandler];
                
            }else{
                
                FLog(@"Location Service problems. Enabled: %d Authorization: %d", [CLLocationManager locationServicesEnabled], [CLLocationManager authorizationStatus]);
                completionHandler(UIBackgroundFetchResultNoData);
            }
            
            
        } else if ([msgID rangeOfString:@"GEOLOC"].location != NSNotFound) {
            
            FLog(@"It is GEOLOC");
            [[LocationManager sharedInstance] setTypeOfGeoPush:0];
            
            NSArray *data = [msgID componentsSeparatedByString:@":"];
            
            if ([CLLocationManager locationServicesEnabled] && ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusAuthorized) ) {
                
                if (![LocationManager sharedInstance].message) {
                    [LocationManager sharedInstance].message = [[UnstoredIncomingMessage alloc] init];
                }
                
                [[LocationManager sharedInstance].message setMessageID:[data objectAtIndex:1]];
                
                CLLocation* coordinate = [[CLLocation alloc] initWithLatitude:[[data objectAtIndex:2] doubleValue] longitude:[[data objectAtIndex:3] doubleValue]];
                
                [[LocationManager sharedInstance] setRadius: [[data objectAtIndex:4] doubleValue]];
                [[LocationManager sharedInstance] setCenter:coordinate];
                
                FLog(@"getting location. background: %d", [[UIApplication sharedApplication] backgroundRefreshStatus]);
                [[LocationManager sharedInstance] startStandardUpdatesWithCompletionHandler:completionHandler];
                
            }else{
                
                FLog(@"Location Service problems. Enabled: %d Authorization: %d", [CLLocationManager locationServicesEnabled], [CLLocationManager authorizationStatus]);
                completionHandler(UIBackgroundFetchResultNoData);
            }
            
            
        } else {
            FLog(@"string does not contain GEOPUSH");
            
            SingleMessageLoader *incomingMessage = [[SingleMessageLoader alloc] init];
            incomingMessage.delegate = self;
            [incomingMessage startLoadingDataForMessageWithID:msgID];

            completionHandler(UIBackgroundFetchResultNoData);
        }
    }
}

/* Begin SDK Integration */

/*
 Called after retrieving a push notification from the server
*/

-(void) messageLoadFinishedReturningMessage:(UnstoredIncomingMessage *)message{
    if (message && message.messageID.length >0){
        [message writeMessageToFile];
                
        // TODO: add the message to the InboxTableViewController
    }
}

/* End SDK Integration */

- (IBAction)refreshButtonPressed:(id)sender {
    //[self refreshInbox];
    
    FLog(@"Refresh button pressed");
    
    /*Check if you are inside a monitored region*/
    
    /*UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Alert" message:[NSString stringWithFormat:@"Delegate: %@", [[LocationManager sharedInstance] delegate]] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
     
     [alert show];*/
    
    if (![[[LocationManager sharedInstance] delegate] isKindOfClass:[LocationManager class]]) {
        
        FLog(@"Refresh button: setting delegate");
        [[LocationManager sharedInstance] setDelegate:[LocationManager sharedInstance]];
        
        
    }
    
    /*alert = [[UIAlertView alloc] initWithTitle:@"Ahora" message:[NSString stringWithFormat:@"Delegate: %@", [[LocationManager sharedInstance] delegate]] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
     
     [alert show];*/
    
    for (CLCircularRegion *region in [[LocationManager sharedInstance] monitoredRegions]) {
        
        FLog(@"Checking region: %@ latidtude: %f longitude: %f radius: %f Enter: %d Exit: %d", [region identifier], region.center.latitude, region.center.longitude, [region radius], [region notifyOnEntry], [region notifyOnExit]);
        
        //[[LocationManager sharedInstance] requestStateForRegion:region];
    }
}


@end
