//
//  AppDelegate.m
//  Messangi
//
//  Created by Oscar Anzola on 5/23/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "AppDelegate.h"
#import "InboxTableViewController.h"
#import <MessangiSDK/MessangiSDK.h>
#import <AdSupport/AdSupport.h>
#import "GeoFenceTableViewController.h"


@implementation AppDelegate

NSInteger receivedNotifications;

- (void) clearNotifications
{
    receivedNotifications = 0;
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber: 0];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];
}

- (void) applicationDidBecomeActive:(UIApplication *)application
{
    [self clearNotifications];
    [[Messangi sharedInstance] getUnreadMessages];
    [[Messangi sharedInstance] logEvent:@"INIT" andExtraInfo:@""];
}

- (void) applicationDidEnterBackground:(UIApplication *)application
{
    [[Messangi sharedInstance] logEvent:@"PAUSE" andExtraInfo:@""];
    [self clearNotifications];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[ NSString stringWithFormat:@"%@-Defaults", [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleName"]] ofType:@"plist"]]];

    [self clearNotifications];

    /*
    if ([application respondsToSelector:@selector(registerUserNotificationSettings:)]) {
        UIUserNotificationType types = UIUserNotificationTypeBadge | UIUserNotificationTypeSound | UIUserNotificationTypeAlert;
        UIUserNotificationSettings *notificationSettings = [UIUserNotificationSettings settingsForTypes:types categories:nil];
        [[UIApplication sharedApplication] registerUserNotificationSettings:notificationSettings];
        if ([[[UIDevice currentDevice] systemVersion] floatValue] < 8.0){
            [[UIApplication sharedApplication] registerForRemoteNotifications];
        }
    } else {
        [[UIApplication sharedApplication] registerForRemoteNotificationTypes: (UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound | UIRemoteNotificationTypeAlert)];
    }
     */


    UIUserNotificationType types = UIUserNotificationTypeBadge | UIUserNotificationTypeSound | UIUserNotificationTypeAlert;
    UIUserNotificationSettings *notificationSettings = [UIUserNotificationSettings settingsForTypes:types categories:nil];
    [[UIApplication sharedApplication] registerUserNotificationSettings:notificationSettings];

    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

    // This is what determines if the authentication screen is shown or not
    _authenticated = [defaults objectForKey:@"clientPhoneNumber"] != nil;
    _authenticated = _authenticated && [defaults objectForKey:@"validated"] != nil;
    
    [[Messangi sharedInstance] setAppName: @"Messangi"];
    [[Messangi sharedInstance] setClientID: @"D79BFFVx2TjXcJrrnktB"];
    [[Messangi sharedInstance] setApiClientPrivateKey: @"wzuuzVeVA8gf9aOOOxMC"];
    
    [[Messangi sharedInstance] setSubscriptionInstanceID:@"8"];
    [[Messangi sharedInstance] setSubscriptionURL:@"http://www.messangi.com/messangi_mmc/rest/api"];


    // Check to see if user is authenticated
    if (_authenticated) {

        // Set the root view controller to the tab bar controller
        [[Messangi sharedInstance] initSubscriptions];
        [[Messangi sharedInstance] initMessangi:true];

        self.window.rootViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]] instantiateInitialViewController];
    } else {
        // User isn't authenticated so load authentication controller
        UIViewController* rootController = [[UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]] instantiateViewControllerWithIdentifier:@"AuthViewController"];
        UINavigationController* navigation = [[UINavigationController alloc] initWithRootViewController:rootController];
        self.window.rootViewController = navigation;
    }
    return YES;
}

-(void)application:(UIApplication *)application didRegisterUserNotificationSettings:(UIUserNotificationSettings *)notificationSettings // available in iOS8
{
    [application registerForRemoteNotifications];
}
							
- (void)application:(UIApplication*)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    [[Messangi sharedInstance] registerDeviceToken:deviceToken];
}

- (void)application:(UIApplication*)application didFailToRegisterForRemoteNotificationsWithError:(NSError*)error
{
    NSString* logMessage = [NSString stringWithFormat:@"didFailToRegisterForRemoteNotificationsWithError: %@",[error debugDescription]];
    [[Messangi sharedInstance] logEvent:@"ERROR" andExtraInfo:logMessage];

    NSString *title = NSLocalizedString(@"Notification Service", nil);
    NSString *message = NSLocalizedString(@"Push notifications not available.", nil);
    NSString *cancelButtonTitle = NSLocalizedString(@"OK", nil);
    
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    
    // Create the action.
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelButtonTitle style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSLog(@"The simple alert's cancel action occured.");
    }];
    
    // Add the action.
    [alertController addAction:cancelAction];
    
    UIWindow *window = [UIApplication sharedApplication].keyWindow;
    UIViewController *vc = [window rootViewController];
    // Present Alert
    [vc presentViewController:alertController animated:YES completion:nil];


    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:nil forKey:@"devicePushID"];
    [defaults synchronize];
}

/*- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler
{
    NSString *msgID = [NSString stringWithFormat:@"%@", [userInfo objectForKey:@"msgId"]];
    NSString *msgType = [NSString stringWithFormat:@"%@", [userInfo objectForKey:@"type"]];
    
    if ([msgType isEqualToString:@"GET_LOC"]) {
        // GeoLocation
        
        [[Messangi sharedInstance] sendLastLocationForMsgID:msgID withHandler:completionHandler];
        
    } else if ([msgType isEqualToString:@"CRT_REGION"] || [msgType isEqualToString:@"UPDT_REGION"]) {
        
        // PUSH TYPE: CRT_REGION || PUSH TYPE: UPDT_REGION
        // PUSH CONTENT: REGION_ID:LATITUDE:LONGITUDE:RADIUS:TRIGGER
        
        [[Messangi sharedInstance] registerRegionForMsgID:msgID];
        completionHandler(UIBackgroundFetchResultNewData);
            
    } else if ([msgType isEqualToString:@"DLT_REGION"]) {
        // PUSH TYPE: DLT_REGION
        // PUSH CONTENT: REGION_ID
        
        [[Messangi sharedInstance] unregisterRegionWithMsgID:msgID];
        
        completionHandler(UIBackgroundFetchResultNewData);
        
    } else if ([msgType isEqualToString:@"TXT_MSG"]) {
        if ([msgID rangeOfString:@"GEOPUSH"].location != NSNotFound) {
            
          // [[LocationManager sharedInstance] setTypeOfGeoPush:1];
            
            if ([[Messangi sharedInstance] statusLocationServicesOK]) {
                
                [[Messangi sharedInstance] processGeoPushForMsgID:msgID withHandler:completionHandler];
                
             } else {
                NSString *title = NSLocalizedString(@"Location Services", nil);
                NSString *message = NSLocalizedString(@"Location not enabled or authorized", nil);
                NSString *cancelButtonTitle = NSLocalizedString(@"OK", nil);
                
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:nil];
                [alert show];
                completionHandler(UIBackgroundFetchResultNoData);
            }
        } else {
            //[[Messangi sharedInstance] processPushNotificationForMsgID:msgID withHandler:completionHandler];
            [[Messangi sharedInstance] getUnreadMessages];
         }
    }
}*/

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler
{
    if ([[Messangi sharedInstance] didReceiveRemoteNotification:userInfo fetchCompletionHandler:completionHandler]) {
        /*
         * Notification was managed by Messangi
         * Add code here to perform any addionat task after Messangi handling the notification
         */
    } else {
        /*
         * Notification received was not processed by Messangi
         * This notification must be managed by host App here
         */
    }
}

/* Begin SDK Integration */

/*
 Called after retrieving a push notification from the server
*/

-(void) messageLoadFinishedReturningMessage:(UnstoredIncomingMessage *)message{

    //Obligatorio para BigData
    [[Messangi sharedInstance] logEvent:@"PUSHRECEIVED" andExtraInfo:message.text withCampaignName:message.subject];

    //Opcional para DEBUG
    NSString* logMessage = [NSString stringWithFormat:@"ID: %@, TITLE: %@, BODY: %@",message.messageID,message.subject,message.text];
    [[Messangi sharedInstance] logEvent:@"MESSAGE" andExtraInfo:logMessage];

    if (message && message.messageID.length > 0){


        UILocalNotification *localNotification = [[UILocalNotification alloc] init];

        // current time plus 10 secs
        NSDate *now = [NSDate date];
        NSDate *dateToFire = [now dateByAddingTimeInterval:1];

        NSLog(@"now time: %@", now);
        NSLog(@"fire time: %@", dateToFire);

        localNotification.fireDate = dateToFire;
        localNotification.alertBody = [self convertHTML: message.text];
        localNotification.soundName = UILocalNotificationDefaultSoundName;
        localNotification.applicationIconBadgeNumber = 1; // increment

        [[UIApplication sharedApplication] scheduleLocalNotification:localNotification];


        [message writeMessageToFile];
        [[UIApplication sharedApplication] setApplicationIconBadgeNumber: ++receivedNotifications];
      
        UITabBarController *tabBar = (UITabBarController *)self.window.rootViewController;
        UINavigationController *navCon0 = [[tabBar viewControllers] objectAtIndex:0];
        InboxTableViewController *inboxView = (InboxTableViewController *) [[navCon0 viewControllers] objectAtIndex:0];
        [inboxView refreshMessangiInboxView];
    } else {
        [[UIApplication sharedApplication] setApplicationIconBadgeNumber: ++receivedNotifications];
    }
}

/* End SDK Integration */

- (NSString *)convertHTML:(NSString *)html {

    NSScanner *myScanner;
    NSString *text = nil;
    myScanner = [NSScanner scannerWithString:html];

    while ([myScanner isAtEnd] == NO) {

        [myScanner scanUpToString:@"<" intoString:NULL] ;

        [myScanner scanUpToString:@">" intoString:&text] ;

        html = [html stringByReplacingOccurrencesOfString:[NSString stringWithFormat:@"%@>", text] withString:@""];
    }
    //
    html = [html stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    
    return html;
}

@end
