//
//  AuthenticationViewController.m
//  Messangi
//
//  Created by Oscar Anzola on 5/23/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "AuthenticationViewController.h"
#import "AppDelegate.h"
#import <MessangiSDK/MessangiSDK.h>

@interface AuthenticationViewController ()

@property(nonatomic,copy)NSString *codeEntered;

@end

@implementation AuthenticationViewController

-(void) RegisterPhoneSucceeded:(BOOL)success withMessage:(NSString *)message{
    if (success) {
        //OJO: Validacion de codigo SMS
        self.status = REQUESTING_CODE;
    }
}
- (IBAction)cancelButtonPressed:(id)sender {
    _countryCodeTextField.text = @"";
    _areaCodeTextField.text = @"";
    _numberTextField.text = @"";
    AppDelegate *appDelegateTemp = (AppDelegate *)[[UIApplication sharedApplication]delegate];
    appDelegateTemp.window.rootViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]] instantiateInitialViewController];
}

-(void)RegisterPhoneReturnedError:(NSString *)error
{
    NSString *title = NSLocalizedString(@"Registration", nil);
    NSString *message = NSLocalizedString(@"Mobile number registration failed", nil);
    NSString *cancelButtonTitle = NSLocalizedString(@"OK", nil);
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    
    // Create the action.
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelButtonTitle style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSLog(@"The simple alert's cancel action occured.");
    }];
    
    // Add the action.
    [alertController addAction:cancelAction];
    
    [self presentViewController:alertController animated:YES completion:nil];

}

-(void)ActivatePhoneSucceeded:(BOOL)success withMessage:(NSString *)message
{
    if (success) {
        [[Messangi sharedInstance] initMessangi:true];
        [[Messangi sharedInstance] initSubscriptions];
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        [defaults setBool:NO forKey:@"isFirstTime"];
        [defaults setObject:@"YES" forKey:@"validated"];
        [defaults synchronize];
        dispatch_async(dispatch_get_main_queue(), ^{
            AppDelegate *appDelegateTemp = (AppDelegate *)[[UIApplication sharedApplication]delegate];
            appDelegateTemp.window.rootViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]] instantiateInitialViewController];
        });
    }
}

-(void)ActivatePhoneReturnedError:(NSString *)error
{
    NSString *title = NSLocalizedString(@"Activation", nil);
    NSString *message = NSLocalizedString(@"Mobile number activation failed", nil);
    NSString *cancelButtonTitle = NSLocalizedString(@"OK", nil);
    
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    
    // Create the action.
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelButtonTitle style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSLog(@"The simple alert's cancel action occured.");
    }];
    
    // Add the action.
    [alertController addAction:cancelAction];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

- (IBAction)sendValidationCodeButton:(id)sender {
    if ([_countryCodeTextField.text length] && [_areaCodeTextField.text length] && [_numberTextField.text length]) {
        RegisterPhoneDataloader *rpdl = [[RegisterPhoneDataloader alloc]init];
        rpdl.delegate = self;
        [rpdl registerDeviceWithAreaCode:[NSString stringWithFormat:@"+%@%@", _countryCodeTextField.text, _areaCodeTextField.text] andPhoneNumber:_numberTextField.text];
        
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        
        [defaults setObject:[NSString stringWithFormat:@"+%@%@%@", _countryCodeTextField.text, _areaCodeTextField.text,_numberTextField.text] forKey:@"clientPhoneNumber"];
        [defaults synchronize];
        [self resignFirstResponder];
        [self showValidationCodeEntryAlert];
    } else {
        UIAlertController *errorAlert = [UIAlertController alertControllerWithTitle:@"Error" message:NSLocalizedString(@"FillInAllThreeFieldsError", @"Error that shows up when the user has not filled in country, area, and phone number") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"Ok" style:UIAlertActionStyleCancel handler:nil];
        [errorAlert addAction:okAction];
        [self presentViewController:errorAlert animated:YES completion:nil];
    }
}

- (IBAction)cancelCodeValidation:(UIStoryboardSegue *)segue {
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    return YES;
}

-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    [_countryCodeTextField becomeFirstResponder];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationController.navigationBarHidden = YES;
    _enterMobilePhoneNumberLabel.text = NSLocalizedString(@"NumberEntry", @"Prompts user to enter mobile number:");
    _countryCodeLabel.text = NSLocalizedString(@"Country", @"Country");
    _areaCodeLabel.text = NSLocalizedString(@"Area", @"Area");
    _phoneNumberLabel.text = NSLocalizedString(@"Number", @"Phone Number");
    [_sendValidationCodeButton setTitle:NSLocalizedString(@"SendValidationButton", @"Sends user validation code") forState:UIControlStateNormal];
    [_sendValidationCodeButton.titleLabel setTextAlignment: NSTextAlignmentCenter];
    [_cancelMobileNumberValidationButton setTitle:NSLocalizedString(@"CancelButton", @"A cancel button") forState:UIControlStateNormal];
    [_cancelMobileNumberValidationButton.titleLabel setTextAlignment: NSTextAlignmentCenter];
    _sendValidationCodeButton.enabled = YES;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
 -(void)hideKeyboard
 {
 [self.view endEditing:YES];
 _tap.enabled = NO;
 }
 @property (strong, nonatomic) UITapGestureRecognizer *tap;
 _tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideKeyboard)];
 _tap.enabled = NO;
 [self.view addGestureRecognizer:_tap];
 
*/

- (void)showValidationCodeEntryAlert {
//    NSString *title = NSLocalizedString(@"ValidationButton", @"Validate this code");
//    NSString *message = NSLocalizedString(@"CodeEntryPrompt", @"Prompts user to enter their validation code");
//    NSString *cancelButtonTitle = NSLocalizedString(@"Cancel", @"A cancel button");
//    NSString *otherButtonTitle = NSLocalizedString(@"OK", nil);
//    
//    
//    
//	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:otherButtonTitle, nil];
//    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
//    [[alert textFieldAtIndex:0] setKeyboardType:UIKeyboardTypeNumberPad];
//    [[alert textFieldAtIndex:0] setDelegate:self];
//    [alert textFieldAtIndex:0].tag = 1;
//    
//    [alert show];
    NSString *title = NSLocalizedString(@"ValidationButton", @"Validate this code");
    NSString *message = NSLocalizedString(@"CodeEntryPrompt", @"Prompts user to enter their validation code");
    NSString *cancelButtonTitle = NSLocalizedString(@"Cancel", @"A cancel button");
    NSString *otherButtonTitle = NSLocalizedString(@"OK", nil);
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    
    // Add the text field for text entry.
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField) {
        // If you need to customize the text field, you can do so here.
        [textField setKeyboardType:UIKeyboardTypeNumberPad];
        [textField setDelegate:self];
    }];
    
    // Create the actions.
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelButtonTitle style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
    }];
    
    UIAlertAction *otherAction = [UIAlertAction actionWithTitle:otherButtonTitle style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        RegisterPhoneDataloader *rpdl = [[RegisterPhoneDataloader alloc]init];
        rpdl.delegate = self;
        [rpdl activatePhoneWithCode:((UITextField *)alertController.textFields[0]).text];
    }];
    
    // Add the actions.
    [alertController addAction:cancelAction];
    [alertController addAction:otherAction];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    if (textField.tag == 1) {
        BOOL result = YES;
        // restrict the maximum number of characters to 5
        if (textField.text.length == 5 && string.length > 0)
            result = NO;
        
        return result;
    }
    else
        return YES;
}

@end
