//
//  BizViewController.m
//  Messangi
//
//  Created by Christopher Choitz on 8/18/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "BizViewController.h"
#import "BizListTableViewController.h"

#define BUSINESSESLISTFILE    @"list.plist"
#define BUSINESSESLISTDIRECTORY    @"https://api.messangi.com/messangi-staging/content/workspaces"

@interface BizViewController ()  <UITextFieldDelegate>

@property (nonatomic) UITapGestureRecognizer *tap;

@end

@implementation BizViewController

-(void) selectNewTemplate:(NSString *)bizName
{
    Business *newBiz = [[Business alloc] initWithNewSelection:[bizName stringByAppendingPathExtension:@"plist"]];
    
    if (newBiz) {
        _currentBusiness = newBiz;
        [_currentBusiness writeCurrentBusinessToFile];
        [self refreshView];
    } else {
//        UIAlertView * errorAlert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Error", nil)
//                                                              message:NSLocalizedString(@"Business template not found.", nil)
//                                                             delegate:self
//                                                    cancelButtonTitle:NSLocalizedString(@"OK", nil)
//                                                    otherButtonTitles:nil];
//        [errorAlert show];
        
        UIAlertController *errorAlert = [UIAlertController alertControllerWithTitle:@"Error" message:@"Business template not found." preferredStyle:UIAlertControllerStyleAlert];
        
        UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"Ok" style:UIAlertActionStyleCancel handler:nil];
        [errorAlert addAction:okAction];
        
        [self presentViewController:errorAlert animated:YES completion:nil];
    }
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    [self selectNewTemplate:textField.text];
    return YES;
}

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    _tap.enabled = YES;
    return YES;
}

-(void)hideKeyboard
{
    [self.view endEditing:YES];
    _tap.enabled = NO;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    _tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideKeyboard)];
    _tap.enabled = NO;
    _nameLabel.adjustsFontSizeToFitWidth = YES;

#ifdef DEBUG
    NSLog(@"BizViewController: viewDidLoad: Business Directory: %@",[Business businessDirectory]);
#endif
     [self fetchBusinessesFromWebFileURL:BUSINESSESLISTFILE fromDirectoryURL:[NSURL URLWithString:BUSINESSESLISTDIRECTORY]];
    
    if ([Business businessSaved]) {
        _currentBusiness = [[Business alloc] initWithFile];
    } else {
        _currentBusiness = [[Business alloc] initDefault];
    }
}

-(void)refreshView
{
    _nameLabel.text = _currentBusiness.businessName;
    _logoImageView.image = _currentBusiness.logo;
    [_descriptionWebView loadHTMLString:_currentBusiness.businessDescription baseURL:nil];
    _bizTemplateTextField.text = @"";
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    [self refreshView];
    //_tabsArray = _currentBusiness.tabs;
    
    // Check if any tabs need to be disabled...
    
 /*
    for (unsigned int i=0; i<4; i++) {
//        tabItem = _currentBusiness.tabs[i];
        [[[[self.tabBarController tabBar] items] objectAtIndex:3] setEnabled:YES];
    }
*/
    self.tabBarController.tabBar.hidden = NO;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(IBAction)returnFromSelection:(UIStoryboardSegue *)unwindSegue
{
    BizListTableViewController *bltc = [unwindSegue sourceViewController];
    if (bltc.currentFileNameSelected.length) {
        _currentBusiness = [[Business alloc] initWithNewSelection:bltc.currentFileNameSelected];
        [_currentBusiness writeCurrentBusinessToFile];
    }
}

-(void)fetchBusinessesFromWebFileURL:(NSString *)businessListFileName fromDirectoryURL:(NSURL *)serverDirectoryURL
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSURL *serverBusinessListURL = [serverDirectoryURL URLByAppendingPathComponent:businessListFileName isDirectory:NO];
        NSString *localBusinessDirectory = [Business businessDirectory];
        
        NSArray *businessDownloadList = [NSArray arrayWithContentsOfURL:serverBusinessListURL];
        
        NSFileManager *fm = [NSFileManager defaultManager];
        for (NSString *businessFile in businessDownloadList) {
            NSURL *serverFileURL = [serverDirectoryURL URLByAppendingPathComponent:businessFile isDirectory:NO];
            NSString *localFilePath = [localBusinessDirectory stringByAppendingPathComponent:businessFile];
            if (![fm fileExistsAtPath:localFilePath]) {
                NSMutableDictionary *businessDict = [NSMutableDictionary dictionaryWithContentsOfURL:serverFileURL];
                [businessDict setObject:businessFile forKey:@"Filename"];
                [businessDict writeToFile:localFilePath atomically:YES];
            }
        }
        
    });
}




@end
