//
//  Business.m
//  Messangi
//
//  Created by Christopher Choitz on 8/19/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "Business.h"
#import <MessangiSDK/MessangiSDK.h>

@implementation Business



// Used when a current exists from previously saved
-(instancetype)initWithFile
{
    self = [super init];
    if (self) {
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        NSString *dir = [paths objectAtIndex:0];
        NSString *filepath = [dir stringByAppendingPathComponent:@"Current.plist"];
    
        if ([[NSFileManager defaultManager] fileExistsAtPath:filepath]) {
            NSMutableDictionary *dict = [[NSMutableDictionary alloc] initWithContentsOfFile:filepath];
            _businessName = dict[@"Name"];
            _businessDescription = dict[@"Description"];
            _logo = [UIImage imageWithData:[dict objectForKey:@"Logo"]];
            _tabs = dict[@"Tabs"];
            _filename = dict[@"Filename"];
            _subscriptionID = dict[@"ID"];
            [[Messangi sharedInstance] setSubscriptionInstanceID:_subscriptionID];
            [[Messangi sharedInstance] initSubscriptions];
            return self;
        }
    }
    return nil;
}

-(instancetype)initDefault
{
    self = [super init];
    if (self) {
        _businessName = @"Messangi";
        _businessDescription = @"<strong>Say Hello to Onmichannel</strong></br>Looking for innovative ways to communicate with customers? A mobile marketing solution that is:<li>Location Aware</li><li>Omnichannel</li><li>Cloud Based</li><li>ERP Compatible</li><li>Big Data Ready</li><li>Campaign Rich</br>Use the Web based <i>Messangi Campaign Manager</i> to create and manage campaigns with a few clicks. Login to your account and start creating amazing interactive campaigns and services. The <i>Messangi REST API</i> connects to existing customer ERP/CRM systems. Use Messangi as the omnichannel messaging broker for your platform. We’ll call your platform back with status and results. IOS SDK easily added to any Mobile App and seamlessly integrated with our Campaign Manager. Focus on your app and let the <i>Messangi Mobile SDK</i> handle the complexities behind indoor and outdoor Location Aware Communication, Push Notifications and Subscriptions.";
        _logo = [UIImage imageNamed:@"LS-Logo"];
        _tabs = @[@YES, @YES, @YES, @YES];
        _filename = @"";
        [[Messangi sharedInstance] setSubscriptionInstanceID:@"8"];
        [[Messangi sharedInstance] initSubscriptions];
        return self;
    }
    return nil;
}

// Used when selecting a new one
-(instancetype)initWithNewSelection:(NSString *)fileName
{
    self = [super init];
    if (self) {
        NSString *bizDirectory = [Business businessDirectory];
        NSString *filepath = [bizDirectory stringByAppendingPathComponent:fileName];
        
        if ([[NSFileManager defaultManager] fileExistsAtPath:filepath]) {
            NSMutableDictionary *dict = [[NSMutableDictionary alloc] initWithContentsOfFile:filepath];
            _businessName = dict[@"Name"];
            _businessDescription = dict[@"Description"];
            NSData *imageData = [[NSData alloc] initWithContentsOfURL:[NSURL URLWithString:[dict objectForKey:@"Logo"]]];
            _logo = [UIImage imageWithData: imageData];
            _tabs = dict[@"Tabs"];
            _filename = dict[@"Filename"];
            _subscriptionID = dict[@"ID"];
            if (!_subscriptionID) {
                _subscriptionID = @"8";
            }
            [[Messangi sharedInstance] setSubscriptionInstanceID:_subscriptionID];
            [[Messangi sharedInstance] initSubscriptions];
            return self;
        }
    }
    return nil;
}

+(NSString *)businessDirectory
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *dir = [[paths objectAtIndex:0] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@",DIRECTORY_BUSINESS]];
    if (![[NSFileManager defaultManager] fileExistsAtPath:dir]) {
        [[NSFileManager defaultManager] createDirectoryAtPath:dir withIntermediateDirectories:NO attributes:nil error:nil]; //Create folder
    }
    return dir;
}

- (void)writeCurrentBusinessToFile
{
    NSDictionary *business = [self businessToDictonaryEntry];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *dir = [paths objectAtIndex:0];
    NSString *businessFile = [dir stringByAppendingPathComponent:@"Current.plist"];
    [business writeToFile:businessFile atomically:YES];
}

- (NSMutableDictionary *)businessToDictonaryEntry
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    dict[@"Name"] = _businessName;
    dict[@"Logo"] = [NSData dataWithData:UIImageJPEGRepresentation(_logo, 1)];
    dict[@"Description"] = _businessDescription;
    dict[@"Tabs"] = _tabs;
    dict[@"Filename"] = _filename;
    dict[@"ID"] = _subscriptionID;
    return dict;
}

+ (BOOL)businessSaved
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *dir = [[paths objectAtIndex:0] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@", DIRECTORY_CURRENT]];
    NSString *businessFile = [dir stringByAppendingPathComponent:@"Current.plist"];
    if ([[NSFileManager defaultManager] fileExistsAtPath:businessFile]) {
        return YES;
    }
    return NO;
}

@end
