//
//  GeoFenceTableViewController.m
//  Messangi
//
//  Created by Oscar Anzola on 5/23/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "GeoFenceTableViewController.h"
#import "GeoFenceViewController.h"
#import <MessangiSDK/MessangiSDK.h>
#import <CoreLocation/CoreLocation.h>

@implementation GeoFenceTableViewController

- (void)refreshView:(UIRefreshControl *)sender {
    [[Messangi sharedInstance] refreshMonitoredRegionsToClosest];
    [self.tableView reloadData];
    [sender endRefreshing];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    
    self.tabBarController.tabBar.hidden = NO;

    [self.tableView reloadData];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _screenTitleGeoFences.title = NSLocalizedString(@"Monitoring", @"Instances of the word GeoFences");
    
    [self.refreshControl addTarget:self action:@selector(refreshView:) forControlEvents:UIControlEventValueChanged];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 2;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    NSString *title;
    NSString *formatString;
    
    switch(section)
    {
        case 0:
            formatString = NSLocalizedString(@"iBeacons (%i)", @"Format string for iBeacon header title.");
            title = [NSString stringWithFormat:formatString, [Messangi sharedInstance].beaconsList.count];
            break;
        case 1:
            formatString = NSLocalizedString(@"GeoFences (%i)", @"Format string for GeoFences header title.");
            title = [NSString stringWithFormat:formatString, [Messangi sharedInstance].geoFencesList.count];
            break;
        default:
            title = @"";
            break;
    }
    return title;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    switch(section)
    {
        case 0:
            return [Messangi sharedInstance].beaconsList.count;
            break;
        case 1:
            return [Messangi sharedInstance].geoFencesList.count;
            break;
        default:
            return 0;
            break;
    }
}

- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell* cell = [tableView cellForRowAtIndexPath:indexPath];
    if(cell.selectionStyle == UITableViewCellSelectionStyleNone){
        return nil;
    }
    return indexPath;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"GeoFenceCell" forIndexPath:indexPath];
    
  
    if (indexPath.section == 0) {
        CLBeaconRegion *beaconItem = [Messangi sharedInstance].beaconsList[indexPath.row][@"Region"];
        if ([[Messangi sharedInstance].messangiLM.monitoredRegions containsObject:beaconItem]) {
            cell.imageView.image = [UIImage imageNamed:@"GreenCircle14"];
        } else {
            cell.imageView.image = [UIImage imageNamed:@"RedCircle14"];
        }
        cell.textLabel.adjustsFontSizeToFitWidth = YES;
        cell.textLabel.text = [Messangi sharedInstance].beaconsList[indexPath.row][@"Name"];
       
        NSString *formatString = NSLocalizedString(@"%@\nMajor: %@, Minor: %@", @"Format string for ranging table cells.");
        cell.detailTextLabel.numberOfLines = 2;
        cell.detailTextLabel.text = [NSString stringWithFormat:formatString, [beaconItem.proximityUUID UUIDString],[beaconItem.major stringValue], [beaconItem.minor stringValue]];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.accessoryType = UITableViewCellAccessoryNone;
    } else {
        CLCircularRegion *geoFenceItem = [Messangi sharedInstance].geoFencesList[indexPath.row][@"Region"];
        CLLocation *geoFenceLocation = [[CLLocation alloc] initWithLatitude:geoFenceItem.center.latitude  longitude:geoFenceItem.center.longitude];
        CLLocation *currentLocation = [Messangi sharedInstance].messangiLastLocation;
        
        CLLocationDistance distance = [geoFenceLocation distanceFromLocation:currentLocation];
        
        if ([[Messangi sharedInstance].messangiLM.monitoredRegions containsObject:geoFenceItem]) {
            cell.imageView.image = [UIImage imageNamed:@"GreenCircle14"];
        } else {
            cell.imageView.image = [UIImage imageNamed:@"RedCircle14"];
        }
        
        cell.textLabel.text = [Messangi sharedInstance].geoFencesList[indexPath.row][@"Name"];
        
        NSNumberFormatter *formatter = [[NSNumberFormatter alloc] init];
        NSString *groupingSeparator = [[NSLocale currentLocale] objectForKey:NSLocaleGroupingSeparator];
        [formatter setGroupingSeparator:groupingSeparator];
        [formatter setGroupingSize:3];
        [formatter setAlwaysShowsDecimalSeparator:NO];
        [formatter setUsesGroupingSeparator:YES];
        
        
        NSString *formattedDistance = [formatter stringFromNumber:[NSNumber numberWithDouble:distance]];
        
        NSString *dist = [NSString stringWithFormat:@"Distance: %@ m", formattedDistance];
        NSMutableAttributedString *attrString = [[NSMutableAttributedString alloc] initWithString:dist];
        NSRange rangeofDist = [attrString.string rangeOfString:@": "];
        rangeofDist.location = NSMaxRange(rangeofDist);
        rangeofDist.length = attrString.string.length - rangeofDist.location;
        [attrString addAttribute:NSFontAttributeName value:[UIFont boldSystemFontOfSize:12] range:rangeofDist];
        cell.detailTextLabel.numberOfLines = 1;
        cell.detailTextLabel.attributedText = attrString;
        cell.selectionStyle = UITableViewCellSelectionStyleBlue;
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }
    return cell;
}

#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    GeoFenceViewController *mvc = [segue destinationViewController];

    if ([[segue identifier] isEqualToString:@"GeoFenceDetail"]) {
        NSIndexPath *index = [self.tableView indexPathForSelectedRow];
        mvc.circularRegion = [Messangi sharedInstance].geoFencesList[index.row];
        mvc.geoFence = YES;
    } else if ([[segue identifier] isEqualToString:@"MyLocation"])  {
        mvc.geoFence = NO;
    }
}

@end
