//
//  GeoFenceViewController.m
//  Messangi
//
//  Created by Oscar Anzola on 5/23/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "GeoFenceViewController.h"
#import <MessangiSDK/MessangiSDK.h>

@interface GeoFenceViewController () <MKMapViewDelegate>

@property (nonatomic, strong) NSMutableArray *geoFenceList;

@end

@implementation GeoFenceViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _geoFenceMapLocation.delegate = self;
    
    CLCircularRegion *circularRegion;
    
    [_geoFenceMapLocation setCenterCoordinate:circularRegion.center animated:YES];

    if (_geoFence) {
        circularRegion = _circularRegion[@"Region"];
        self.title = _circularRegion[@"Name"];
        
       _geoFenceMapLocation.showsUserLocation = YES;
        MKPointAnnotation *point = [[MKPointAnnotation alloc] init];
        point.coordinate = circularRegion.center;
        point.title = _circularRegion[@"Name"];
        [_geoFenceMapLocation addAnnotation:point];
        [_geoFenceMapLocation selectAnnotation:point animated:YES];
        MKCircle *circle = [MKCircle circleWithCenterCoordinate:circularRegion.center radius:circularRegion.radius];
        [_geoFenceMapLocation addOverlay:circle];
    } else {
        _geoFenceMapLocation.showsUserLocation = NO;
        MKPointAnnotation *point = [[MKPointAnnotation alloc] init];
        circularRegion = [[CLCircularRegion alloc] initWithCenter:[Messangi sharedInstance].messangiLastLocation.coordinate radius:100.0 identifier:@"MyLocation"];
        self.title = _circularRegion[@"My Location"];

        point.coordinate = circularRegion.center;
        point.title = @"Your Location";
        [_geoFenceMapLocation addAnnotation:point];
        [_geoFenceMapLocation selectAnnotation:point animated:YES];
        CLCircularRegion *geoFenceRegion;
        for (NSDictionary *geoFenceDict in [Messangi sharedInstance].geoFencesList) {
            geoFenceRegion = geoFenceDict[@"Region"];
            MKPointAnnotation *point = [[MKPointAnnotation alloc] init];
            point.coordinate = geoFenceRegion.center;
            [_geoFenceMapLocation addAnnotation:point];
            MKCircle *circle = [MKCircle circleWithCenterCoordinate:geoFenceRegion.center radius:geoFenceRegion.radius];
            [_geoFenceMapLocation addOverlay:circle];
        }
    }
    MKCoordinateRegion region;
    region.center = circularRegion.center;
    MKCoordinateSpan span;
    span.latitudeDelta  = circularRegion.radius * .0001;
    span.longitudeDelta = circularRegion.radius * .0001;
    region.span = span;
    [_geoFenceMapLocation setRegion:region animated:YES];
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    self.tabBarController.tabBar.hidden = YES;
}

- (MKOverlayView *)mapView:(MKMapView *)mapView viewForOverlay:(id<MKOverlay>)overlay
{
    MKCircleView *circleView = [[MKCircleView alloc] initWithOverlay:overlay];
    [circleView setFillColor:[UIColor blueColor]];
    [circleView setStrokeColor:[UIColor whiteColor]];
    [circleView setAlpha:0.5f];
    return circleView;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
