//
//  InboxTableViewController.m
//  Messangi
//
//  Created by Oscar Anzola on 5/23/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "InboxTableViewController.h"
#import "UnstoredIncomingMessage.h"
#import "MessageViewController.h"
#import "Messangi.h"

@interface InboxTableViewController () <UIActionSheetDelegate>

@property (nonatomic, strong) NSMutableArray *messageList;
@property (nonatomic, copy) NSString *inboxDirectory;

@property (weak, nonatomic) IBOutlet UIBarButtonItem *refreshButton;
@property (strong, nonatomic) IBOutlet UIBarButtonItem *editButton;
@property (strong, nonatomic) IBOutlet UIBarButtonItem *cancelButton;
@property (strong, nonatomic) IBOutlet UIBarButtonItem *deleteButton;



@end

@implementation InboxTableViewController

- (IBAction)refreshButtonPressed:(id)sender {
    _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
    [self.tableView reloadData];
}

- (void)refresh {
    _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
    [self.tableView reloadData];
}

-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
    [self.tableView reloadData];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [[Messangi sharedInstance] initMessangi];
    
#warning This is to test the init and write funcion
    if (!_messageList) {
        NSDate *today = [NSDate date];
        
        UnstoredIncomingMessage *message1 = [[UnstoredIncomingMessage alloc] initWithMessageID:@"Messangi10" withSubject:@"Hello" withReceivedDate:today];
        [message1 writeMessageToFile];
        UnstoredIncomingMessage *message2 = [[UnstoredIncomingMessage alloc] initWithMessageID:@"Messangi20" withSubject:@"Goodbye" withReceivedDate:today];
        [message2 writeMessageToFile];
        UnstoredIncomingMessage *message3 = [[UnstoredIncomingMessage alloc] initWithMessageID:@"Messangi30" withSubject:@"Hello Goodbye" withReceivedDate:today];
        [message3 writeMessageToFile];
        
        _inboxDirectory = [UnstoredIncomingMessage documentsInboxPath];
        _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
         
    }

    // Load the buttons
    self.navigationItem.rightBarButtonItem = self.editButtonItem;

    self.tableView.allowsMultipleSelectionDuringEditing = YES;
    
    
    
    [self updateButtonsToMatchTableState];
    
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Update the delete button's title based on how many items are selected.
    [self updateDeleteButtonTitle];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [self updateButtonsToMatchTableState];
}

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
	// The user tapped one of the OK/Cancel buttons.
	if (buttonIndex == 0)
	{
		// Delete what the user selected.
        NSArray *selectedRows = [self.tableView indexPathsForSelectedRows];
        BOOL deleteSpecificRows = selectedRows.count > 0;
        if (deleteSpecificRows)
        {
            // Build an NSIndexSet of all the objects to delete, so they can all be removed at once.
            NSMutableIndexSet *indicesOfItemsToDelete = [NSMutableIndexSet new];
            for (NSIndexPath *selectionIndex in selectedRows)
            {
                [indicesOfItemsToDelete addIndex:selectionIndex.row];
                [UnstoredIncomingMessage deleteMessageFileName:[self fileNameWithExtensionFromURL:_messageList[selectionIndex.row]]];
            }
            // Delete the objects from our data model.
            [self.messageList removeObjectsAtIndexes:indicesOfItemsToDelete];
            
            // Tell the tableView that we deleted the objects
            [self.tableView deleteRowsAtIndexPaths:selectedRows withRowAnimation:UITableViewRowAnimationAutomatic];
        }
        // Exit editing mode after the deletion.
        [self.tableView setEditing:NO animated:YES];
        [self updateButtonsToMatchTableState];
	}
}

- (NSArray *)listFilesAtPath:(NSString *)path
{
    NSURL *urlFilePath = [NSURL fileURLWithPath:path];
    NSArray *directoryContent = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:urlFilePath includingPropertiesForKeys:NULL options:NSDirectoryEnumerationSkipsHiddenFiles error:NULL];
    return directoryContent;
}

- (IBAction)editAction:(id)sender
{
    [self.tableView setEditing:YES animated:YES];
    
    [self updateButtonsToMatchTableState];
}


- (IBAction)cancelAction:(id)sender
{
    [self.tableView setEditing:NO animated:YES];
    
    [self updateButtonsToMatchTableState];
}

- (IBAction)deleteAction:(id)sender
{
    // Open a dialog with just an OK button.
	NSString *actionTitle;
    if (([[self.tableView indexPathsForSelectedRows] count] == 1)) {
        actionTitle = NSLocalizedString(@"Are you sure you want to delete this message?", @"");
    }
    else
    {
        actionTitle = NSLocalizedString(@"Are you sure you want to delete these messages?", @"");
    }
    
    NSString *cancelTitle = NSLocalizedString(@"Cancel", @"Cancel title for message removal action");
    NSString *okTitle = NSLocalizedString(@"OK", @"OK title for message removal action");
    UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:actionTitle delegate:self cancelButtonTitle:cancelTitle destructiveButtonTitle:okTitle otherButtonTitles:nil];
    
	actionSheet.actionSheetStyle = UIActionSheetStyleDefault;
    // Show from our table view (pops up in the middle of the table).
	[actionSheet showInView:self.view];
}

-(void)deleteMessageFile:(NSURL *)file
{
    
    BOOL success = [[NSFileManager defaultManager] removeItemAtURL:file error:NULL];
    if (!success) {
        NSLog(@"Error removing file at path: %@", [self fileNameWithExtensionFromURL:file]);
    }
    /*
     if ([[NSFileManager defaultManager] isDeletableFileAtPath:completeFilePath]) {
     BOOL success = [[NSFileManager defaultManager] removeItemAtPath:completeFilePath error:NULL];
     if (!success) {
     NSLog(@"Error removing file at path: %@", completeFilePath);
     }
     }
     */
}


- (void)updateButtonsToMatchTableState
{
    if (self.tableView.editing)
    {
        self.navigationItem.rightBarButtonItem = self.cancelButton;
        
        [self updateDeleteButtonTitle];
        // Show the delete button.
        self.navigationItem.leftBarButtonItem = _deleteButton;
    }
    else
    {
        // Not in editing mode.
        self.navigationItem.leftBarButtonItem = nil;
        
        // Show the edit button, but disable the edit button if there's nothing to edit.
        if (self.messageList.count > 0)
        {
            self.editButton.enabled = YES;
        }
        else
        {
            self.editButton.enabled = NO;
        }
        self.navigationItem.rightBarButtonItem = self.editButton;
    }
}

- (void)updateDeleteButtonTitle
{
    // Update the delete button's title, based on how many items are selected
    NSArray *selectedRows = [self.tableView indexPathsForSelectedRows];
    
    BOOL noItemsAreSelected = selectedRows.count == 0;
    
    if (noItemsAreSelected) {
        self.deleteButton.title = NSLocalizedString(@"Select Message(s)", @"");
    } else {
        NSString *titleFormatString = NSLocalizedString(@"Delete (%d)", @"Title for delete button with placeholder for number");
        self.deleteButton.title = [NSString stringWithFormat:titleFormatString, selectedRows.count];
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [_messageList count];
}

- (NSString *)fileNameWithExtensionFromURL:(NSURL *)file
{
    return [file lastPathComponent];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"MessageCell" forIndexPath:indexPath];
    
    // Configure the cell...
    NSString *fileName = [self fileNameWithExtensionFromURL:_messageList[indexPath.row]];
    
    cell.textLabel.text = [fileName stringByDeletingPathExtension];
    
    return cell;
}


- (BOOL)shouldPerformSegueWithIdentifier:(NSString *)identifier sender:(id)sender
{
    if (self.tableView.isEditing) {
        return NO;
    }
    return YES;
}

#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"MessageDetail"]) {
        MessageViewController *mvc = [segue destinationViewController];
        
        NSIndexPath *index = [self.tableView indexPathForSelectedRow];
        NSURL *file = _messageList[index.row];
        UnstoredIncomingMessage *item = [[UnstoredIncomingMessage alloc] initFromFile:[self fileNameWithExtensionFromURL:file]];
        
        mvc.navigationItem.title = item.messageID;
    }
}

@end

