//
//  InboxTableViewController.m
//  Messangi
//
//  Created by Oscar Anzola on 5/23/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "InboxTableViewController.h"
#import <MessangiSDK/MessangiSDK.h>
#import "MessageViewController.h"

@interface InboxTableViewController () <UIActionSheetDelegate>

@property (nonatomic, strong) NSMutableArray *messageList;
@property (nonatomic, copy) NSString *inboxDirectory;

@property (strong, nonatomic) IBOutlet UIBarButtonItem *editButton;
@property (strong, nonatomic) IBOutlet UIBarButtonItem *cancelButton;
@property (strong, nonatomic) IBOutlet UIBarButtonItem *deleteButton;

@end

@implementation InboxTableViewController

int unreadCount = 0;

- (void)refreshView:(UIRefreshControl *)sender {
    unreadCount = 0;
    _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
    [self checkNotificatioNumber];
    [self.tableView reloadData];
    [sender endRefreshing];
}

- (void)refreshMessangiInboxView {
    unreadCount = 0;
    _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
    [self checkNotificatioNumber];
    [self.tableView reloadData];
}

-(void)viewWillAppear:(BOOL)animated
{
    unreadCount = 0;
    [super viewWillAppear:YES];
    _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
    [self checkNotificatioNumber];
    [self.tableView reloadData];
    
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
   if (!_messageList) {
        
        _inboxDirectory = [UnstoredIncomingMessage documentsInboxPath];
        _messageList = [NSMutableArray arrayWithArray:[self listFilesAtPath:_inboxDirectory]];
    }
      
    [super.tabBarController.viewControllers[0] tabBarItem].title = NSLocalizedString(@"Inbox", @"Instances of the word inbox");
    
    [super.tabBarController.viewControllers[1] tabBarItem].title = NSLocalizedString(@"GeoFences", @"Instances of the word GeoFences");
    
    _screenTitleInbox.title = NSLocalizedString(@"Inbox", @"Instances of the word inbox");
    
    // Load the buttons
    self.navigationItem.rightBarButtonItem = self.editButtonItem;
    
    self.tableView.allowsMultipleSelectionDuringEditing = YES;
    
    [self.refreshControl addTarget:self action:@selector(refreshView:) forControlEvents:UIControlEventValueChanged];
    
    [self updateButtonsToMatchTableState];
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Update the delete button's title based on how many items are selected.
    [self updateDeleteButtonTitle];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [self updateButtonsToMatchTableState];
}

// Do not need this method anymore because it's implemented in the delete action.
//- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
//{
//	// The user tapped one of the OK/Cancel buttons.
//	if (buttonIndex == 0)
//	{
//		// Delete what the user selected.
//        NSArray *selectedRows = [self.tableView indexPathsForSelectedRows];
//        BOOL deleteSpecificRows = selectedRows.count > 0;
//        if (deleteSpecificRows)
//        {
//            // Build an NSIndexSet of all the objects to delete, so they can all be removed at once.
//            NSMutableIndexSet *indicesOfItemsToDelete = [NSMutableIndexSet new];
//            for (NSIndexPath *selectionIndex in selectedRows)
//            {
//                [indicesOfItemsToDelete addIndex:selectionIndex.row];
//                NSString *fileName = [self fileNameWithExtensionFromURL:_messageList[selectionIndex.row]];
//                if (![UnstoredIncomingMessage messageReadStatusFromFileName:fileName]) {
//                    unreadCount--;
//                    [self setNotificationCount];
//                }
//                [UnstoredIncomingMessage deleteMessageFileName:[self fileNameWithExtensionFromURL:_messageList[selectionIndex.row]]];
//            }
//            // Delete the objects from our data model.
//            [self.messageList removeObjectsAtIndexes:indicesOfItemsToDelete];
//            
//            // Tell the tableView that we deleted the objects
//            [self.tableView deleteRowsAtIndexPaths:selectedRows withRowAnimation:UITableViewRowAnimationAutomatic];
//        }
//        // Exit editing mode after the deletion.
//        [self.tableView setEditing:NO animated:YES];
//        [self updateButtonsToMatchTableState];
//	}
//}

- (NSArray *)listFilesAtPath:(NSString *)path
{
    NSURL *urlFilePath = [NSURL fileURLWithPath:path];
    NSArray *directoryContent = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:urlFilePath includingPropertiesForKeys:NULL options:NSDirectoryEnumerationSkipsHiddenFiles error:NULL];
    NSArray *sortedArray = [self sortMessageList:directoryContent];
    return sortedArray;
}

- (IBAction)editAction:(id)sender
{
    [self.tableView setEditing:YES animated:YES];
    
    [self updateButtonsToMatchTableState];
}


- (IBAction)cancelAction:(id)sender
{
    [self.tableView setEditing:NO animated:YES];
    
    [self updateButtonsToMatchTableState];
}

- (IBAction)deleteAction:(id)sender
{
    // Open a dialog with just an OK button.
	NSString *actionTitle;
    if (([[self.tableView indexPathsForSelectedRows] count] == 1)) {
        actionTitle = NSLocalizedString(@"Are you sure you want to delete this message?", @"Delete confirmation message (single message)");
    }
    else
    {
        actionTitle = NSLocalizedString(@"Are you sure you want to delete these messages?", @"Delete confirmation message (multiple messages)");
    }
    
    NSString *cancelTitle = NSLocalizedString(@"Cancel", @"Cancel title for message removal action");
    NSString *okTitle = NSLocalizedString(@"OK", @"OK title for message removal action");
//    UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:actionTitle delegate:self cancelButtonTitle:cancelTitle destructiveButtonTitle:okTitle otherButtonTitles:nil];
//    
//	actionSheet.actionSheetStyle = UIActionSheetStyleDefault;
//    // Show from our table view (pops up in the middle of the table).
//	[actionSheet showInView:self.view];
    
    UIAlertController *sheet = [UIAlertController alertControllerWithTitle:@"Delete" message:actionTitle preferredStyle:UIAlertControllerStyleActionSheet];
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:okTitle style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        
        // Delete what the user selected.
        NSArray *selectedRows = [self.tableView indexPathsForSelectedRows];
        BOOL deleteSpecificRows = selectedRows.count > 0;
        if (deleteSpecificRows)
        {
            // Build an NSIndexSet of all the objects to delete, so they can all be removed at once.
            NSMutableIndexSet *indicesOfItemsToDelete = [NSMutableIndexSet new];
            for (NSIndexPath *selectionIndex in selectedRows)
            {
                [indicesOfItemsToDelete addIndex:selectionIndex.row];
                NSString *fileName = [self fileNameWithExtensionFromURL:_messageList[selectionIndex.row]];
                if (![UnstoredIncomingMessage messageReadStatusFromFileName:fileName]) {
                    unreadCount--;
                    [self setNotificationCount];
                }
                [UnstoredIncomingMessage deleteMessageFileName:[self fileNameWithExtensionFromURL:_messageList[selectionIndex.row]]];
            }
            // Delete the objects from our data model.
            [self.messageList removeObjectsAtIndexes:indicesOfItemsToDelete];
            
            // Tell the tableView that we deleted the objects
            [self.tableView deleteRowsAtIndexPaths:selectedRows withRowAnimation:UITableViewRowAnimationAutomatic];
        }
        // Exit editing mode after the deletion.
        [self.tableView setEditing:NO animated:YES];
        [self updateButtonsToMatchTableState];
    }];
    [sheet addAction:okAction];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelTitle style:UIAlertActionStyleCancel handler:nil];
    [sheet addAction:cancelAction];
    [self presentViewController:sheet animated:YES completion:nil];
}

-(void)deleteMessageFile:(NSURL *)file
{
    
    BOOL success = [[NSFileManager defaultManager] removeItemAtURL:file error:NULL];
    if (!success) {
        NSLog(@"Error removing file at path: %@", [self fileNameWithExtensionFromURL:file]);
    }
}


- (void)updateButtonsToMatchTableState
{
    if (self.tableView.editing)
    {
        self.navigationItem.rightBarButtonItem = self.cancelButton;
        
        [self updateDeleteButtonTitle];
        // Show the delete button.
        self.navigationItem.leftBarButtonItem = _deleteButton;
    }
    else
    {
        // Not in editing mode.
        self.navigationItem.leftBarButtonItem = nil;
        
        // Show the edit button, but disable the edit button if there's nothing to edit.
        if (self.messageList.count > 0)
        {
            self.editButton.enabled = YES;
        }
        else
        {
            self.editButton.enabled = NO;
        }
        self.navigationItem.rightBarButtonItem = self.editButton;
    }
}

- (void)updateDeleteButtonTitle
{
    // Update the delete button's title, based on how many items are selected
    NSArray *selectedRows = [self.tableView indexPathsForSelectedRows];
    
    BOOL noItemsAreSelected = selectedRows.count == 0;
    
    if (noItemsAreSelected) {
        self.deleteButton.title = NSLocalizedString(@"Select Message(s)", @"Select messages for deletion (one or many)");
    } else {
        NSString *titleFormatString = NSLocalizedString(@"Delete (%d)", @"Title for delete button with placeholder for number");
        self.deleteButton.title = [NSString stringWithFormat:titleFormatString, selectedRows.count];
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [_messageList count];
}

- (NSString *)fileNameWithExtensionFromURL:(NSURL *)file
{
    return [file lastPathComponent];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"MessageCell" forIndexPath:indexPath];
    
    // Configure the cell...
    NSString *fileName = [self fileNameWithExtensionFromURL:_messageList[indexPath.row]];
    
    cell.textLabel.text = [UnstoredIncomingMessage messageSubjectFromFileName:[fileName stringByDeletingPathExtension]];
    
    NSDate *savedDate = [UnstoredIncomingMessage messageReceivedDateFromFileName:[fileName stringByDeletingPathExtension]];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"EEE, d MMM yyyy"];
    cell.detailTextLabel.text = [dateFormatter stringFromDate:savedDate];
    if (![UnstoredIncomingMessage messageReadStatusFromFileName:fileName]) {
        cell.textLabel.textColor = [UIColor redColor];
//        unreadCount++;
//        [self setNotificationCount];
    } else {
        cell.textLabel.textColor = [UIColor blackColor];
    }
    return cell;
}


- (BOOL)shouldPerformSegueWithIdentifier:(NSString *)identifier sender:(id)sender
{
    if (self.tableView.isEditing) {
        return NO;
    }
    return YES;
}

#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"MessageDetail"]) {
        MessageViewController *mvc = [segue destinationViewController];
        
        NSIndexPath *index = [self.tableView indexPathForSelectedRow];
        NSURL *file = _messageList[index.row];
        UnstoredIncomingMessage *item = [[UnstoredIncomingMessage alloc] initFromFile:[self fileNameWithExtensionFromURL:file]];
        
        mvc.navigationItem.title = item.subject;
        mvc.messangiMessage = item;
        
        if(![UnstoredIncomingMessage messageReadStatusFromFileName:[self fileNameWithExtensionFromURL:file]]) {
            unreadCount--;
            [self setNotificationCount];
        }
        
        [UnstoredIncomingMessage messageFileNameMarkRead:[self fileNameWithExtensionFromURL:file]];
    }
}

- (void)setNotificationCount
{
    NSString *unreadCountString = [NSString stringWithFormat:@"%d", unreadCount];
    if (unreadCount > 0) {
        [super.tabBarController.viewControllers[0] tabBarItem].badgeValue = unreadCountString;
    } else {
        [super.tabBarController.viewControllers[0] tabBarItem].badgeValue = nil;
    }
}

- (void)checkNotificatioNumber
{
    unreadCount = 0;
    for (id obj in _messageList) {
        NSString *fileName = [self fileNameWithExtensionFromURL:obj];
        if (![UnstoredIncomingMessage messageReadStatusFromFileName:fileName]) {
            unreadCount++;
        }
    }
    [self setNotificationCount];
    [self updateButtonsToMatchTableState];
}

- (NSArray*)sortMessageList:(NSArray *)unsortedArray
{
    NSArray *sortedArray = [unsortedArray sortedArrayUsingComparator:^(id obj1, id obj2) {
        
        NSString *fileName1 = [self fileNameWithExtensionFromURL:obj1];
        NSString *fileName2 = [self fileNameWithExtensionFromURL:obj2];
        
        NSDate *date1 = [UnstoredIncomingMessage messageReceivedDateFromFileName:fileName1];
        NSDate *date2 = [UnstoredIncomingMessage messageReceivedDateFromFileName:fileName2];
        if ([date2 compare:date1] == NSOrderedDescending) {
            return (NSComparisonResult)NSOrderedDescending;
        }
        
        if ([date2 compare:date1] == NSOrderedAscending) {
            return (NSComparisonResult)NSOrderedAscending;
        }
        return (NSComparisonResult)NSOrderedSame;
    }];
    return sortedArray;
}



@end

