//
//  iBeaconsTableViewController.m
//  Messangi
//
//  Created by Oscar Anzola on 7/1/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "iBeaconsTableViewController.h"
#import "iBeaconDefaults.h"
@import CoreLocation;

@interface iBeaconsTableViewController () <CLLocationManagerDelegate>

@property NSMutableDictionary *beacons;
@property NSMutableDictionary *rangedRegions;
@property NSMutableSet *monitoredRegions;
@property(nonatomic)NSInteger regionCounter;
@property CLLocationManager *demoLM;

@end

@implementation iBeaconsTableViewController

- (void)stopMonitoringSupportedUUIDs
{
    for (CLBeaconRegion *region in _monitoredRegions)
    {
        if ([_demoLM.monitoredRegions containsObject:region]) {
            [_demoLM stopMonitoringForRegion:region];
        }
    }
}


- (void)startMonitoringSupportedUUIDs
{
    unsigned int i = 1;
    
    for (NSUUID *uuid in [iBeaconDefaults sharedDefaults].supportedProximityUUIDs)
    {
        CLBeaconRegion *region = [[CLBeaconRegion alloc] initWithProximityUUID:uuid identifier:[NSString stringWithFormat:@"Messangi%i", i++]];
        region.notifyEntryStateOnDisplay = YES;
        region.notifyOnEntry = YES;
        region.notifyOnExit = YES;
        [_demoLM startMonitoringForRegion:region];
        [_monitoredRegions addObject:region];
        [_demoLM startRangingBeaconsInRegion:region];
    }
}

-(void)locationManager:(CLLocationManager *)manager didStartMonitoringForRegion:(CLRegion *)region
{
#ifdef DEBUG
    NSLog(@"****didStartMonitoringForRegion: %@", region.identifier);
#endif
}

-(void)locationManager:(CLLocationManager *)manager monitoringDidFailForRegion:(CLRegion *)region withError:(NSError *)error
{
#ifdef DEBUG
    NSLog(@"****monitoringDidFailForRegion: %@", region.identifier);
#endif
}

- (void)locationManager:(CLLocationManager *)manager didEnterRegion:(CLRegion *)region
{
    if ([region isKindOfClass:[CLBeaconRegion class]]) {
        
        CLBeaconRegion *beaconRegion = (CLBeaconRegion*)region;
        _regionCounter++;
        [_demoLM startRangingBeaconsInRegion:beaconRegion];
    }
}

-(void)locationManager:(CLLocationManager *)manager didExitRegion:(CLRegion *)region
{
    _regionCounter--;
    [_demoLM stopRangingBeaconsInRegion:(CLBeaconRegion*)region];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    [self.tableView reloadData];
}

- (void)refreshView:(UIRefreshControl *)sender {
    [self.tableView reloadData];
    [sender endRefreshing];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.refreshControl addTarget:self action:@selector(refreshView:) forControlEvents:UIControlEventValueChanged];
    
    _demoLM = [[CLLocationManager alloc] init];
    _demoLM.delegate = self;
    self.beacons = [[NSMutableDictionary alloc] init];
    self.rangedRegions = [[NSMutableDictionary alloc] init];
    _monitoredRegions = [[NSMutableSet alloc] initWithCapacity:[iBeaconDefaults sharedDefaults].supportedProximityUUIDs.count];
    _regionCounter = 0;
    
    [self startMonitoringSupportedUUIDs];
    
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:YES];
    [self stopMonitoringSupportedUUIDs];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
 - (void)locationManager:(CLLocationManager *)manager
 didRangeBeacons:(NSArray *)beacons
 inRegion:(CLBeaconRegion *)region
 {
 if ([beacons count] > 0) {
 CLBeacon *nearest = [beacons objectAtIndex:0];
 if (CLProximityImmediate == nearest.proximity) {
 [self showReceipt];
 }
 } else {
 [self hideReceipt];
 }
 }
 */

- (void)locationManager:(CLLocationManager *)manager didRangeBeacons:(NSArray *)beacons inRegion:(CLBeaconRegion *)region
{
    /*
     CoreLocation will call this delegate method at 1 Hz with updated range information.
     Beacons will be categorized and displayed by proximity.  A beacon can belong to multiple
     regions.  It will be displayed multiple times if that is the case.  If that is not desired,
     use a set instead of an array.
     */

    self.rangedRegions[region] = beacons;
    [self.beacons removeAllObjects];
    
    NSMutableArray *allBeacons = [NSMutableArray array];
    
    for (NSArray *regionResult in [self.rangedRegions allValues])
    {
        [allBeacons addObjectsFromArray:regionResult];
    }
    
    for (NSNumber *range in @[@(CLProximityUnknown), @(CLProximityImmediate), @(CLProximityNear), @(CLProximityFar)])
    {
        NSArray *proximityBeacons = [allBeacons filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"proximity = %d", [range intValue]]];
        if([proximityBeacons count])
        {
            self.beacons[range] = proximityBeacons;
        }
    }

    [self.tableView reloadData];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return self.beacons.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSArray *sectionValues = [self.beacons allValues];
    return [sectionValues[section] count];
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    NSString *title;
    NSArray *sectionKeys = [self.beacons allKeys];
    
    // The table view will display beacons by proximity.
    NSNumber *sectionKey = sectionKeys[section];
    
    switch([sectionKey integerValue])
    {
        case CLProximityImmediate:
            title = NSLocalizedString(@"Immediate", @"Immediate section header title");
            break;
            
        case CLProximityNear:
            title = NSLocalizedString(@"Near", @"Near section header title");
            break;
            
        case CLProximityFar:
            title = NSLocalizedString(@"Far", @"Far section header title");
            break;
            
        default:
            title = NSLocalizedString(@"Unknown", @"Unknown section header title");
            break;
    }
    
    return title;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString *identifier = @"iBeaconCell";
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:identifier];
    
    // Display the UUID, major, minor and accuracy for each beacon.
    NSNumber *sectionKey = [self.beacons allKeys][indexPath.section];
    CLBeacon *beacon = self.beacons[sectionKey][indexPath.row];
    cell.textLabel.text = [beacon.proximityUUID UUIDString];
    
    NSString *formatString = NSLocalizedString(@"Major: %@, Minor: %@, Acc: %.2fm", @"Format string for ranging table cells.");
    cell.detailTextLabel.text = [NSString stringWithFormat:formatString, beacon.major, beacon.minor, beacon.accuracy];
	
    return cell;
}

@end
