//
//  CheckSubscriptions.h
//  MessangiSDK
//
//  Created by Robert Volante on 7/14/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#ifndef checkSubscriptions_H
#define checkSubscriptions_H

#import <Foundation/Foundation.h>
#import <CoreLocation/CoreLocation.h>
#import "Subscription.h"
#import <UIKit/UIKit.h>

@protocol CheckSubscriptionsDelegate <NSObject>
@required
/*!
 *  @brief  invoke when the server respond successfully
 *  @param updatedSubscriptions A Array with the subscriptions updated
 */
- (void) checkSubscriptionsSuccess:(NSMutableArray*)updatedSubscriptions;

/*!
 *  @brief  invoke when the server respond failure
 *  @param error A String with response from server
 */
- (void) checkSubscriptionsReturnedError:(NSString *)error;
@end

/*!
 *  @brief  Interface for Check or unCheck subscriptions from Distribution List
 */
@interface CheckSubscriptions : NSObject <NSXMLParserDelegate>
{
    NSMutableString *currentStringValue;
    NSString *currentProperty;
    id <CheckSubscriptionsDelegate> delegate;
    
    void (^completion)(UIBackgroundFetchResult);
}

@property BOOL successful;
@property (nonatomic, strong) Subscription *currentSubscription;
@property (nonatomic, strong) NSString *currentSubscriptionIdentifier;
@property (nonatomic, strong) NSString *currentSubscriptionName;
@property (strong) NSMutableArray *subscriptions;
@property int operation;
@property (retain) id delegate;
@property (nonatomic, strong) NSMutableString *currentStringValue;
@property (nonatomic, strong) NSString *currentProperty;

/*!
 *  @brief  Load a list of possibles subscriptions from Server
 *  @param completionHandler ???
 */
-(void)getUpdatedSubscriptionsWithCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler;

/*!
 *  @brief  Send to Server request to subscribe to list
 *  @param subscriptionID The ID from Distribution list to Subscribe
 */
-(void)subscribeToList:(NSString *)subscriptionID;

/*!
 *  @brief  Send to Server request to unsubscribe from list
 *  @param subscriptionID The ID from Distribution list to unSubscribe
 */
-(void)unsubscribeFromList:(NSString *)subscriptionID;
@end

#endif