//
//  RegisterPhoneDataloader.h
//  Messangi
//
//  Created by Omar Diaz on 05/11/13.
//  Copyright (c) 2013 Ogangi. All rights reserved.
//

#ifndef DeviceSuscriberDelegate_H
#define DeviceSuscriberDelegate_H

#import <Foundation/Foundation.h>

@protocol RegisterPhoneDelegate <NSObject>

/*!
 *  @brief  A protocol need Implement this methods for Authentication
 */
@required
/*!
 *  @brief  if registerDeviceWithAreaCode Works successfully, invoke this method
 *  @param success true
 *  @param message Response from Server
 */
- (void)RegisterPhoneSucceeded:(BOOL)success withMessage:(NSString *)message;

/*!
 *  @brief   When registerDeviceWithAreaCode fail, call this method
 *  @param error Error message send by Server
 */
- (void)RegisterPhoneReturnedError:(NSString *)error;

/*!
 *  @brief  When activatePhoneWithCode work successfully, call this method
 *  @param success true
 *  @param message Response from Server
 */
- (void)ActivatePhoneSucceeded:(BOOL)success withMessage:(NSString *)message;

/*!
 *  @brief  When activatePhoneWithCode fail, call this method
 *  @param error Error message from Server
 */
- (void)ActivatePhoneReturnedError:(NSString *)error;
@end

/*!
 *  @brief  Interface for User Registration, need Implement for Own Registration Dialog
 */
@interface RegisterPhoneDataloader : NSObject <NSXMLParserDelegate>
{
	NSMutableString *currentStringValue;
	NSString *currentProperty;
	id <RegisterPhoneDelegate> delegate;
}

@property BOOL successful;
@property (strong) NSString *message;
@property int operation;
//@property (strong) ServiceCategory *currentCategory;
//@property (strong) NSMutableArray *categories;
@property (retain) id delegate;
@property (nonatomic, strong) NSMutableString *currentStringValue;
@property (nonatomic, strong) NSString *currentProperty;

/*!
 *  @brief  Send Phone Number to Messangi Backend
 *  @param areaCode    Country Code with Area Code
 *  @param phoneNumber seven digits Phone Number
 */
- (void)registerDeviceWithAreaCode:(NSString *)areaCode andPhoneNumber:(NSString *)phoneNumber;

/*!
 *  @brief  Send Validation Code received by SMS to Messangi Backend
 *  @param code Code Received by SMS
 */
- (void)activatePhoneWithCode:(NSString *)code;

@end

#endif
