//
//  checkGeoFences.m
//  Messangi
//
//  Created by Omar Diaz on 10/04/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "MessangiAPI.h"
#import "CheckGeoFences.h"
#import "Messangi.h"

@implementation CheckGeoFences

@synthesize currentProperty, currentStringValue, delegate, operation, successful, currentRegion, currentRegionCenter, currentRegionRadius, currentRegionIdentifier, currentRegionEventType, geofences;


- (void)getUpdatedGeoFencesWithCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
    NSLog(@"getUpdatedGeoFencesWithCompletionHandler");
	operation = 0;
	completion = completionHandler;

	///getGeoFences/{appName}/{clientId}/{timeStamp}/{signature}

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;

	NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%F", [Messangi sharedInstance].appName, [Messangi sharedInstance].clientID, timeStamp];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

	NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

    NSString *urlString = [NSString stringWithFormat:@"%@/getGeoFences/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString :%@",urlString);

    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"getGeoFences: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"getGeoFences: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
	currentProperty = elementName;

	if ([currentProperty isEqualToString:@"geoFences"] && !geofences) {
		geofences = [[NSMutableArray alloc] init];
	}
	return;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (!self.currentStringValue) {
		// currentStringValue is an NSMutableString instance variable
		self.currentStringValue = [[NSMutableString alloc] initWithCapacity:50];
	}
	[self.currentStringValue appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	if (([elementName isEqualToString:@"geoFences"])) {
		[self.delegate checkGeoFencesSuccess:geofences];

		if (completion) {
			completion(UIBackgroundFetchResultNewData);
		}

		return;
	}

	if (([elementName isEqualToString:@"geoFence"])) {
		currentRegion = [[CLCircularRegion alloc] initWithCenter:currentRegionCenter radius:currentRegionRadius identifier:currentRegionIdentifier];

		if ([currentRegionEventType isEqualToString:@"ENTER"]) {
			currentRegion.notifyOnExit = NO;
		}
		else if ([currentRegionEventType isEqualToString:@"EXIT"]) {
			currentRegion.notifyOnEntry = NO;
		}

		NSMutableDictionary *dictionary = [NSMutableDictionary dictionary];
		[dictionary setObject:currentRegion forKey:@"Region"];
		[dictionary setObject:_currentRegionEventTitle forKey:@"Name"];
		[geofences addObject:dictionary];

		currentRegion = nil;

		return;
	}


	NSString *prop = [self currentProperty];

	if ([prop isEqualToString:@"geoFenceId"]) {
		currentRegionIdentifier = currentStringValue;
	}
	else if ([prop isEqualToString:@"latitude"]) {
		currentRegionCenter.latitude = [currentStringValue doubleValue];
	}
	else if ([prop isEqualToString:@"longitude"]) {
		currentRegionCenter.longitude = [currentStringValue doubleValue];
	}
	else if ([prop isEqualToString:@"radius"]) {
		currentRegionRadius = [currentStringValue doubleValue];
	}
	else if ([prop isEqualToString:@"eventType"]) {
		currentRegionEventType = currentStringValue;
	}
	else if ([prop isEqualToString:@"msgTitle"]) {
		_currentRegionEventTitle = currentStringValue;
	}
	currentStringValue = nil;
}

- (void)parserDidStartDocument:(NSXMLParser *)parser {
	//FLog(@"found file and started parsing");
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	NSString *errorString = [NSString stringWithFormat:NSLocalizedString(@"System Error", nil), [parseError code]];
	//FLog(@"error parsing XML: %@", errorString);

	[delegate checkGeoFencesReturnedError:[NSString stringWithFormat:@"CheckGeoFences Error: %@", errorString]];


	if (completion) {
		completion(UIBackgroundFetchResultNewData);
	}
}

+ (NSBundle *)bundleWithName:(NSString *)name {
	NSString *mainBundlePath = [[NSBundle mainBundle] resourcePath];
	NSString *frameworkBundlePath = [mainBundlePath stringByAppendingPathComponent:name];
	if ([[NSFileManager defaultManager] fileExistsAtPath:frameworkBundlePath]) {
		return [NSBundle bundleWithPath:frameworkBundlePath];
	}
	return nil;
}

@end
