//
//  CheckSubscriptions.m
//  MessangiSDK
//
//  Created by Robert Volante on 7/14/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "CheckSubscriptions.h"
#import "Messangi.h"
#import "MessangiAPI.h"

@implementation CheckSubscriptions

@synthesize currentProperty, currentStringValue, delegate, operation, successful, currentSubscription, currentSubscriptionIdentifier, currentSubscriptionName, subscriptions;


- (void)getUpdatedSubscriptionsWithCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	NSLog(@"getUpdatedSubscriptionsWithCompletionHandler");
	operation = 0;
	completion = completionHandler;

	NSString *subURL = [Messangi sharedInstance].subscriptionURL;
	NSString *subInstanceID = [Messangi sharedInstance].subscriptionInstanceID;

	//End of url is /a as a placeholder. It will eventually become some sort of authentication
	NSString *urlString = [NSString stringWithFormat:@"%@/distributionlists/%@/a", subURL, subInstanceID];

    NSLog(@"urlString = %@",urlString);
    
    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"distributionlists: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"distributionlists: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)subscribeToList:(NSString *)subscriptionID {
	operation = 0;

	//@Path("addtodistlist/{dlId}/{mobilenumber}")
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSString *deviceUniqueID = [defaults objectForKey:@"clientPhoneNumber"];
	NSString *subURL = [Messangi sharedInstance].subscriptionURL;

	NSString *urlString = [NSString stringWithFormat:@"%@/addtodistlist/%@/%@", subURL, subscriptionID, deviceUniqueID];
    NSLog(@"urlString = %@",urlString);
    
    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"addtodistlist: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"addtodistlist: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)unsubscribeFromList:(NSString *)subscriptionID {
	operation = 0;

	//@Path("removefromdistlist/{dlId}/{mobilenumber}")
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSString *deviceUniqueID = [defaults objectForKey:@"clientPhoneNumber"];
	NSString *subURL = [Messangi sharedInstance].subscriptionURL;

	NSString *urlString = [NSString stringWithFormat:@"%@/removefromdistlist/%@/%@", subURL, [subscriptionID stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]], [deviceUniqueID stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]]];
    NSLog(@"urlString = %@",urlString);
    
    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"removefromdistlist: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"removefromdistlist: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
	currentProperty = elementName;

	if ([currentProperty isEqualToString:@"distributionLists"] && !subscriptions) {
		subscriptions = [[NSMutableArray alloc] init];
	}

	return;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (!self.currentStringValue) {
		// currentStringValue is an NSMutableString instance variable
		self.currentStringValue = [[NSMutableString alloc] initWithCapacity:50];
	}
	[self.currentStringValue appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	if (([elementName isEqualToString:@"distributionLists"])) {
		[self.delegate checkSubscriptionsSuccess:subscriptions];

		if (completion) {
			completion(UIBackgroundFetchResultNewData);
		}

		return;
	}

	if (([elementName isEqualToString:@"distributionlist"])) {
		currentSubscription = [[Subscription alloc] init];
		currentSubscription.subID = currentSubscriptionIdentifier;
		currentSubscription.name = currentSubscriptionName;

		[subscriptions addObject:currentSubscription];

		currentSubscription = nil;

		return;
	}


	NSString *prop = [self currentProperty];

	if ([prop isEqualToString:@"id"]) {
		currentSubscriptionIdentifier = currentStringValue;
	}
	else if ([prop isEqualToString:@"name"]) {
		currentSubscriptionName = currentStringValue;
	}
	currentStringValue = nil;
}

- (void)parserDidStartDocument:(NSXMLParser *)parser {
	//FLog(@"found file and started parsing");
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	NSString *errorString = [NSString stringWithFormat:NSLocalizedString(@"System Error", nil), [parseError code]];
	//FLog(@"error parsing XML: %@", errorString);

	[delegate checkSubscriptionsReturnedError:[NSString stringWithFormat:@"CheckSubscriptions Error: %@", errorString]];

	if (completion) {
		completion(UIBackgroundFetchResultNewData);
	}
}

@end
