//
//  LoadBeacons.m
//  MessangiSDK
//
//  Created by Oscar Anzola on 8/26/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "LoadBeacons.h"
#import "MessangiAPI.h"
#import "Messangi.h"

@implementation LoadBeacons

void (^completion)(UIBackgroundFetchResult);

- (void)getUpdatedBeaconsWithCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	_operation = 0;
	completion = completionHandler;

    ///getGeoFences/{appName}/{clientId}/{timeStamp}/{signature}

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;

	NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%F", [Messangi sharedInstance].appName, [Messangi sharedInstance].clientID, timeStamp];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

	NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

	NSString *urlString = [NSString stringWithFormat:@"%@/getBeacons/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString = %@",urlString);
    
    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                              NSLog(@"getBeacons: completed successfully");
                                          }
                                          else
                                          {
                                              NSLog(@"getBeacons: completed with error: %@", [error localizedDescription]);
                                           }
                                      }];
    [task resume];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
	_currentProperty = elementName;

	if ([_currentProperty isEqualToString:@"BeaconRegions"] && !_beacons) {
		_beacons = [[NSMutableArray alloc] init];
	}
	return;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (!_currentStringValue) {
		// currentStringValue is an NSMutableString instance variable
		_currentStringValue = [[NSMutableString alloc] initWithCapacity:50];
	}
	[_currentStringValue appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	if (([elementName isEqualToString:@"BeaconRegions"])) {
		[_delegate loadBeaconsSuccess:_beacons];

		if (completion) {
			completion(UIBackgroundFetchResultNewData);
		}

		return;
	}

	if (([elementName isEqualToString:@"beacon"])) {
//        _currentRegion = [[CLBeaconRegion alloc] initWithCenter:_currentRegionCenter radius:_currentRegionRadius identifier:_currentRegionIdentifier];
		_currentRegion = [[CLBeaconRegion alloc] initWithProximityUUID:_currentRegionUUID major:_currentRegionMajor minor:_currentRegionMinor identifier:_currentRegionIdentifier];

		if (!_currentRegion) {
			return;
		}

		if ([_currentRegionEventType isEqualToString:@"ENTER"]) {
			_currentRegion.notifyOnExit = NO;
		}
		else if ([_currentRegionEventType isEqualToString:@"EXIT"]) {
			_currentRegion.notifyOnEntry = NO;
		}

		NSMutableDictionary *dictionary = [NSMutableDictionary dictionary];
		[dictionary setObject:_currentRegion forKey:@"Region"];
		[dictionary setObject:_currentRegionEventTitle forKey:@"Name"];
		[dictionary setObject:[NSDate distantPast] forKey:@"TimeStamp"];

		[_beacons addObject:dictionary];

		_currentRegion = nil;

		return;
	}

	NSString *prop = _currentProperty;

	if ([prop isEqualToString:@"regionId"]) {
		_currentRegionIdentifier = _currentStringValue;
	}
	else if ([prop isEqualToString:@"major"]) {
		_currentRegionMajor = [_currentStringValue integerValue];
	}
	else if ([prop isEqualToString:@"minor"]) {
		_currentRegionMinor = [_currentStringValue integerValue];
	}
	else if ([prop isEqualToString:@"uuid"]) {
		_currentRegionUUID = [[NSUUID alloc] initWithUUIDString:_currentStringValue];
	}
	else if ([prop isEqualToString:@"eventType"]) {
		_currentRegionEventType = _currentStringValue;
	}
	else if ([prop isEqualToString:@"distanceString"]) {
		NSInteger distanceKey;

		if ([_currentStringValue isEqualToString:@"IMMEDIATE"]) {
			distanceKey = CLProximityImmediate;
		}
		else if ([_currentStringValue isEqualToString:@"NEAR"]) {
			distanceKey = CLProximityNear;
		}
		else if ([_currentStringValue isEqualToString:@"FAR"]) {
			distanceKey = CLProximityFar;
		}

		_currentRegionDistance = distanceKey;
	}
	else if ([prop isEqualToString:@"name"]) {
		_currentRegionEventTitle = _currentStringValue;
	}
	_currentStringValue = nil;
}

- (void)parserDidStartDocument:(NSXMLParser *)parser {
	//FLog(@"found file and started parsing");
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	NSString *errorString = [NSString stringWithFormat:NSLocalizedString(@"System Error", nil), [parseError code]];
	//FLog(@"error parsing XML: %@", errorString);

	[_delegate loadBeaconsReturnedError:[NSString stringWithFormat:@"LoadBeacons Error: %@", errorString]];


	if (completion) {
		completion(UIBackgroundFetchResultNewData);
	}
}

+ (NSBundle *)bundleWithName:(NSString *)name {
	NSString *mainBundlePath = [[NSBundle mainBundle] resourcePath];
	NSString *frameworkBundlePath = [mainBundlePath stringByAppendingPathComponent:name];
	if ([[NSFileManager defaultManager] fileExistsAtPath:frameworkBundlePath]) {
		return [NSBundle bundleWithPath:frameworkBundlePath];
	}
	return nil;
}

@end
