//
//  LocationDataLoader.h
//  Messangi
//
//  Created by Omar Diaz on 25/03/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#ifndef LocationDataLoader_H
#define LocationDataLoader_H

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

@protocol LocationDelegate <NSObject>
@required
- (void)LocationCallSuccess:(BOOL)success withMessage:(NSString *)msg;
- (void)LocationCallReturnedError:(NSString *)error;
@end

/*!
 *  @brief  Interface for manipulate GeoFence and Beacon
 */
@interface LocationDataLoader : NSObject <NSXMLParserDelegate, LocationDelegate>
{
	NSMutableString *currentStringValue;
	NSString *currentProperty;
	id <LocationDelegate> delegate;

	void (^completion)(UIBackgroundFetchResult);
}

@property BOOL successful;
@property (strong) NSString *message;
@property int operation;
@property (retain) id delegate;
@property (nonatomic, strong) NSMutableString *currentStringValue;
@property (nonatomic, strong) NSString *currentProperty;


/*!
 *  @brief Send Phone Location latitude and longitude to Server
 *  @param requestId         The ID from Push notification with type GET_LOC
 *  @param latitude          float with latitude coordinate from GPS
 *  @param longitude         float with longitude coordinate from GPS
 *  @param completionHandler ???
 */
- (void)sendLocationWithRequestID:(NSString *)requestId latitude:(float)latitude longitude:(float)longitude andCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler;

// ???: not know when called
- (void)alertEventSyncInRegion:(NSString *)regionId withStatus:(NSString *)status atLatitude:(float)latitude longitude:(float)longitude;

/*!
 *  @brief  Deprecate, now call fireEventInRegion method
 *  @param regionId          ID from Region fired
 *  @param status            A NSString with status of ENTER or EXIT
 *  @param latitude          float with latitude of Region
 *  @param longitude         float with longitude of region
 *  @param completionHandler ???
 */
- (void)alertEventInRegion:(NSString *)regionId withStatus:(NSString *)status atLatitude:(float)latitude longitude:(float)longitude andCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler;




/*!
 *  @brief  Send to Server when fire a event in Region
 *  @param regionId          ID from Region fired
 *  @param type              A NSString BEACON or GEOFENCE
 *  @param status            A NSString with status of ENTER or EXIT
 *  @param latitude          float with latitude of Region
 *  @param longitude         float with longitude of region
 *  @param completionHandler ???
 */
- (void)fireEventInRegion:(NSString *)regionId regionType:(NSString *)type withStatus:(NSString *)status atLatitude:(float)latitude longitude:(float)longitude andCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler;


@end

#endif
