//
//  LocationDataLoader.m
//  Messangi
//
//  Created by Omar Diaz on 25/03/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "MessangiAPI.h"
#import "LocationDataLoader.h"
#import "Messangi.h"

@implementation LocationDataLoader

@synthesize currentStringValue, currentProperty, delegate, successful, message, operation;

- (id)init {
	if (!self) {
		self = [super init];
	}

	return self;
}

- (void)sendLocationWithRequestID:(NSString *)requestId latitude:(float)latitude longitude:(float)longitude andCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	operation = 0;
	completion = completionHandler;

	//sendLocation/{requestId}/{deviceId}/{date}/{latitude}/{longitude}/{method}/{clientId}/{timeStamp}/{signature}

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;
	long long today = [[NSNumber numberWithDouble:timeStamp] longLongValue];

	NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%lld/%F/%F/%@/%@/%F", requestId, [defaults objectForKey:@"deviceUniqueID"], today, latitude, longitude, @"APP", [Messangi sharedInstance].clientID, timeStamp];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

	NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

	NSString *urlString = [NSString stringWithFormat:@"%@/sendLocation/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString = %@",urlString);

    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"sendLocation: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"sendLocation: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)alertEventInRegion:(NSString *)regionId withStatus:(NSString *)status atLatitude:(float)latitude longitude:(float)longitude andCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	operation = 1;
	completion = completionHandler;

	//geoFenceEvent/{regionId}/{deviceId}/{status}/{date}/{latitude}/{longitude}/{appName}/{platform}/{clientId}/{timeStamp}/{signature}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;
	long long today = [[NSNumber numberWithDouble:timeStamp] longLongValue];

	NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%@/%lld/%F/%F/%@/%@/%@/%F", regionId, [defaults objectForKey:@"clientPhoneNumber"], status, today, latitude, longitude, [Messangi sharedInstance].appName, @"IOS", [Messangi sharedInstance].clientID, timeStamp];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

	NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

	NSString *urlString = [NSString stringWithFormat:@"%@/geoFenceEvent/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString = %@",urlString);

    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"geoFenceEvent: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"geoFenceEvent: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)fireEventInRegion:(NSString *)regionId regionType:(NSString *)type withStatus:(NSString *)status atLatitude:(float)latitude longitude:(float)longitude andCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	operation = 1;
	completion = completionHandler;

	//notifyRegionEvent/{regionId}/{deviceId}/{status}/{date}/{latitude}/{longitude}/{type}/{appName}/{platform}/{clientId}/{timestamp}/{signature}

	//geoFenceEvent/{regionId}/{deviceId}/{status}/{date}/{latitude}/{longitude}/{appName}/{platform}/{clientId}/{timeStamp}/{signature}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;
	long long today = [[NSNumber numberWithDouble:timeStamp] longLongValue];

	NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%@/%lld/%F/%F/%@/%@/%@/%@/%F", regionId, [defaults objectForKey:@"deviceUniqueID"], status, today, latitude, longitude, type, [Messangi sharedInstance].appName, @"IOS", [Messangi sharedInstance].clientID, timeStamp];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

    NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

	NSString *urlString = [NSString stringWithFormat:@"%@/notifyRegionEvent/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString = %@",urlString);

    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"notifyRegionEvent: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                         }
                                          else
                                          {
                                              NSLog(@"notifyRegionEvent: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)alertEventSyncInRegion:(NSString *)regionId withStatus:(NSString *)status atLatitude:(float)latitude longitude:(float)longitude {
	//geoFenceEvent/{regionId}/{deviceId}/{status}/{date}/{latitude}/{longitude}/{appName}/{platform}/{clientId}/{timeStamp}/{signature}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;
	long long today = [[NSNumber numberWithDouble:timeStamp] longLongValue];
	NSString *prefix = [defaults objectForKey:@"messangiServicesPrefix"];

	NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%@/%lld/%F/%F/%@/%@/%@/%F", regionId, [defaults objectForKey:@"clientPhoneNumber"], status, today, latitude, longitude, [defaults objectForKey:@"messangiAppName"], @"IOS", [defaults objectForKey:@"messangiClientID"], timeStamp];

	NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[defaults objectForKey:@"apiClientPublicKey"]];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

	NSString *urlString = [NSString stringWithFormat:@"%@/geoFenceEvent/%@/%@", prefix, encodedStringToSign, signedString];

	NSMutableURLRequest *theRequest = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlString]
	                                                          cachePolicy:NSURLRequestUseProtocolCachePolicy
	                                                      timeoutInterval:20.0];
	[theRequest setHTTPMethod:@"GET"];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
	currentProperty = elementName;

	return;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (!self.currentStringValue) {
		// currentStringValue is an NSMutableString instance variable
		self.currentStringValue = [[NSMutableString alloc] initWithCapacity:50];
	}
	[self.currentStringValue appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	if (([elementName isEqualToString:@"transactionResult"])) {
		switch (operation) {
			case 0:
				[self.delegate LocationCallSuccess:self.successful withMessage:[NSString stringWithFormat:@"SendLocation Message: %@", self.message]];
				break;

			case 1:
				[self.delegate LocationCallSuccess:self.successful withMessage:[NSString stringWithFormat:@"AlertEvent Message: %@", self.message]];
				break;
		}

		if (completion) {
			completion(UIBackgroundFetchResultNewData);
		}

		return;
	}


	NSString *prop = [self currentProperty];



	if ([prop isEqualToString:@"message"]) {
		self.message = currentStringValue;
	}

	if ([prop isEqualToString:@"successful"]) {
		if ([currentStringValue isEqual:@"true"])
			self.successful = YES;
		else
			self.successful = NO;
	}
	currentStringValue = nil;
}

- (void)parserDidStartDocument:(NSXMLParser *)parser {
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	NSString *errorString = [NSString stringWithFormat:NSLocalizedString(@"System Error", nil), [parseError code]];

	switch (operation) {
		case 0:
			[delegate LocationCallReturnedError:[NSString stringWithFormat:@"SendLocation Error: %@", errorString]];
			break;

		case 1:
			[delegate LocationCallReturnedError:[NSString stringWithFormat:@"AlertEvent Error: %@", errorString]];
			break;
	}

	if (completion) {
		completion(UIBackgroundFetchResultNewData);
	}
}

- (void)LocationCallSuccess:(BOOL)success withMessage:(NSString *)msg {
}

- (void)LocationCallReturnedError:(NSString *)error {
}

@end
