//
//  Messangi.m
//  Messangi
//
//  Created by Francisco Andrades Grassi on 6/2/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "Messangi.h"
#import "MessangiAPI.h"
#import "LocationDataLoader.h"
#import "SingleMessageLoader.h"
#import "DeviceSubscriber.h"
#import "RegisterPhoneDataloader.h"
#import <MapKit/MapKit.h>
#import "PendingMessageDataLoader.h"
#import "EventLogger.h"

/*
 * AdSupport is being used only to get an unique identifier for each device that do not change across uninstalls/installs
 */
#import <AdSupport/AdSupport.h>

@interface Messangi ()
@property NSMutableSet *geoFencesToAdd;
@property NSMutableSet *geoFencesToDelete;
@property NSMutableSet *beaconsToAdd;
@property NSMutableSet *beaconsToDelete;
@end

@implementation Messangi

+ (Messangi *)sharedInstance {
	static Messangi *instance = nil;

	if (nil == instance) {
		instance = [[[self class] alloc] init];
		instance.messangiLM = [[CLLocationManager alloc] init];
		//   instance.messangiBeaconLM = [[CLLocationManager alloc] init];
		instance.geoFencesList = [[NSMutableOrderedSet alloc] init];
		instance.beaconsList = [[NSMutableOrderedSet alloc] init];
		instance.subscriptionsList = [[NSMutableArray alloc] init];
		instance.geoFencesToAdd = [[NSMutableSet alloc] init];
		instance.geoFencesToDelete = [[NSMutableSet alloc] init];
		instance.beaconsToAdd = [[NSMutableSet alloc] init];
		instance.beaconsToDelete = [[NSMutableSet alloc] init];

		instance.lastPendingMessagesFetch = 0;
	}
	return instance;
}

- (BOOL)statusLocationServicesOK {
	if ([CLLocationManager locationServicesEnabled] && ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusAuthorizedAlways))
		return YES;
	return NO;
}

- (void)initMessangi:(BOOL)first {
	if (first) {
		_messangiLM.delegate = self;
		[_messangiLM requestAlwaysAuthorization];
		[self startStandardUpdates];
		CheckGeoFences *updateGeoFences = [[CheckGeoFences alloc] init];
		updateGeoFences.delegate = self;
		[updateGeoFences getUpdatedGeoFencesWithCompletionHandler:nil];
		LoadBeacons *updateBeacons = [[LoadBeacons alloc] init];
		updateBeacons.delegate = self;
		[updateBeacons getUpdatedBeaconsWithCompletionHandler:nil];
		_useMessangiSDK = true;
	}
	else {
		[self showPhoneNumberEntryAlert];
	}
}

- (void)initSubscriptions {
	_subscriptionsList = [[NSMutableArray alloc] init];
	CheckSubscriptions *updateSubs = [[CheckSubscriptions alloc] init];
	updateSubs.delegate = self;
	[updateSubs getUpdatedSubscriptionsWithCompletionHandler:nil];
}

- (void)subscribeToList:(NSString *)subscriptionID {
	CheckSubscriptions *updateSubs = [[CheckSubscriptions alloc] init];
	updateSubs.delegate = self;
	[updateSubs subscribeToList:subscriptionID];
}

- (void)unsubscribeFromList:(NSString *)subscriptionID  {
	CheckSubscriptions *updateSubs = [[CheckSubscriptions alloc] init];
	updateSubs.delegate = self;
	[updateSubs unsubscribeFromList:subscriptionID];
}

- (void)showPhoneNumberEntryAlert {
	NSString *title = NSLocalizedString(@"Your Mobile Number", @"Title for mobile number entry dialog");
	NSString *message = NSLocalizedString(@"Include country code, example: 17864428125", @"Phone number entry prompt");
	NSString *otherButtonTitle = NSLocalizedString(@"OK", nil);
	NSString *cancelButtonTitle = NSLocalizedString(@"Cancel", @"A cancel button");

	UIAlertController *alert = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
	[alert addTextFieldWithConfigurationHandler: ^(UITextField *textField) {
	    // If you need to customize the text field, you can do so here.
	    [textField setKeyboardType:UIKeyboardTypeNumberPad];
	    [textField setDelegate:self];
	}];
	UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelButtonTitle style:UIAlertActionStyleCancel handler: ^(UIAlertAction *action) {
	}];
	UIAlertAction *okAction = [UIAlertAction actionWithTitle:otherButtonTitle style:UIAlertActionStyleDefault handler: ^(UIAlertAction *action) {
	    RegisterPhoneDataloader *rpdl = [[RegisterPhoneDataloader alloc]init];
	    rpdl.delegate = self;
	    [rpdl registerDeviceWithAreaCode:@"+" andPhoneNumber:((UITextField *)alert.textFields[0]).text];

	    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	    [defaults setObject:[NSString stringWithFormat:@"+%@", ((UITextField *)alert.textFields[0]).text] forKey:@"clientPhoneNumber"];
	    [defaults synchronize];
	    [self showValidationCodeEntryAlert];
	}];
	[alert addAction:cancelAction];
	[alert addAction:okAction];
	UIWindow *window = [UIApplication sharedApplication].keyWindow;
	UIViewController *vc = [window rootViewController];
	// Present Alert
	[vc presentViewController:alert animated:YES completion:nil];

	// Old Code
//	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:otherButtonTitle, nil];
//    alert.tag = 0;
//    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
//    [[alert textFieldAtIndex:0] setKeyboardType:UIKeyboardTypePhonePad];
//    [alert show];
}

- (void)showValidationCodeEntryAlert {
	NSString *title = NSLocalizedString(@"Phone Number Validation", @"Validate this code");
	NSString *message = NSLocalizedString(@"Enter validation code", @"Prompts user to enter their validation code");
	NSString *cancelButtonTitle = NSLocalizedString(@"Cancel", @"A cancel button");
	NSString *otherButtonTitle = NSLocalizedString(@"OK", nil);

	UIAlertController *alert = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
	[alert addTextFieldWithConfigurationHandler: ^(UITextField *textField) {
	    // If you need to customize the text field, you can do so here.
	    [textField setKeyboardType:UIKeyboardTypeNumberPad];
	    textField.tag = 1;
	    [textField setDelegate:self];
	}];

	UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelButtonTitle style:UIAlertActionStyleCancel handler: ^(UIAlertAction *action) {
	}];
	UIAlertAction *okAction = [UIAlertAction actionWithTitle:otherButtonTitle style:UIAlertActionStyleDefault handler: ^(UIAlertAction *action) {
	    RegisterPhoneDataloader *rpdl = [[RegisterPhoneDataloader alloc]init];
	    rpdl.delegate = self;
	    [rpdl activatePhoneWithCode:((UITextField *)alert.textFields[0]).text];
	}];
	[alert addAction:cancelAction];
	[alert addAction:okAction];
	UIWindow *window = [UIApplication sharedApplication].keyWindow;
	UIViewController *vc = [window rootViewController];
	// Present Alert
	[vc presentViewController:alert animated:YES completion:nil];

	// Old Code
//	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:otherButtonTitle, nil];
//    alert.tag = 1;
//    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
//    [[alert textFieldAtIndex:0] setKeyboardType:UIKeyboardTypeNumberPad];
//    [[alert textFieldAtIndex:0] setDelegate:self];
//
//    [alert show];
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
	if (textField.tag == 1) {
		BOOL result = YES;
		// restrict the maximum number of characters to 5
		if (textField.text.length == 5 && string.length > 0)
			result = NO;

		return result;
	}
	else
		return YES;
}

- (void)ActivatePhoneSucceeded:(BOOL)success withMessage:(NSString *)message {
	if (success) {
		[[Messangi sharedInstance] initMessangi:true];
		NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
		[defaults setBool:NO forKey:@"isFirstTime"];
		[defaults setObject:@"YES" forKey:@"validated"];
		[defaults synchronize];
	}
	else {
		[self showPhoneNumberEntryAlert];
	}
}

- (NSUInteger)distinctUUIDinBeaconList {
	NSSet *onlyRegion = [[_beaconsList valueForKey:@"Region"] valueForKey:@"proximityUUID"];
	return onlyRegion.count;
}

- (void)refreshMonitoredRegionsToClosest {
	NSSet *beaconsAdded = [NSSet setWithSet:_beaconsToAdd];
	[_beaconsList unionSet:beaconsAdded];
	[_beaconsToAdd minusSet:beaconsAdded];

	CLBeaconRegion *beaconRegion;
	NSMutableSet *deleteAllTheseBeacons = [[NSMutableSet alloc] initWithCapacity:_beaconsToDelete.count];
	NSSet *beaconsDeleted = [NSSet setWithSet:_beaconsToDelete];
	for (NSString *regionToDeleteIdentifier in beaconsDeleted) {
		for (NSDictionary *beacon in _beaconsList) {
			beaconRegion = beacon[@"Region"];
			if ([beaconRegion.identifier isEqualToString:regionToDeleteIdentifier]) {
				if ([_messangiLM.monitoredRegions containsObject:beaconRegion]) {
					[_messangiLM stopMonitoringForRegion:beaconRegion];
				}
				[deleteAllTheseBeacons addObject:beacon];
				break;
			}
		}
	}
	[_beaconsList minusSet:deleteAllTheseBeacons];
	[_beaconsToDelete minusSet:beaconsDeleted];

	NSSet *geoFencesAdded = [NSSet setWithSet:_geoFencesToAdd];
	[_geoFencesList unionSet:geoFencesAdded];
	[_geoFencesToAdd minusSet:geoFencesAdded];

	CLCircularRegion *circularRegion;
	NSMutableSet *deleteAllThese = [[NSMutableSet alloc] initWithCapacity:_geoFencesToDelete.count];
	NSSet *geoFencesDeleted = [NSSet setWithSet:_geoFencesToDelete];
	for (NSString *regionToDeleteIdentifier in geoFencesDeleted) {
		for (NSDictionary *geoFence in _geoFencesList) {
			circularRegion = geoFence[@"Region"];
			if ([circularRegion.identifier isEqualToString:regionToDeleteIdentifier]) {
				if ([_messangiLM.monitoredRegions containsObject:circularRegion]) {
					[_messangiLM stopMonitoringForRegion:circularRegion];
				}
				[deleteAllThese addObject:geoFence];
				break;
			}
		}
	}
	[_geoFencesList minusSet:deleteAllThese];
	[_geoFencesToDelete minusSet:geoFencesDeleted];

	CLLocation *currentLocation = _messangiLastLocation;
	[_geoFencesList sortUsingComparator: ^(NSDictionary *geoFence1, NSDictionary *geoFence2) {
	    CLCircularRegion *obj1 = geoFence1[@"Region"];
	    CLCircularRegion *obj2 = geoFence2[@"Region"];
	    CLLocation *loc1 = [[CLLocation alloc] initWithLatitude:obj1.center.latitude longitude:obj1.center.longitude];
	    CLLocation *loc2 = [[CLLocation alloc] initWithLatitude:obj2.center.latitude longitude:obj2.center.longitude];
	    CLLocationDistance distance1 = [loc1 distanceFromLocation:currentLocation];
	    CLLocationDistance distance2 = [loc2 distanceFromLocation:currentLocation];

	    if (distance2 < distance1) {
	        return (NSComparisonResult)NSOrderedDescending;
		}
	    if (distance2 > distance1) {
	        return (NSComparisonResult)NSOrderedAscending;
		}
	    return (NSComparisonResult)NSOrderedSame;
	}];

	NSUInteger numGeoGences = _geoFencesList.count;
	NSSet *monitoredRegions = _messangiLM.monitoredRegions;
	NSUInteger numBeacons = _beaconsList.count;
	NSUInteger i;

	/*
	 * A better approach to this is to look at all regions being monitored
	 * for the GeoFences that are monitored and have an index >= MIN(numGeoFences,MAX_REGIONS)
	 * stop monitoring them
	 */
	for (i = (numBeacons >= MAX_REGIONS) ? 0 : (MAX_REGIONS - numBeacons); i < numGeoGences; i++) {
		if ([monitoredRegions containsObject:_geoFencesList[i][@"Region"]]) {
			[_messangiLM stopMonitoringForRegion:_geoFencesList[i][@"Region"]];
		}
	}
	for (i = 0; i < MIN(MAX_REGIONS, numBeacons); i++) {
		if (![monitoredRegions containsObject:_beaconsList[i][@"Region"]]) {
			[_messangiLM startMonitoringForRegion:_beaconsList[i][@"Region"]];
		}
	}
	for (i = 0; i < MIN(MAX_REGIONS - numBeacons, numGeoGences); i++) {
		if (![monitoredRegions containsObject:_geoFencesList[i][@"Region"]]) {
			[_messangiLM startMonitoringForRegion:_geoFencesList[i][@"Region"]];
		}
	}
}

#pragma mark Location monitoring and delegate

- (void)startStandardUpdates {

	_messangiLM.desiredAccuracy = kCLLocationAccuracyBest;
#warning OA: we leave this in zero, I recommend we with clam analyse and fine-tune
	_messangiLM.distanceFilter = kCLDistanceFilterNone;
	[_messangiLM startUpdatingLocation];

    //[_messangiLM startMonitoringSignificantLocationChanges];
}

- (void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray *)locations {
	CLLocation *lastLocation = [locations lastObject];
	CLLocationDistance distanceDeltaFromLastKnowLocation = [_messangiLastLocation distanceFromLocation:lastLocation];
	_messangiLastLocation = lastLocation;

	if (distanceDeltaFromLastKnowLocation >= GEOFENCELIST_METER_UPDATE_THRESHOLD) {
        //Opcional para Debug
     //   NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: didUpdateLocations: longitude: %f, latitude: %f, accuracy: %f,distanceDeltaFromLastKnowLocation: %f", lastLocation.coordinate.longitude,lastLocation.coordinate.latitude,lastLocation.horizontalAccuracy, distanceDeltaFromLastKnowLocation];
    //    [self logEvent:@"LOCATION_UPDATE" andExtraInfo:logMessage];
		[self refreshMonitoredRegionsToClosest];
	}
}

#pragma mark GeoLocation

- (void)sendLastLocationForMsgID:(NSString *)msgID withHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	LocationDataLoader *ldl = [[LocationDataLoader alloc] init];
	ldl.delegate = ldl;

    //TODO: REVISAR ESTE NOMBRE DE CAMPAÑA
    // Necesario para BigData
    [self logEvent:@"GET_LOC" andExtraInfo:msgID withCampaignName:nil];

	[ldl sendLocationWithRequestID:msgID latitude:_messangiLastLocation.coordinate.latitude longitude:_messangiLastLocation.coordinate.longitude andCompletionHandler:completionHandler];
}

#pragma mark GeoPush

- (void)processGeoPushForMsgID:(NSString *)msgID withHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
    NSLog(@"PROCCESS PUSH");
	NSArray *data = [msgID componentsSeparatedByString:@":"];

	CLLocation *coordinate = [[CLLocation alloc] initWithLatitude:[[data objectAtIndex:2] doubleValue] longitude:[[data objectAtIndex:3] doubleValue]];
	CLLocationDistance distance = [_messangiLastLocation distanceFromLocation:coordinate];

    //Opcional para Debug
    NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: processGeoPushForMsgID, ID: %@,LATITUDE: %f, LONGITUDE: %f, RADIUS: %f, DISTANCE: %f",[data objectAtIndex:1],[[data objectAtIndex:2] doubleValue],[[data objectAtIndex:3] doubleValue],[[data objectAtIndex:4] doubleValue], fabs(distance) ];
    [self logEvent:@"GEO_PUSH_INFO" andExtraInfo:logMessage];


	if (fabs(distance) <= [[data objectAtIndex:4] doubleValue]) {
        SingleMessageLoader *incomingMessage = [[SingleMessageLoader alloc] init];
		incomingMessage.delegate = [[UIApplication sharedApplication] delegate];
		[incomingMessage startLoadingDataForMessageWithID:[data objectAtIndex:1] andCompletionHandler:completionHandler];

	}
	else {
		completionHandler(UIBackgroundFetchResultNoData);
	}
}

- (void)getUnreadMessages {
	double now = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;

	// Avoid multiple simultaneous calls
	if (now - _lastPendingMessagesFetch <= 10000) {
		return;
	}

	PendingMessageDataLoader *incomingMessage = [[PendingMessageDataLoader alloc] init];
	incomingMessage.delegate = [[UIApplication sharedApplication] delegate];
	[incomingMessage startLoadingDataForMessages];
}

- (void)tagPendingMessagesFetch {
	_lastPendingMessagesFetch = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;
}

- (void)untagPendingMessagesFetch {
	_lastPendingMessagesFetch = 0;
}

#pragma mark Regular push notification

- (void)processPushNotificationForMsgID:(NSString *)msgID withHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	SingleMessageLoader *incomingMessage = [[SingleMessageLoader alloc] init];
	incomingMessage.delegate = [[UIApplication sharedApplication] delegate];
	[incomingMessage startLoadingDataForMessageWithID:msgID andCompletionHandler:completionHandler];
}

#pragma mark Region section

- (void)registerRegionForMsgID:(NSString *)msgID withMessageType:(NSString *)msgType{
	NSArray *data = [msgID componentsSeparatedByString:@":"];

	CLLocationCoordinate2D mapCoordinates;

    NSString* uuid = [data objectAtIndex:0];

	mapCoordinates.latitude = [[data objectAtIndex:1] doubleValue];
	mapCoordinates.longitude = [[data objectAtIndex:2] doubleValue];

	CLLocationDegrees radius = [[data objectAtIndex:3] doubleValue];
	if (radius > _messangiLM.maximumRegionMonitoringDistance) {
		radius = _messangiLM.maximumRegionMonitoringDistance;
	}

    NSString* trigger = [data objectAtIndex:4];
    NSString* name =[data objectAtIndex:5];

    //Opcional para Debug
    NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: registerRegionForMsgID: uuid: %@, latitude: %f,longitude: %f, radius: %f, trigger: %@",uuid,mapCoordinates.latitude,mapCoordinates.longitude,radius,trigger];
    [self logEvent:@"REGISTER_REGION" andExtraInfo:logMessage];

    // Necesario para BigData
    [self logEvent:msgType andExtraInfo:msgID withCampaignName:name];

	// Create the geographic region to be monitored.
	CLCircularRegion *geoRegion = [[CLCircularRegion alloc]
	                               initWithCenter:mapCoordinates
	                                       radius:radius
	                                   identifier:uuid];

    geoRegion.notifyOnEntry = NO;
    geoRegion.notifyOnExit = YES;
	if ([trigger isEqualToString:@"ENTER"]) {
		geoRegion.notifyOnExit = NO;
	}
	else if ([trigger isEqualToString:@"EXIT"]) {
		geoRegion.notifyOnEntry = NO;
	}
	NSMutableDictionary *dictionary = [NSMutableDictionary dictionary];
	[dictionary setObject:geoRegion forKey:@"Region"];
	[dictionary setObject:name forKey:@"Name"];
	[_geoFencesToAdd addObject:dictionary];
}

- (void)unregisterRegionWithMsgID:(NSString *)msgID andMessageType:(NSString *)msgType{
    //Opcional para Debug
    NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: unregisterRegionWithMsgID: msgID: %@",msgID];
    [self logEvent:@"UNREGISTER_REGION" andExtraInfo:logMessage];

    //TODO: El nombre de la campaña no viene incluido en el msgID
    // Necesario para BigData
    [self logEvent:msgType andExtraInfo:msgID withCampaignName:nil];

	[_geoFencesToDelete addObject:msgID];
}

- (void)locationManager:(CLLocationManager *)manager monitoringDidFailForRegion:(CLRegion *)region withError:(NSError *)error {
	// Needs work on actions and message to be communicated. This is a placeholder
}

- (void)locationManager:(CLLocationManager *)manager didStartMonitoringForRegion:(CLRegion *)region {
	// [[Messangi sharedInstance].messangiLM requestStateForRegion:region];
}

- (void)locationManager:(CLLocationManager *)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status {
	if (status == kCLAuthorizationStatusAuthorizedAlways) {
		[self refreshMonitoredRegionsToClosest];
	}
}

- (BOOL)isRegionInGeofenceList:(CLRegion *)region {
	CLRegion *geoRegion;
	for (int i = 0; i < _geoFencesList.count; i++) {
		geoRegion = _geoFencesList[i][@"Region"];
		if ([region.identifier isEqualToString:geoRegion.identifier]) {
			return true;
		}
	}
	return false;
}

- (void)locationManager:(CLLocationManager *)manager didEnterRegion:(CLRegion *)region {



    NSString* regionType = [self isRegionInGeofenceList:region] ? @"GEOFENCE":@"BEACON";


    NSString* campaignName = nil;
    if ([regionType  isEqual: @"BEACON"]) {
        for (NSDictionary *beacon in _beaconsList) {
            CLRegion *beaconRegion = beacon[@"Region"];
            if ([beaconRegion.identifier isEqualToString: region.identifier]) {
                campaignName = beacon[@"Name"];
                break;
            }
        }
    } else {
        for (NSDictionary *geoFence in _geoFencesList) {
            CLRegion *circularRegion = geoFence[@"Region"];
            if ([circularRegion.identifier isEqualToString: region.identifier]) {
                campaignName = geoFence[@"Name"];
                break;
            }
        }
    }


    //Opcional para Debug
    NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: didEnterRegion: regionType: %@, latitude: %f,longitude: %f, trigger: ENTER",regionType,manager.location.coordinate.latitude,manager.location.coordinate.longitude];
    [self logEvent:@"ENTER_REGION" andExtraInfo:logMessage];

    //Obligatorio para BigData
    NSString *eventType  = [regionType stringByAppendingString:@"_ENTER"];
    [self logEvent:eventType andExtraInfo:region.identifier withCampaignName:campaignName];


    LocationDataLoader *ldl = [[LocationDataLoader alloc] init];
    ldl.delegate = ldl;
    [ldl fireEventInRegion:region.identifier regionType:regionType withStatus:@"ENTER" atLatitude:manager.location.coordinate.latitude longitude:manager.location.coordinate.longitude andCompletionHandler:nil];


}

- (void)locationManager:(CLLocationManager *)manager didExitRegion:(CLRegion *)region {
    NSString* regionType = [self isRegionInGeofenceList:region] ? @"GEOFENCE": @"BEACON";

    NSString* campaignName = nil;
    if ([regionType  isEqual: @"BEACON"]) {
        for (NSDictionary *beacon in _beaconsList) {
            CLRegion *beaconRegion = beacon[@"Region"];
            if ([beaconRegion.identifier isEqualToString: region.identifier]) {
                campaignName = beacon[@"Name"];
                break;
            }
        }
    } else {
        for (NSDictionary *geoFence in _geoFencesList) {
            CLRegion *circularRegion = geoFence[@"Region"];
            if ([circularRegion.identifier isEqualToString: region.identifier]) {
                campaignName = geoFence[@"Name"];
                break;
            }
        }
    }

    //Opcional para Debug
    NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: didExitRegion: regionType: %@, latitude: %f,longitude: %f, trigger: EXIT",regionType,manager.location.coordinate.latitude,manager.location.coordinate.longitude];
    [self logEvent:@"EXIT_REGION" andExtraInfo:logMessage];


    //Obligatorio para BigData
    NSString *eventType  = [regionType stringByAppendingString:@"_EXIT"];
    [self logEvent:eventType andExtraInfo:region.identifier withCampaignName:campaignName];



    LocationDataLoader *ldl = [[LocationDataLoader alloc] init];
    ldl.delegate = ldl;
    [ldl fireEventInRegion:region.identifier regionType:regionType withStatus:@"EXIT" atLatitude:manager.location.coordinate.latitude longitude:manager.location.coordinate.longitude andCompletionHandler:nil];
}

- (NSSet *)monitoredGeoFences {
	return [_messangiLM monitoredRegions];
}

- (void)deviceSubscriberWithUniqueID {
	DeviceSubscriber *ds = [[DeviceSubscriber alloc] init];
	ds.delegate = self;
	[ds registerDeviceWithUniqueID];
}

- (void)registerDeviceToken:(NSData *)deviceToken {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSString *hexToken = [MessangiAPI NSDataToNSString:deviceToken];
	NSString *oldToken = [defaults objectForKey:@"devicePushID"];

	if (![oldToken isEqualToString:hexToken]) {
		[defaults setObject:hexToken forKey:@"devicePushID"];

		NSString *strUniqueIdentifier = [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString];

		[defaults setObject:strUniqueIdentifier forKey:@"deviceUniqueID"];

		[defaults synchronize];
		[self deviceSubscriberWithUniqueID];
	}
}

#pragma mark LoadBeacon delegate

- (void)loadBeaconsSuccess:(NSMutableArray *)updatedBeacons {
	// Cleans all the current Beacons and sets them up from scratch based on newly parsed list
	[_beaconsList removeAllObjects];
	[_beaconsList addObjectsFromArray:updatedBeacons];
}

- (void)loadBeaconsReturnedError:(NSString *)error {
	/*
	   NSString *title = NSLocalizedString(@"GeoFences", nil);
	   NSString *message = error;
	   NSString *cancelButtonTitle = NSLocalizedString(@"OK", nil);
	   UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:nil];
	   [alert show];
	 */
}

#pragma mark CheckGeoFences delegate

- (void)checkGeoFencesSuccess:(NSMutableArray *)updatedGeoFences {
	// Cleans all the current GeoFences and sets them up from scratch based on newly parsed list
	[_geoFencesList removeAllObjects];
	[_geoFencesToAdd removeAllObjects];
	[_geoFencesToDelete removeAllObjects];
	[_geoFencesList addObjectsFromArray:updatedGeoFences];
}

- (void)checkGeoFencesReturnedError:(NSString *)error {
	/*
	   NSString *title = NSLocalizedString(@"GeoFences", nil);
	   NSString *message = error;
	   NSString *cancelButtonTitle = NSLocalizedString(@"OK", nil);
	   UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:nil];
	   [alert show];
	 */
}

#pragma mark CheckSubscriptions delegate

- (void)checkSubscriptionsSuccess:(NSMutableArray *)updatedSubscriptions {
	for (Subscription *subscription in updatedSubscriptions) {
		[_subscriptionsList addObject:subscription];
	}
}

- (void)checkSubscriptionsReturnedError:(NSString *)error {
	/*
	   NSString *title = NSLocalizedString(@"Subscriptions", nil);
	   NSString *message = error;
	   NSString *cancelButtonTitle = NSLocalizedString(@"OK", nil);
	   UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:message delegate:self cancelButtonTitle:cancelButtonTitle otherButtonTitles:nil];
	   [alert show];
	 */
}

#pragma mark - iBeacon section

- (void)registerBeaconForMsgID:(NSString *)msgID withMessageType:(NSString *)msgType{
	// Create the beacon region to be monitored.
	NSArray *data = [msgID componentsSeparatedByString:@":"];

	// Create the iBeacon region to be monitored.
	NSUUID *uuid = [[NSUUID alloc] initWithUUIDString:data[1]];
	NSNumber *major = [NSNumber numberWithInteger:[data[2] integerValue]];
	NSNumber *minor = [NSNumber numberWithInteger:[data[3] integerValue]];

    //Opcional para Debug
    NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: registerBeaconForMsgID: uuid: %@, major: %@,minor: %@, identifier: %@",uuid,major,minor,data[0]];
    [self logEvent:@"REGISTER_BEACON" andExtraInfo:logMessage];

    // Necesario para BigData
    [self logEvent:msgType andExtraInfo:msgID withCampaignName:data[7]];

	CLBeaconRegion *region = [[CLBeaconRegion alloc] initWithProximityUUID:uuid major:[major intValue] minor:[minor intValue] identifier:data[0]];
	NSString *trigger = [data objectAtIndex:6];
	if ([trigger isEqualToString:@"ENTER"]) {
		region.notifyEntryStateOnDisplay = NO;
	}
	else if ([trigger isEqualToString:@"EXIT"]) {
		region.notifyOnEntry = NO;
	}
	NSInteger distanceKey;
	if ([data[5] isEqualToString:@"IMMEDIATE"]) {
		distanceKey = CLProximityImmediate;
	}
	else if ([data[5] isEqualToString:@"NEAR"]) {
		distanceKey = CLProximityNear;
	}
	else if ([data[5] isEqualToString:@"FAR"]) {
		distanceKey = CLProximityFar;
	}
	NSDictionary *beacon = @{ @"Name" : data[7],
		                      @"TimeStamp" : [NSDate distantPast],
		                      @"Region" : region };
	[_beaconsToAdd addObject:beacon];
}

- (void)unregisterBeaconWithMsgID:(NSString *)msgID andMessageType:(NSString *)msgType{
    //Opcional para Debug
    NSString* logMessage = [NSString stringWithFormat:@"MessangiSDK: unregisterBeaconForMsgID: msgID: %@",msgID];
    [self logEvent:@"UNREGISTER_BEACON" andExtraInfo:logMessage];

    //TODO: El nombre de la campaña no viene incluido en el msgID
    // Necesario para BigData
    [self logEvent:msgType andExtraInfo:msgID withCampaignName:nil];
    
	[_beaconsToDelete addObject:msgID];
}

#pragma  mark - Notifications and Setting

- (BOOL)didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler {
	NSLog(@"didReceiveRemoteNotification");
	NSString *msgID = [NSString stringWithFormat:@"%@", [userInfo objectForKey:@"msgId"]];
	NSString *msgType = [NSString stringWithFormat:@"%@", [userInfo objectForKey:@"type"]];


	BOOL consumed = false;

	if ([msgType isEqualToString:@"GET_LOC"]) {
        // GeoLocation
		consumed = true;
		[[Messangi sharedInstance] sendLastLocationForMsgID:msgID withHandler:completionHandler];
	}
	else if ([msgType isEqualToString:@"CRT_REGION"] || [msgType isEqualToString:@"UPDT_REGION"]) {
		// PUSH TYPE: CRT_REGION || PUSH TYPE: UPDT_REGION
		// PUSH CONTENT: REGION_ID:LATITUDE:LONGITUDE:RADIUS:TRIGGER:NAME
		consumed = true;
        [[Messangi sharedInstance] registerRegionForMsgID:msgID withMessageType:msgType];
		completionHandler(UIBackgroundFetchResultNewData);
	}
	else if ([msgType isEqualToString:@"DLT_REGION"]) {
		// PUSH TYPE: DLT_REGION
		// PUSH CONTENT: REGION_ID
		consumed = true;
		[[Messangi sharedInstance] unregisterRegionWithMsgID:msgID andMessageType:msgType];
		completionHandler(UIBackgroundFetchResultNewData);
	}
	else if ([msgType isEqualToString:@"TXT_MSG"] || [msgType isEqualToString:@"GEOFENCE"] || [msgType isEqualToString:@"BEACON"]){
        // PUSH TYPE: TXT_MSG, GEOFENCE, BEACON
        // PUSH CONTENT: PUSH_ID

        // TODO: Revisar como Agregar el nombre de la campaña a esto
        // Necesario para BigData
        [self logEvent:msgType andExtraInfo:msgID withCampaignName:nil];


		consumed = true;
		if ([msgID rangeOfString:@"GEOPUSH"].location != NSNotFound) {
			// [[LocationManager sharedInstance] setTypeOfGeoPush:1];
			if ([[Messangi sharedInstance] statusLocationServicesOK]) {
				[[Messangi sharedInstance] processGeoPushForMsgID:msgID withHandler:completionHandler];
			}
			else {
				completionHandler(UIBackgroundFetchResultNoData);
			}
		}
		else {
			//[[Messangi sharedInstance] processPushNotificationForMsgID:msgID withHandler:completionHandler];
			[[Messangi sharedInstance] getUnreadMessages];
		}
	}
	else if ([msgType isEqualToString:@"GEO_PUSH"] || [msgType isEqualToString:@"GEOPUSH"]) {
        // PUSH TYPE: GEO_PUSH || PUSH TYPE: GEOPUSH
        // PUSH CONTENT: GEOPUSH:PUSH_ID:LATITUDE:LONGITUDE:RADIUS

        // TODO: Revisar como Agregar el nombre de la campaña a esto
        // Necesario para BigData
        [self logEvent:msgType andExtraInfo:msgID withCampaignName:nil];

		consumed = true;
		if ([msgID rangeOfString:@"GEOPUSH"].location != NSNotFound) {
            NSLog(@"Location service OK :%s",[[Messangi sharedInstance] statusLocationServicesOK] ? "YES" : "NO");
			if ([[Messangi sharedInstance] statusLocationServicesOK]) {
				[[Messangi sharedInstance] processGeoPushForMsgID:msgID withHandler:completionHandler];
			}
			else {
				completionHandler(UIBackgroundFetchResultNoData);
			}
		}
	}
	else if ([msgType isEqualToString:@"CRT_BEACON"]) {
		// PUSH TYPE: CRT_BEACON
		// PUSH CONTENT: REGION_ID:UUID:MAJOR:MINOR:MANUFACTURER:DISTANCE:TYPE
		// DISTANCE: IMMEDIATE, NEAR, FAR
		// TYPE: ENTER, EXIT, ENTER-EXIT
		consumed = true;
		[[Messangi sharedInstance] registerBeaconForMsgID:msgID withMessageType:msgType];
		completionHandler(UIBackgroundFetchResultNewData);
	}
	else if ([msgType isEqualToString:@"DLT_BEACON"]) {
		// PUSH TYPE: DLT_BEACON
		// PUSH CONTENT: REGION_ID
		consumed = true;
		[[Messangi sharedInstance] unregisterBeaconWithMsgID:msgID andMessageType:msgType];
		completionHandler(UIBackgroundFetchResultNewData);
	}

	return consumed;
}

- (void)application:(UIApplication *)application didReceiveLocalNotification:(UILocalNotification *)notification {
	// If the application is in the foreground, we will notify the user of the region's state via an alert.
    NSString *cancelButtonTitle = NSLocalizedString(@"OK", @"Title for cancel button in local notification");
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:notification.alertBody message:nil preferredStyle:UIAlertControllerStyleAlert];
    
    // Create the action.
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelButtonTitle style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSLog(@"The application:didReceiveLocalNotification alert's cancel action occured.");
    }];
    
    // Add the action.
    [alertController addAction:cancelAction];
    UIWindow *window = [UIApplication sharedApplication].keyWindow;
    UIViewController *vc = [window rootViewController];
    
    [vc presentViewController:alertController animated:YES completion:nil];
}

- (void)setAppName:(NSString *)newName {
	_appName = newName;
}

- (void)setClientID:(NSString *)newClientID {
	_clientID = newClientID;
}

- (void)setApiClientPrivateKey:(NSString *)newPrivateKey {
	_apiClientPrivateKey = newPrivateKey;
}

- (void)setSubscriptionURL:(NSString *)newURL {
	_subscriptionURL = newURL;
}

- (void)setSubscriptionInstanceID:(NSString *)newID {
	_subscriptionInstanceID = newID;
}

- (void)logEvent:(NSString *)eventType andExtraInfo:(NSString *)extraInfo {
	[EventLogger logEvent:eventType andExtraInfo:extraInfo withCampaignName:nil];
}

- (void)logEvent:(NSString *)eventType andExtraInfo:(NSString *)extraInfo withCampaignName:(NSString *)campaignName{
    [EventLogger logEvent:eventType andExtraInfo:extraInfo withCampaignName:campaignName];
}

- (void)setLogAllEventsDisabled:(BOOL)disabled {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults setBool:disabled forKey:@"logeventsDisabled"];
}

- (BOOL)isLogAllEventsDisabled {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	BOOL logeventsDisabled = [defaults boolForKey:@"logeventsDisabled"];
	if (logeventsDisabled == NO) {
		logeventsDisabled = FALSE;
	}
	return logeventsDisabled;
}

- (void)setGetLocationDisabled:(BOOL)disabled {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults setBool:disabled forKey:@"getlocDisabled"];
}

- (BOOL)isGetLocationDisabled {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	BOOL getlocDisabled = [defaults boolForKey:@"getlocDisabled"];
	if (getlocDisabled == NO) {
		getlocDisabled = FALSE;
	}
	return getlocDisabled;
}

@end
