//
//  PendingMessageDataLoader.h
//  Messangi
//
//  Created by Robert Volante on 7/7/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "UnstoredIncomingMessage.h"
#import <UIKit/UIKit.h>


@protocol PendingMessageDataLoaderDelegate <NSObject>

/*!
 *  @brief  Abstract methods needed be implement by the client in PendingMessageDataLoaderDelegate Protocol
 */
@required
/*!
 *  @brief  Call when the message is finished loaded for the server
 *  @param message A UnstoredIncomingMessage returned by server
 */
- (void)messageLoadFinishedReturningMessage:(UnstoredIncomingMessage *)message;

/*!
 *  @brief  On Failed Data Loaded
 *  @param error An NSString with error send by Backend
 */
- (void)dataloaderReturnedError:(NSString *)error;
@end

/*!
 *  @brief  Interface for retrieve Push Messages from Messangi Backend
 */
@interface PendingMessageDataLoader : NSObject <NSXMLParserDelegate>
{
	NSString *statusCode;
	NSString *statusDescription;
	UnstoredIncomingMessage *currentMessage;

	NSMutableString *currentStringValue;
	NSString *currentProperty;
	id <PendingMessageDataLoaderDelegate> delegate;

	void (^completion)(UIBackgroundFetchResult);
}

/*!
 *  @brief  Status code 
 */
@property (strong) NSString *statusCode;

/*!
 *  @brief  Status Description
 */
@property (strong) NSString *statusDescription;

/*!
 *  @brief  A message Loading
 */
@property (strong) UnstoredIncomingMessage *currentMessage;

/*!
 *  @brief  Delegate that contain, messageLoadFinishedReturningMessage and dataloaderReturnedError methods
 */
@property (retain) id delegate;
@property (nonatomic, strong) NSMutableString *currentStringValue;
@property (nonatomic, strong) NSString *currentProperty;
@property (nonatomic, strong) void (^completion)(UIBackgroundFetchResult);

/*!
 *  @brief  when complete load this message from server
 */
@property BOOL toComplete;

/*!
 *  @brief  Make Request to retrieve a Data from the server
 */
- (void)startLoadingDataForMessages;

@end
