//
//  PendingMessageDataLoader.m
//  Messangi
//
//  Created by Robert Volante on 7/7/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "MessangiAPI.h"
#import "PendingMessageDataLoader.h"
#import "Messangi.h"

@implementation PendingMessageDataLoader

@synthesize currentMessage, statusCode, statusDescription, currentStringValue, currentProperty, delegate, completion, toComplete;

- (id)init {
	if (!self) {
		self = [super init];
	}
	return self;
}

- (void)startLoadingDataForMessages {
	[[Messangi sharedInstance] tagPendingMessagesFetch];

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	toComplete = NO;

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;

    NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%@/%@/%F", [defaults objectForKey:@"deviceUniqueID"], [Messangi sharedInstance].appName, @"IOS", [Messangi sharedInstance].clientID, timeStamp];

    NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

    NSString *urlString = [NSString stringWithFormat:@"%@/getPendingMessages/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString :%@",urlString);
    
    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"getPendingMessages: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"getPendingMessages: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
	if ([elementName isEqualToString:@"message"]) {
		// currentPerson is an ABPerson instance variable
		currentMessage = [[UnstoredIncomingMessage alloc] init];
		return;
	}

	currentProperty = elementName;

	return;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (!self.currentStringValue) {
		// currentStringValue is an NSMutableString instance variable
		self.currentStringValue = [[NSMutableString alloc] initWithCapacity:50];
	}
	[self.currentStringValue appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {

	if ([elementName isEqualToString:@"message"]) {
		return;
	}

	if ([elementName isEqualToString:@"id"]) {
		self.currentMessage.isRead = NO;
		self.currentMessage.receivedDate = [NSDate date];
		self.currentMessage.messageID = currentStringValue;
	}else if ([elementName isEqualToString:@"subject"]) {
		self.currentMessage.subject = currentStringValue;
	}else if ([elementName isEqualToString:@"clientId"]) {
		self.currentMessage.clientID = currentStringValue;
    }else if ([elementName isEqualToString:@"type"]){
        self.currentMessage.type = currentStringValue;
    }else if ([elementName isEqualToString:@"text"]) {
		self.currentMessage.text = currentStringValue;

		if (toComplete) {
			completion(UIBackgroundFetchResultNewData);
		}

		[self.delegate messageLoadFinishedReturningMessage:self.currentMessage];
		return;
    }

	currentStringValue = nil;
}

- (void)parserDidStartDocument:(NSXMLParser *)parser {
	//FLog(@"found file and started parsing");
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	if (toComplete) {
		completion(UIBackgroundFetchResultFailed);
	}
}

@end
