//
//  RegisterPhoneDataloader.m
//  Messangi
//
//  Created by Omar Diaz on 05/11/13.
//  Copyright (c) 2013 Ogangi. All rights reserved.
//

#import "MessangiAPI.h"
#import "RegisterPhoneDataloader.h"
#import "Messangi.h"

@implementation RegisterPhoneDataloader

@synthesize currentStringValue, currentProperty, delegate, successful, message, operation;

- (id)init {
	if (!self) {
		self = [super init];
	}

	return self;
}

- (void)registerDeviceWithAreaCode:(NSString *)areaCode andPhoneNumber:(NSString *)phoneNumber {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	operation = 0;
	//registerPhone/{deviceId}/{appName}/{platform}/{phoneNumber}/{clientId}/{timeStamp}/{signature}

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;

    NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%@/%@%@/%@/%F", [defaults objectForKey:@"deviceUniqueID"], [Messangi sharedInstance].appName, @"IOS", areaCode, phoneNumber, [Messangi sharedInstance].clientID, timeStamp];

    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

	NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

	NSString *urlString = [NSString stringWithFormat:@"%@/registerPhone/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString = %@",urlString);

    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"registerPhone: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                         }
                                          else
                                          {
                                              NSLog(@"registerPhone: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)activatePhoneWithCode:(NSString *)code {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	operation = 1;
	//activatePhone/{deviceId}/{appName}/{platform}/{activationCode}/{clientId}/{timeStamp}/{signature}

	double timeStamp = ((double)[[NSDate date]timeIntervalSince1970]) * 1000;

    NSString *stringToSign = [NSString stringWithFormat:@"%@/%@/%@/%@/%@/%F", [defaults objectForKey:@"deviceUniqueID"], [Messangi sharedInstance].appName, @"IOS", code, [Messangi sharedInstance].clientID, timeStamp];
  
    NSString *encodedStringToSign = [stringToSign stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLPathAllowedCharacterSet]];

	NSString *signedString = [MessangiAPI encodeString:stringToSign usingKey:[Messangi sharedInstance].apiClientPrivateKey];

	NSString *urlString = [NSString stringWithFormat:@"%@/activatePhone/%@/%@", PREFIX, encodedStringToSign, signedString];
    NSLog(@"urlString = %@",urlString);

    NSURLSession* mySession = [NSURLSession sharedSession];
    NSURLSessionTask* task = [mySession dataTaskWithURL:[NSURL URLWithString:urlString]
                                      completionHandler:^(NSData* data,
                                                          NSURLResponse* response, NSError* error) {
                                          if (error == nil)
                                          {
                                              NSLog(@"activatePhone: completed successfully");
                                              NSXMLParser *parser = [[NSXMLParser alloc] initWithData:data];
                                              parser.delegate = self;
                                              [parser parse];
                                          }
                                          else
                                          {
                                              NSLog(@"activatePhone: completed with error: %@", [error localizedDescription]);
                                          }
                                      }];
    [task resume];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
	currentProperty = elementName;

	return;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (!self.currentStringValue) {
		// currentStringValue is an NSMutableString instance variable
		self.currentStringValue = [[NSMutableString alloc] initWithCapacity:50];
	}
	[self.currentStringValue appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	//NSLog(@"parser : %@,%@,%@",elementName,namespaceURI,qName);


	if (([elementName isEqualToString:@"transactionResult"])) {
		//NSLog(@"Operation : %d",operation);
		switch (operation) {
			case 0:
				//NSLog(@"[self.delegate RegisterPhoneSucceeded:%d withMessage:%@]",self.successful,self.message);
				[self.delegate RegisterPhoneSucceeded:self.successful withMessage:self.message];
				break;

			case 1:
				//NSLog(@"[self.delegate ActivatePhoneSucceeded:%d withMessage:%@]",self.successful,self.message);
				[self.delegate ActivatePhoneSucceeded:self.successful withMessage:self.message];
				break;

			default:
				break;
		}

		return;
	}

	NSString *prop = [self currentProperty];

	if ([prop isEqualToString:@"message"]) {
		self.message = currentStringValue;
		//NSLog(@"message : %@",[self message]);
	}

	if ([prop isEqualToString:@"successful"]) {
		if ([currentStringValue isEqual:@"true"])
			self.successful = YES;
		else
			self.successful = NO;
		//NSLog(@"successful : %d",[self successful]);
	}
	currentStringValue = nil;
}

- (void)parserDidStartDocument:(NSXMLParser *)parser {
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	NSString *errorString = [NSString stringWithFormat:NSLocalizedString(@"System Error", nil), [parseError code]];
	switch (operation) {
		case 0:
			[delegate RegisterPhoneReturnedError:errorString];
			break;

		case 1:
			[delegate ActivatePhoneReturnedError:errorString];
			break;
	}
}

@end
