//
//  Subscription.m
//  MessangiSDK
//
//  Created by Robert Volante on 7/14/14.
//  Copyright (c) 2014 Ogangi. All rights reserved.
//

#import "Subscription.h"

@implementation Subscription

+ (NSString *)documentsSubPath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDir = [[paths objectAtIndex:0] stringByAppendingPathComponent:@"MessangiSubscriptions"];
	if (![[NSFileManager defaultManager] fileExistsAtPath:documentsDir]) {
		[[NSFileManager defaultManager] createDirectoryAtPath:documentsDir withIntermediateDirectories:NO attributes:nil error:nil]; //Create folder
	}
	return documentsDir;
}

- (BOOL)isSaved {
	NSMutableArray *temp = [NSMutableArray arrayWithArray:[Subscription listSubDirectory]];
	for (int i = 0; i < temp.count; i++) {
		if ([[Subscription subIDfromFileName:temp[i]] isEqualToString:_subID]) {
			return true;
		}
	}
	return false;
}

+ (NSArray *)listSubDirectory {
	NSURL *urlFilePath = [NSURL fileURLWithPath:[Subscription documentsSubPath]];
	NSArray *directoryContent = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:urlFilePath includingPropertiesForKeys:NULL options:NSDirectoryEnumerationSkipsHiddenFiles error:NULL];

	NSMutableArray *onlyLastPathComponent = [NSMutableArray arrayWithCapacity:directoryContent.count];
	for (NSURL *fileURL in directoryContent) {
		[onlyLastPathComponent addObject:[fileURL lastPathComponent]];
	}
	return onlyLastPathComponent;
}

+ (NSString *)subIDfromFileName:(NSString *)messageFileName {
	NSArray *components = [messageFileName componentsSeparatedByString:@"**"];
	if (components.count) {
		return components[0];
	}
	return NSLocalizedString(@"no MessageID", @"Placeholder to notify that a messageID for the message could not be determined");
}

- (NSMutableDictionary *)subToDictonaryEntry {
	NSMutableDictionary *dict = [NSMutableDictionary dictionary];
	dict[@"SubID"] = _subID;
	dict[@"SubName"] = _name;
	return dict;
}

- (void)writeSubToFile {
	NSDictionary *message = [self subToDictonaryEntry];

	NSString *messageFile = [[Subscription documentsSubPath] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@**%@.plist", _subID, _name]];

	[message writeToFile:messageFile atomically:YES];
}

- (id)initFromFile:(NSString *)fileName {
	self = [super init];
	if (self) {
		NSString *completeFilePath = [[Subscription documentsSubPath] stringByAppendingPathComponent:fileName];
		if ([[NSFileManager defaultManager] fileExistsAtPath:completeFilePath]) {
			NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:completeFilePath];
			_subID = [dict objectForKey:@"SubID"];
			_name = [dict objectForKey:@"SubName"];
			return self;
		}
	}
	return nil;
}

- (BOOL)deleteSub {
	NSURL *urlFilePath = [NSURL fileURLWithPath:[Subscription documentsSubPath]];
	NSString *fileName = [NSString stringWithFormat:@"%@**%@.plist", _subID, _name];
	NSURL *file = [urlFilePath URLByAppendingPathComponent:fileName];

	BOOL success = [[NSFileManager defaultManager] removeItemAtURL:file error:NULL];
	if (!success) {
		NSLog(@"Error removing file at path: %@", fileName);
	}
	return success;
}

@end
