//
//  UnstoredIncomingMessage.m
//  ego
//
//  Created by Jose V. Salazar on 12/23/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "UnstoredIncomingMessage.h"

@implementation UnstoredIncomingMessage

- (BOOL)saveMessageInDatabase {
	return YES;
}

- (BOOL)saveMessageInWaitingTable:(UnstoredIncomingMessage *)im {
	return YES;
}

+ (NSArray *)retreiveMessagesFromTable:(NSString *)tableName {
	return nil;
}

+ (NSArray *)retreiveMessagesFromTable:(NSString *)tableName olderThan:(NSTimeInterval)oldDate {
	return nil;
}

+ (BOOL)deleteMessageWithID:(NSString *)messageID {
	return YES;
}

+ (BOOL)deleteWaitingMessageWithID:(NSString *)messageID {
	return YES;
}

+ (NSString *)documentsInboxPath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDir = [[paths objectAtIndex:0] stringByAppendingPathComponent:@"MessangiInbox"];
	if (![[NSFileManager defaultManager] fileExistsAtPath:documentsDir]) {
		[[NSFileManager defaultManager] createDirectoryAtPath:documentsDir withIntermediateDirectories:NO attributes:nil error:nil]; //Create folder
	}
	return documentsDir;
}

+ (NSString *)messageIDfromFileName:(NSString *)messageFileName {
	NSArray *components = [messageFileName componentsSeparatedByString:@"**"];
	if (components.count) {
		return components[0];
	}
	return NSLocalizedString(@"no MessageID", @"Placeholder to notify that a messageID for the message could not be determined");
}

+ (NSString *)messageSubjectFromFileName:(NSString *)messageFileName {
	NSArray *components = [messageFileName componentsSeparatedByString:@"**"];
	if (components.count > 1) {
		return components[1];
	}
	return NSLocalizedString(@"no Subject", @"Placeholder to notify that a Subject for the message could not be determined");
}

+ (NSDate *)messageReceivedDateFromFileName:(NSString *)messageFileName {
	NSArray *components = [messageFileName componentsSeparatedByString:@"**"];
	if (components.count >= 8) {
		NSDateComponents *comps = [[NSDateComponents alloc] init];
		[comps setMonth:[components[2] integerValue]];
		[comps setDay:[components[3] integerValue]];
		[comps setYear:[components[4] integerValue]];
		[comps setHour:[components[5] integerValue]];
		[comps setMinute:[components[6] integerValue]];
		[comps setSecond:[components[7] integerValue]];
		NSCalendar *gregorian = [[NSCalendar alloc]
		                         initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
		NSDate *date = [gregorian dateFromComponents:comps];
		return date;
	}
	return nil;
}

+ (BOOL)messageReadStatusFromFileName:(NSString *)messageFileName {
	NSArray *components = [messageFileName componentsSeparatedByString:@"**"];
	NSString *status = components[8];
	return [status isEqualToString:@"R.plist"];
}

- (NSMutableDictionary *)messageToDictonaryEntry {
	NSMutableDictionary *dict = [NSMutableDictionary dictionary];
	dict[@"MessageID"] = _messageID;
	dict[@"Subject"] = _subject;
	dict[@"Text"] = _text;
	dict[@"ReceivedDate"] = _receivedDate;
    dict[@"Type"] = _type;
	return dict;
}

- (void)writeMessageToFile {
	NSDictionary *message = [self messageToDictonaryEntry];
	NSDateComponents *components = [[NSCalendar currentCalendar] components:NSCalendarUnitDay | NSCalendarUnitMonth | NSCalendarUnitYear | NSCalendarUnitHour | NSCalendarUnitMinute |  NSCalendarUnitSecond | NSCalendarUnitTimeZone fromDate:_receivedDate];
	NSInteger day = [components day];
	NSInteger month = [components month];
	NSInteger year = [components year];
	NSInteger hour = [components hour];
	NSInteger minute = [components minute];
	NSInteger second = [components second];

	// the last component is U for unread file and R for read file
	NSString *messageFile = [[UnstoredIncomingMessage documentsInboxPath] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@**%@**%li**%li**%li**%li**%li**%li**%@.plist", _messageID, _subject, (long)month, (long)day, (long)year, (long)hour, (long)minute, (long)second, @"U"]];

	[message writeToFile:messageFile atomically:YES];
}

- (id)initFromFile:(NSString *)fileName {
	self = [super init];
	if (self) {
		NSString *completeFilePath = [[UnstoredIncomingMessage documentsInboxPath] stringByAppendingPathComponent:fileName];
		if ([[NSFileManager defaultManager] fileExistsAtPath:completeFilePath]) {
			NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:completeFilePath];
			_messageID = [dict objectForKey:@"MessageID"];
			_clientID = [dict objectForKey:@"ClientID"];
			_clientName = [dict objectForKey:@"ClientName"];
			_serviceID = [dict objectForKey:@"ServiceID"];
			_serviceName = [dict objectForKey:@"ServiceName"];
			_iconImage = [dict objectForKey:@"IconImage"];
			_subject = [dict objectForKey:@"Subject"];

            @try {
                _type = [dict objectForKey:@"Type"];
            }
            @catch (NSException *exception) {
                NSLog(@"%@", exception.reason);
                _type = @"TEXT";
            }

			_text = [dict objectForKey:@"Text"];
			_messageURL = [dict objectForKey:@"MessageURL"];
			NSNumber *delivered = [dict objectForKey:@"IsDelivered"];
			_isDelivered = [delivered boolValue];
			NSNumber *read = [dict objectForKey:@"IsRead"];
			_isRead = [read boolValue];
			_receivedDate = [dict objectForKey:@"ReceivedDate"];
			_category = [dict objectForKey:@"Category"];
			return self;
		}
	}
	return nil;
}

+ (BOOL)messageFileNameMarkRead:(NSString *)messageFileName {
	NSURL *urlFilePath = [NSURL fileURLWithPath:[UnstoredIncomingMessage documentsInboxPath]];
	NSURL *fromPath = [urlFilePath URLByAppendingPathComponent:messageFileName];

	NSString *newFileName = [NSString stringWithString:[messageFileName stringByReplacingOccurrencesOfString:@"**U.plist" withString:@"**R.plist"]];

	NSURL *newPath = [urlFilePath URLByAppendingPathComponent:newFileName];

	BOOL success = [[NSFileManager defaultManager] moveItemAtURL:fromPath toURL:newPath error:nil];
	return success;
}

- (id)initWithMessageID:(NSString *)messageID withSubject:(NSString *)subject withReceivedDate:(NSDate *)receivedDate {
	self = [super init];
	if (self) {
		_messageID = messageID;
		_clientID = @"ClientID";
		_clientName = @"ClientName";
		_serviceID = @"ServiceID";
		_serviceName = @"ServiceName";
		_iconImage = @"IconImage";
		_subject = subject;
		_type = @"Type";
		_text = @"Text";
		_messageURL = @"MessageURL";
		_isDelivered = YES;
		_isRead = YES;
		_receivedDate = receivedDate;
		_category = @"Category";
		return self;
	}
	return nil;
}

+ (BOOL)deleteMessageFileName:(NSString *)fileName {
	NSURL *urlFilePath = [NSURL fileURLWithPath:[UnstoredIncomingMessage documentsInboxPath]];
	NSURL *file = [urlFilePath URLByAppendingPathComponent:fileName];

	BOOL success = [[NSFileManager defaultManager] removeItemAtURL:file error:NULL];
	if (!success) {
		NSLog(@"Error removing file at path: %@", fileName);
	}
	return success;
}

@end
