package com.ogangi.messangi.android.demo;

import android.app.Notification;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.location.Location;
import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.os.Parcelable;
import android.preference.PreferenceManager;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.TabLayout;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import com.ogangi.messangi.android.demo.beacons.beaconFragment;
import com.ogangi.messangi.android.demo.geofences.MapsActivity;
import com.ogangi.messangi.android.demo.geofences.geofenceFragment;
import com.ogangi.messangi.android.demo.inbox.NotificationFragment;
import com.ogangi.messangi.android.demo.inbox.webviewActivity;
import com.ogangi.messangi.android.sdk.Messangi;
import com.ogangi.messangi.android.sdk.MessangiListener;
import com.ogangi.messangi.android.sdk.MessangiService;
import com.ogangi.messangi.android.sdk.vo.BeaconVO;
import com.ogangi.messangi.android.sdk.vo.GeoFenceVO;
import com.ogangi.messangi.android.sdk.vo.MessageVO;

import java.util.List;


public class MainActivity extends AppCompatActivity implements NotificationFragment.OnFragmentInteractionListener, geofenceFragment.OnFragmentInteractionListener, beaconFragment.OnFragmentInteractionListener, MessangiListener {
    /**
     * The {@link android.support.v4.view.PagerAdapter} that will provide
     * fragments for each of the sections. We use a
     * {@link FragmentPagerAdapter} derivative, which will keep every
     * loaded fragment in memory. If this becomes too memory intensive, it
     * may be best to switch to a
     * {@link android.support.v4.app.FragmentStatePagerAdapter}.
     */
    private SectionsPagerAdapter mSectionsPagerAdapter;
    private FloatingActionButton fab;
    private View.OnClickListener goMyMap = new View.OnClickListener() {
        @Override
        public void onClick(View view) {
            Location myLocation;
            try {
                myLocation = Messangi.getInstance().getService().getLastKnownLocation();
            } catch (NullPointerException _e) {
                myLocation = null;
            }
            if (myLocation != null) {
                Intent mapIntent = new Intent(getApplicationContext(), MapsActivity.class);
                mapIntent.putExtra("LATITUDE", myLocation.getLatitude());
                mapIntent.putExtra("LONGITUDE", myLocation.getLongitude());
                mapIntent.putExtra("MYFOCUS", true);
                startActivity(mapIntent);
            }
        }
    };
    private View.OnClickListener deleteNotifications = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            if(mViewPager != null) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        SectionsPagerAdapter adapter = ((SectionsPagerAdapter) mViewPager.getAdapter());
                        NotificationFragment nFragment = (NotificationFragment) adapter.getFragment(0);
                        if (nFragment != null) {
                            nFragment.deleteNotifications();
                        }
                    }

                });
            }
        }
    };

    /**
     * The {@link ViewPager} that will host the section contents.
     */
    private ViewPager mViewPager;
    private Parcelable pageInfo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        PreferenceManager.setDefaultValues(this, R.xml.pref_general, false);
        PreferenceManager.setDefaultValues(this, R.xml.pref_data_sync, false);
        PreferenceManager.setDefaultValues(this, R.xml.pref_location, false);
        PreferenceManager.setDefaultValues(this, R.xml.pref_notification, false);


        setContentView(R.layout.activity_main);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        // Create the adapter that will return a fragment for each of the three
        // primary sections of the activity.
        mSectionsPagerAdapter = new SectionsPagerAdapter(getSupportFragmentManager());

        // Set up the ViewPager with the sections adapter.
        mViewPager = (ViewPager) findViewById(R.id.container);
        mViewPager.setAdapter(mSectionsPagerAdapter);
        //mViewPager.setCurrentItem(1);

        TabLayout tabLayout = (TabLayout) findViewById(R.id.tabs);
        tabLayout.setupWithViewPager(mViewPager);



        fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(goMyMap);

        /**  Forma valida pero altenativa (no es lo comun en android)
        // Messangi Credentials
        Messangi.getInstance().setAppName("Messangi");
        Messangi.getInstance().setApiClientPrivateKey("wzuuzVeVA4hR9Xc8kWQB");
        Messangi.getInstance().setClientId("D79BFFVx2TjXcJrrnktB");

        // GCM Credentials
        Messangi.getInstance().setGcmApiKey("AIzaSyDoThF8Mbagpnt4sDUy5ENf-GwfbU4zoEc");
        Messangi.getInstance().setGcmProjectId("554154250704");

        // Subscription manager
        Messangi.getInstance().setSubscriptionURL("https://www.messangi.com/messangi_mmc/rest/api");
        Messangi.getInstance().setSubscriptionInstanceId("8");

        **/
        Messangi.getInstance().init(this);
        //Messangi.getInstance().useTestingAPI();
        Messangi.getInstance().addMessangiListener(this);
        Messangi.getInstance().registerDialog(this, this);
    }

    @Override
    protected void onStart() {
        super.onStart();
        Messangi.getInstance().init(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        Messangi.getInstance().bindService();
    }

    @Override
    protected void onPause() {
        Messangi.getInstance().unBindService();
        super.onPause();
    }

    @Override
    public void postInit() {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mViewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
                    @Override
                    public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

                    }

                    @Override
                    public void onPageSelected(int position) {
                        SectionsPagerAdapter adapter = ((SectionsPagerAdapter) mViewPager.getAdapter());
                        Fragment tmp = adapter.getFragment(position);
                        if(null != tmp) {
                            switch (position) {
                                case 0:
                                    ((NotificationFragment) tmp).updateDataSet();
                                    break;
                                case 1:
                                    ((geofenceFragment) tmp).updateDataSet();
                                    break;
                                case 2:
                                    ((beaconFragment) tmp).updateDataSet();
                                    break;
                            }
                        }
                    }

                    @Override
                    public void onPageScrollStateChanged(int state) {

                    }
                });

            }
        });
    }

    @Override
    public void pushReceived(MessageVO message) {
        Messangi.getInstance().logEvent(this, "PUSHRECEIVED", message.getMessageContent(),message.getMessageTitle());
        if(mViewPager != null) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    SectionsPagerAdapter adapter = ((SectionsPagerAdapter) mViewPager.getAdapter());
                    NotificationFragment nFragment = (NotificationFragment) adapter.getFragment(0);
                    if (nFragment != null) {
                        nFragment.updateDataSet();
                    }
                }

            });
        }
    }

    @Override
    public void updateFencesStatus(List<com.google.android.gms.location.Geofence> list, int i, Location location) {

    }

    @Override
    public void onLocationChange(final Location location) {
        if(mViewPager != null) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    SectionsPagerAdapter adapter = ((SectionsPagerAdapter) mViewPager.getAdapter());
                    geofenceFragment gFragment = ((geofenceFragment) adapter.getFragment(1));
                    if (gFragment != null) {
                        gFragment.updateDistances(location);
                    }
                }
            });
        }
    }

    @Override
    public void onGeofenceUpdate(String type, String geoFenceID) {
        if(mViewPager != null) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    SectionsPagerAdapter adapter = ((SectionsPagerAdapter) mViewPager.getAdapter());
                    geofenceFragment gFragment = ((geofenceFragment) adapter.getFragment(1));
                    if (gFragment != null) {
                        gFragment.updateDataSet();
                    }
                }
            });
        }
    }

    @Override
    public void onBeaconUpdate(String type, String beaconID) {
        if(mViewPager != null) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    SectionsPagerAdapter adapter = ((SectionsPagerAdapter) mViewPager.getAdapter());
                    beaconFragment bFragment = ((beaconFragment) adapter.getFragment(2));
                    if (bFragment != null) {
                        bFragment.updateDataSet();
                    }
                }
            });
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            Intent settingsIntent = new Intent(this, SettingsActivity.class);
            startActivity(settingsIntent);
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onFragmentInteraction(String type, String id) {
        if(type.equals("Notification")) {
            Intent detailIntent = new Intent(this, webviewActivity.class);
            detailIntent.putExtra("NOTIFICATION_ID", id);
            startActivity(detailIntent);
        }
    }

    @Override
    public void deleteAction(boolean active) {
        if(active){
            fab.setImageResource(R.drawable.ic_delete_white_24dp);
            fab.setOnClickListener(deleteNotifications);
        }else{
            fab.setImageResource(android.R.drawable.ic_dialog_map);
            fab.setOnClickListener(goMyMap);
        }
    }


    /**
     * A {@link FragmentPagerAdapter} that returns a fragment corresponding to
     * one of the sections/tabs/pages.
     */
    public class SectionsPagerAdapter extends FragmentStatePagerAdapter {

        private NotificationFragment nFragment = null;
        private geofenceFragment gFragment = null;
        private beaconFragment bFragment = null;

        public SectionsPagerAdapter(FragmentManager fm) {
            super(fm);
            nFragment = null;
            gFragment = null;
            bFragment = null;
        }

        @Override
        public Fragment getItem(int position) {
            // getItem is called to instantiate the fragment for the given page.
            //Fragment actualFragment = null;
            switch (position){
                case 0:
                    nFragment = NotificationFragment.newInstance(getApplicationContext());
                    return nFragment;
                case 1:
                    gFragment = geofenceFragment.newInstance(getApplicationContext());
                    return gFragment;
                case 2:
                    bFragment =  beaconFragment.newInstance(getApplicationContext());
                    return bFragment;
            }
            return null;
            //mFragments.add(position,actualFragment);
            //return actualFragment;
        }



        @Override
        public int getCount() {
            // Show 3 total pages.
            return 3;
        }

        @Override
        public CharSequence getPageTitle(int position) {
            switch (position) {
                case 0:
                    return "Inbox";
                case 1:
                    return "Geofences";
                case 2:
                    return "Beacons";
            }
            return null;
        }

        public Fragment getFragment(int index) {
            switch (index){
                case 0:
                    return nFragment;
                case 1:
                    return gFragment;
                case 2:
                    return bFragment;
            }
            return null;
        }
    }
}
