package com.ogangi.messangi.android.demo;

import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.drawable.BitmapDrawable;
import android.net.Uri;
import android.preference.PreferenceManager;
import android.support.v4.app.NotificationCompat;
import android.support.v4.app.TaskStackBuilder;
import android.text.Html;

import com.ogangi.messangi.android.demo.inbox.webviewActivity;
import com.ogangi.messangi.android.sdk.vo.MessageVO;

import java.util.Random;

/**
 * Notification Engine for Show Notifications
 */
public class PushNotification extends BroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {

        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);

        if(!pref.getBoolean("notifications_new_message",true)){
            return;
        }

        String json = intent.getStringExtra("message");
        MessageVO message = MessageVO.fromJson(json);
        Intent resultIntent = new Intent(context, webviewActivity.class);

        resultIntent.putExtra("NOTIFICATION_ID", message.getMessageIDAux());
        resultIntent.putExtra("TITLE", message.getMessageTitle());
        resultIntent.putExtra("HTML", message.getMessageContent());

        TaskStackBuilder stackBuilder = TaskStackBuilder.create(context);
        // Adds the back stack
        stackBuilder.addParentStack(webviewActivity.class);
        // Adds the Intent to the top of the stack
        stackBuilder.addNextIntent(resultIntent);
        // Gets a PendingIntent containing the entire back stack
        PendingIntent resultPendingIntent =
                stackBuilder.getPendingIntent(0, PendingIntent.FLAG_UPDATE_CURRENT);



        NotificationCompat.Builder mBuilder =
                new NotificationCompat.Builder(context)
                        .setSmallIcon(R.mipmap.ic_launcher)
                        .setLargeIcon((((BitmapDrawable) context.getResources().getDrawable(R.mipmap.ic_launcher)).getBitmap()))
                        .setContentTitle(message.getMessageTitle())
                        .setContentText(Html.fromHtml(message.getMessageContent()).toString())
                        .setContentIntent(resultPendingIntent)
                        .setAutoCancel(true)
                        .setVisibility(NotificationCompat.VISIBILITY_PUBLIC);


        mBuilder.setSound(Uri.parse(pref.getString("notifications_new_message_ringtone","content://settings/system/notification_sound")));
        if(pref.getBoolean("notifications_new_message_vibrate",true)){
            mBuilder.setVibrate(new long[] { 1000, 1000, 1000, 1000, 1000 });
        }else{
            mBuilder.setVibrate(new long[] {0});
        }


        // Sets an ID for the notification
        int mNotificationId = 001;
        if(!pref.getBoolean("only_last_notification",true)){
            mNotificationId = new Random().nextInt(10000);
        }


        // Gets an instance of the NotificationManager service
        NotificationManager mNotifyMgr =
                (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);

        // Builds the notification and issues it.
        mNotifyMgr.notify(mNotificationId, mBuilder.build());

    }
}
