package com.ogangi.messangi.android.demo.geofences;

import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.CircleOptions;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.ogangi.messangi.android.demo.R;
import com.ogangi.messangi.android.demo.geofences.geofence.Geofence;

public class MapsActivity extends AppCompatActivity implements OnMapReadyCallback {

    private GoogleMap mMap;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maps);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);




        // Obtain the SupportMapFragment and get notified when the map is ready to be used.
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);
    }


    /**
     * Manipulates the map once available.
     * This callback is triggered when the map is ready to be used.
     * This is where we can add markers or lines, add listeners or move the camera. In this case,
     * we just add a marker near Sydney, Australia.
     * If Google Play services is not installed on the device, the user will be prompted to install
     * it inside the SupportMapFragment. This method will only be triggered once the user has
     * installed Google Play services and returned to the app.
     */
    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;

        double lng = getIntent().getDoubleExtra("LONGITUDE", Double.NaN);
        double lat = getIntent().getDoubleExtra("LATITUDE", Double.NaN);
        float  rad = getIntent().getFloatExtra("RADIUS", 0.0f);

        boolean myFocus = getIntent().getBooleanExtra("MYFOCUS",false);

        if(myFocus){
            LatLng myLoc = new LatLng(lat,lng);
            mMap.addMarker(new MarkerOptions().position(myLoc).title("My Position"));
            for (Geofence.GeofenceItem geofence : Geofence.Geofences){
                makeCircle(geofence.latitude,geofence.longitude,geofence.radius,geofence.type);
            }
        }else {
            CircleOptions options = new CircleOptions();
            options.center(new LatLng(lat, lng));
            options.radius(rad);
            options.fillColor(0X3300FFFF);
            mMap.addCircle(options);
        }
        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(lat, lng), 13));
    }


    public void makeCircle (double lat, double lng, float rad, String type){
        CircleOptions options = new CircleOptions();
        options.center(new LatLng(lat, lng));
        options.radius(rad);
        options.fillColor(type.equals("ENTER") ? 0x33FF00FF: type.equals("EXIT") ? 0x3300FFFF: 0x33FFFF00);
        mMap.addCircle(options);
    }

}