package com.ogangi.messangi.android.demo.geofences;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.view.View;
import android.widget.ListView;


import com.ogangi.messangi.android.demo.geofences.geofence.Geofence;
import com.ogangi.messangi.android.sdk.Messangi;
import com.ogangi.messangi.android.sdk.MessangiService;
import com.ogangi.messangi.android.sdk.vo.GeoFenceVO;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * A fragment representing a list of Items.
 * <p/>
 * <p/>
 * Activities containing this fragment MUST implement the {@link OnFragmentInteractionListener}
 * interface.
 */
public class geofenceFragment extends ListFragment {
    private static Context context;
    private OnFragmentInteractionListener mListener;
    private GeofenceAdapter mAdapter;

    public static geofenceFragment newInstance(Context ctx) {
        context = ctx;
        return new geofenceFragment();
    }

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public geofenceFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mAdapter = new GeofenceAdapter(getActivity(),Geofence.Geofences);
        setListAdapter(mAdapter);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        this.setEmptyText("No geofence registered yet, please go to mesangi.com to do some Geofences");
        updateDataSet();
    }

    public void updateDataSet() {
        List<GeoFenceVO> geofences = null;
        try {
            geofences = Messangi.getInstance().getService().getGeofences();
        } catch (NullPointerException _n) {
            _n.printStackTrace();
            geofences = new ArrayList<>();
        }
        for (GeoFenceVO geofence : geofences) {
            Geofence.GeofenceItem storageGeofence = Geofence.getGeofence(geofence.getId());
            if (storageGeofence == null) {
                Geofence.GeofenceItem tmp = new Geofence.GeofenceItem(
                        geofence.getId(),
                        geofence.getName(),
                        geofence.getRadius(),
                        geofence.getLatitude(),
                        geofence.getLongitude(),
                        geofence.getEventType()
                );
                Geofence.addItem(tmp);
            } else {
                storageGeofence.title    = geofence.getName();
                storageGeofence.radius   = geofence.getRadius();
                storageGeofence.latitude = geofence.getLatitude();
                storageGeofence.longitude= geofence.getLongitude();
                storageGeofence.type     = geofence.getEventType();
            }
        }
        Location location = null;
        try {
            location = Messangi.getInstance().getService().getLastKnownLocation();
        } catch (NullPointerException _n) {
            _n.printStackTrace();
        }
        this.updateDistances(location);
    }

    public void updateDistances(Location location){
        if(location!=null){
            Geofence.updateDistances(location.getLatitude(),location.getLongitude());
            mAdapter.sort(new Comparator<Geofence.GeofenceItem>() {
                @Override
                public int compare(Geofence.GeofenceItem geoA, Geofence.GeofenceItem geoB) {
                    return Float.compare(geoA.distance, geoB.distance);
                }
            });
        }
        mAdapter.notifyDataSetChanged();
    }

    public void notifyDataChange(){
        mAdapter.notifyDataSetChanged();
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mListener = (OnFragmentInteractionListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);

        Geofence.GeofenceItem geofence = Geofence.getGeofence(position);

        Intent mapIntent = new Intent(getActivity(), MapsActivity.class);
        mapIntent.putExtra("LATITUDE", geofence.latitude);
        mapIntent.putExtra("LONGITUDE", geofence.longitude);
        mapIntent.putExtra("RADIUS",geofence.radius);
        startActivity(mapIntent);

        if (null != mListener) {
            // Notify the active callbacks interface (the activity, if the
            // fragment is attached to one) that an item has been selected.
            mListener.onFragmentInteraction("Geofence",geofence.id);
        }


    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     */
    public interface OnFragmentInteractionListener {
        void onFragmentInteraction(String type, String id);
    }

}


