package com.ogangi.messangi.android.demo.inbox;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;

import com.ogangi.messangi.android.demo.R;
import com.ogangi.messangi.android.demo.inbox.notification.Notification;
import com.ogangi.messangi.android.sdk.Messangi;
import com.ogangi.messangi.android.sdk.MessangiService;
import com.ogangi.messangi.android.sdk.Util;
import com.ogangi.messangi.android.sdk.vo.MessageVO;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * A fragment representing a list of Items.
 * Activities containing this fragment MUST implement the {@link OnFragmentInteractionListener}
 * interface.
 */
public class NotificationFragment extends ListFragment{
    private static Context context;
    private OnFragmentInteractionListener mListener;
    private NotificationAdapter mAdapter;

    public static NotificationFragment newInstance(Context ctx) {
        context = ctx;
        return new NotificationFragment();
    }

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public NotificationFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mAdapter = new NotificationAdapter(getActivity(), Notification.Notifications);
        setListAdapter(mAdapter);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        this.setEmptyText("No notifications yet, please go to mesangi.com to start sending push notifications");

        ListView lv = getListView();
        AdapterView.OnItemClickListener cListener = new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if (null != mListener) {
                    // Notify the active callbacks interface (the activity, if the
                    // fragment is attached to one) that an item has been selected.
                    mListener.onFragmentInteraction("Notification", Notification.getNotification(position).id);
                }
            }
        };
        lv.setOnItemClickListener(cListener);


        AdapterView.OnItemLongClickListener listener = new AdapterView.OnItemLongClickListener() {
            @Override
            public boolean onItemLongClick(AdapterView<?> arg0, View arg1, int position, long id) {
                Notification.NotificationItem tmp = Notification.getNotification(position);
                tmp.selected = !tmp.selected;
                mAdapter.notifyDataSetChanged();

                if(null != mListener){
                    mListener.deleteAction(Notification.itemToDelete());
                }
                return true;
            }
        };
        lv.setOnItemLongClickListener(listener);
        updateDataSet();
    }

    public void updateDataSet(){
        List<MessageVO> messages;
        try {
            messages = Messangi.getInstance().getService().getAllInboxMessages(context);
        }catch (NullPointerException _n){
            _n.printStackTrace();
            messages = new ArrayList<>();
        }
        for(MessageVO message : messages){
            if(Notification.getNotification(message.getMessageIDAux()) == null) {
                Notification.NotificationItem tmp = new Notification.NotificationItem(
                        message.getMessageIDAux(),
                        message.getMessageTitle(),
                        message.getMessageContent(),
                        message.getMessageDate()
                );
                tmp.setType(message.getMessageType());
                tmp.selected = false;
                Notification.addItem(tmp);
            }
        }
        mAdapter.sort(new Comparator<Notification.NotificationItem>() {
            @Override
            public int compare(Notification.NotificationItem notA, Notification.NotificationItem notB) {
                if (notA.date.after(notB.date)) {
                    return -1;
                } else if (notA.date.before(notB.date)) {
                    return 1;
                } else {
                    return 0;
                }
            }
        });
        mAdapter.notifyDataSetChanged();
    }

    public void deleteNotifications(){
        Notification.deleteSelected(getContext());
        mAdapter.notifyDataSetChanged();
        if(null != mListener){
            mListener.deleteAction(false);
        }
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mListener = (OnFragmentInteractionListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /*
    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);

        if (null != mListener) {
            // Notify the active callbacks interface (the activity, if the
            // fragment is attached to one) that an item has been selected.
            mListener.onFragmentInteraction("Notification", Notification.getNotification(position).id);
        }
    }
    */

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     */
    public interface OnFragmentInteractionListener {
        public void onFragmentInteraction(String type, String id);
        public void deleteAction(boolean active);
    }

}


