package com.android.gcm;

import android.content.Context;
import android.location.Location;
import android.os.Bundle;

import com.google.android.gms.gcm.GcmListenerService;
import com.ogangi.messangi.android.sdk.Logger;
import com.ogangi.messangi.android.sdk.Messangi;
import com.ogangi.messangi.android.sdk.MessangiLocationManager;
import com.ogangi.messangi.android.sdk.RemoteAPI;
import com.ogangi.messangi.android.sdk.Util;
import com.ogangi.messangi.android.sdk.vo.MessageVO;

import java.util.Date;

/**
 * Listener to Wait GCM push notifications
 */
public class GcmListener extends GcmListenerService {

    /**
     * Function that receives all notifications
     *
     * @param from String that represent the origin of the PUSH
     * @param data Data send by gcm Server to Application
     */
    @Override
    public void onMessageReceived(String from, Bundle data) {

        try {
            final String message = data.getString("content");
            final String msgType = data.getString("type");
            Util.debug("On message. (" + msgType + ") Message: " + message + " from: " + from);
            Logger.getInstance(this).addMessage(message, msgType, new Date().getTime());

            if (msgType != null && message != null) {
                String str[];
                String id;
                switch (msgType){
                    case "GEOFENCE":
                    case "BEACON":
                        //TODO: Prueba esto para una proxima version
                        //Messangi.getInstance().logEvent(this, msgType, message);
                        //pushReceived(this,message);
                        //break;
                    case "TXT_MSG":
                        Messangi.getInstance().logEvent(this, msgType, message);
                        pushReceived(this);
                        break;
                    case "CRT_REGION":
                    case "UPDT_REGION":
                        str = message.split(":");
                        id = str[0];
                        Messangi.getInstance().logEvent(this, msgType, message,str[5]);
                        MessangiLocationManager.getInstance().addGeofence(id, message);
                        MessangiLocationManager.getInstance().watchGeofence();
                        Messangi.getInstance().fireGeofenceUpdate(msgType,id);
                        break;
                    case "DLT_REGION":
                        Messangi.getInstance().logEvent(this, msgType, message);
                        MessangiLocationManager.getInstance().deleteGeofence(message);
                        Messangi.getInstance().fireGeofenceUpdate(msgType,message);
                        break;
                    case "CRT_BEACON":
                    case "UPDT_BEACON":
                        str = message.split(":");
                        id = str[0];
                        Messangi.getInstance().logEvent(this, msgType, message,str[7]);
                        MessangiLocationManager.getInstance().deleteBeacon(id);
                        MessangiLocationManager.getInstance().addBeacon(id, message);
                        Messangi.getInstance().fireBeaconUpdate(msgType,id);
                        break;
                    case "DLT_BEACON":
                        Messangi.getInstance().logEvent(this, msgType, message);
                        MessangiLocationManager.getInstance().deleteBeacon(message);
                        Messangi.getInstance().fireBeaconUpdate(msgType, message);
                        break;
                    case "GET_LOC":
                    case "GEO_LOC":
                        Messangi.getInstance().logEvent(this, msgType, message);
                        Location location = MessangiLocationManager.getInstance().getLastKnownLocation();
                        if (location != null) {
                            RemoteAPI.sendLocation(this, message, location.getLatitude(), location.getLongitude());
                        }
                        break;
                    case "GEO_PUSH":
                    case "GEOPUSH":
                        Messangi.getInstance().logEvent(this, msgType, message);
                        str = message.split(":");
                        id  = str[1];
                        String lat = str[2];
                        String lon = str[3];
                        String radius = str[4];
                        MessangiLocationManager.getInstance().onGeoPush(this, this, id, lat, lon, radius);
                        break;
                }
            }
        } catch (Exception _e) {
            Messangi.getInstance().reportError(_e);
        }
    }

    /**
     * Retrive all Push notifications
     *
     * @param context the context
     */
    public void pushReceived(Context context) {
        Messangi.getInstance().getUnreadMessages(context);
    }

    /**
     * Retrieve a single Push Notification
     *
     * @param context the context
     * @param id the id of push to retrieve
     */
    public void pushReceived(Context context, String id) {
        try {
            MessageVO message = RemoteAPI.fetchMessage(context, id);
            if (message == null) {
                Util.debug("Connection error");
            } else {
                Util.debug("Messangi Content:" + message.getMessageTitle() + " ; Content: " + message.getMessageContent());
                Messangi.getInstance().firePushNotificationEvent(context, message, true);
            }
        } catch (Exception _e) {
            Messangi.getInstance().reportError(_e);
        }
    }
}

