package com.ogangi.messangi.android.sdk;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.location.Location;

import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.GeofenceStatusCodes;
import com.google.android.gms.location.GeofencingEvent;
import com.ogangi.messangi.android.sdk.vo.GeoFenceVO;

import java.util.Calendar;
import java.util.List;

/**
 * Alternative to GeofenceTransitionIntentService
 *
 */
public class GeofenceReceiver extends BroadcastReceiver {
    Context context;

    Intent broadcastIntent = new Intent();

    /**
     * Handles incoming intents.
     * @param context App Context
     * @param intent sent by Location Services. This Intent is provided to Location
     *               Services (inside a PendingIntent) when addGeofences() is called.
     */
    @Override
    public void onReceive(Context context, Intent intent) {
        this.context = context;

        Util.debug("GeoFence Received");

        final GeofencingEvent geofencingEvent = GeofencingEvent.fromIntent(intent);
        if (geofencingEvent.hasError()) {
            handleError(geofencingEvent);
        }else{
            new Thread() {
                public void run() {
                    handleEnterExit(geofencingEvent);
                }
            }.start();

        }
    }

    /**
     * Handles Error events.
     * @param geofencingEvent event sent in intent
     */
    private void handleError(GeofencingEvent geofencingEvent){
        // Get the error code
        int errorCode = geofencingEvent.getErrorCode();

        // Get the error message
        String errorMessage =GeofenceStatusCodes.getStatusCodeString(errorCode);

        // Log the error
        Util.debug(errorMessage);
    }

    /**
     * Handler Geofence Enter/Exit Event
     * @param geofencingEvent event sent in intent
     */
    private void handleEnterExit(GeofencingEvent geofencingEvent) {
        // Get the transition type.
        int geofenceTransition = geofencingEvent.getGeofenceTransition();
        List<Geofence> triggeringGeofences = geofencingEvent.getTriggeringGeofences();
        Location location = geofencingEvent.getTriggeringLocation();

        SimpleGeofenceStore geofenceStore = new SimpleGeofenceStore(context);

        for (Geofence geofence : triggeringGeofences) {
            String key = geofence.getRequestId();
            GeoFenceVO triggered = geofenceStore.getGeofence(key);

            if(triggered != null) {

                long now = Calendar.getInstance().getTimeInMillis();
                long elapsedSinceCreation = now - triggered.getCreationTime();
                long elapsedSinceEnter = now - triggered.getLastEnterTime();
                long elapsedSinceExit = now - triggered.getLastExitTime();

                Util.debug("elapsedSinceCreation: " + elapsedSinceCreation);
                Util.debug("elapsedSinceEnter: " + elapsedSinceEnter);
                Util.debug("elapsedSinceExit: " + elapsedSinceExit);

                if (geofenceTransition == Geofence.GEOFENCE_TRANSITION_ENTER) {
                    triggered.setLastEnterTime(now);
                    RemoteAPI.geoFenceEvent(context, key, "ENTER", location.getLatitude(), location.getLongitude(), "GEOFENCE");
                    Messangi.getInstance().logEvent(context, "GEOFENCE_ENTER", key,triggered.getName());
                    geofenceStore.saveGeofence(key, triggered);
                }
                if (geofenceTransition == Geofence.GEOFENCE_TRANSITION_EXIT) {
                    triggered.setLastExitTime(now);
                    RemoteAPI.geoFenceEvent(context, key, "EXIT", location.getLatitude(), location.getLongitude(), "GEOFENCE");
                    Messangi.getInstance().logEvent(context, "GEOFENCE_EXIT", key,triggered.getName());
                    geofenceStore.saveGeofence(key, triggered);
                }
            }
        }
        Messangi.getInstance().fireGeofenceTrigger(triggeringGeofences,geofenceTransition,location);
    }
}