package com.ogangi.messangi.android.sdk;

import android.app.IntentService;
import android.content.Intent;
import android.location.Location;

import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.GeofencingEvent;
import com.ogangi.messangi.android.sdk.vo.GeoFenceVO;

import java.util.Calendar;
import java.util.List;

/**
 * Wait GeoFence ENTER or EXIT Event
 * NOT IN USE >> view GeofenceReceiver
 */
public class GeofenceTransitionsIntentService extends IntentService{
    private static final long CREATIONTIME_GRACE_PERIOD = Constants.CREATION_TIME_GRACE_PERIOD;
    private static final long ENTERTIME_GRACE_PERIOD = Constants.ENTER_TIME_GRACE_PERIOD;
    private static final long EXITTIME_GRACE_PERIOD = Constants.EXIT_TIME_GRACE_PERIOD;
    protected static final String TAG = "geofence-transitions-service";

    /**
     * This constructor is required, and calls the super IntentService(String)
     * constructor with the name for a worker thread.
     */
    public GeofenceTransitionsIntentService() {
        // Use the TAG to name the worker thread.
        super(TAG);
    }

    @Override
    public void onCreate() {
        super.onCreate();
    }

    /**
     * Handles incoming intents.
     * @param intent sent by Location Services. This Intent is provided to Location
     *               Services (inside a PendingIntent) when addGeofences() is called.
     */
    @Override
    protected void onHandleIntent(Intent intent) {
        Util.debug("GeoFence Received");

        GeofencingEvent geofencingEvent = GeofencingEvent.fromIntent(intent);
        if (geofencingEvent.hasError()) {
            return;
        }
        // Get the transition type.
        int geofenceTransition = geofencingEvent.getGeofenceTransition();
        List<Geofence> triggeringGeofences = geofencingEvent.getTriggeringGeofences();
        Location location = geofencingEvent.getTriggeringLocation();

        SimpleGeofenceStore geofenceStore = new SimpleGeofenceStore(getApplicationContext());

        for (Geofence geofence : triggeringGeofences) {
            String key = geofence.getRequestId();
            GeoFenceVO triggered = geofenceStore.getGeofence(key);

            if(triggered != null) {

                long now = Calendar.getInstance().getTimeInMillis();
                long elapsedSinceCreation = now - triggered.getCreationTime();
                long elapsedSinceEnter = now - triggered.getLastEnterTime();
                long elapsedSinceExit = now - triggered.getLastExitTime();

                Util.debug("elapsedSinceCreation: " + elapsedSinceCreation);
                Util.debug("elapsedSinceEnter: " + elapsedSinceEnter);
                Util.debug("elapsedSinceExit: " + elapsedSinceExit);

                if (geofenceTransition == Geofence.GEOFENCE_TRANSITION_ENTER) {
                    triggered.setLastEnterTime(now);
                    RemoteAPI.geoFenceEvent(getApplicationContext(), key, "ENTER", location.getLatitude(), location.getLongitude(), "GEOFENCE");
                    Messangi.getInstance().logEvent(getApplicationContext(), "GEOFENCE_ENTER", key,triggered.getName());
                    geofenceStore.saveGeofence(key, triggered);
                }
                if (geofenceTransition == Geofence.GEOFENCE_TRANSITION_EXIT) {
                    triggered.setLastExitTime(now);
                    RemoteAPI.geoFenceEvent(getApplicationContext(), key, "EXIT", location.getLatitude(), location.getLongitude(), "GEOFENCE");
                    Messangi.getInstance().logEvent(getApplicationContext(), "GEOFENCE_EXIT", key,triggered.getName());
                    geofenceStore.saveGeofence(key, triggered);
                }
            }else{
                MessangiLocationManager.getInstance().deleteGeofence(key);
            }
        }

        Messangi.getInstance().fireGeofenceTrigger(triggeringGeofences,geofenceTransition,location);
    }

}
