package com.ogangi.messangi.android.sdk;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.support.annotation.Nullable;

import com.google.gson.Gson;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

class MessangiLog{
    private String message;
    private String type;
    private long timeStamp;

    public MessangiLog(String message, String type, long timeStamp) {
        this.message = message;
        this.type = type;
        this.timeStamp = timeStamp;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public long getTimeStamp() {
        return timeStamp;
    }

    public void setTimeStamp(long timeStamp) {
        this.timeStamp = timeStamp;
    }
}

//TODO: Hacer los nombres de columnas constantes
public class Logger extends SQLiteOpenHelper {
    private static Logger instance;
	private static final int databaseVersion = 3;
	private static final String databaseName = "logger";
	private static final String LoggerTableCreate = "CREATE TABLE " + databaseName + " (ID INTEGER PRIMARY KEY AUTOINCREMENT, MESSAGE TEXT, TYPE TEXT, TIMESTAMP UNSIGNED BIG INT);";

	private Logger(Context context) {
		super(context, databaseName, null, databaseVersion);
	}

    public static Logger getInstance(Context context){
        if(instance == null){
            instance = new Logger(context);
        }
        return instance;
    }

	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL(LoggerTableCreate);
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {

	}

	public void addMessage(String message, String type, long timeStamp) throws SQLException {

        ContentValues newRow = new ContentValues();
        newRow.put("MESSAGE", message);
        newRow.put("TYPE", type);
        newRow.put("TIMESTAMP", timeStamp);

        SQLiteDatabase db = this.getWritableDatabase();
		if(db.insert(databaseName,null,newRow) == -1){
			throw new SQLException();
		}
        //db.close();

	}



	public List<MessangiLog> getMessages(long init,long end){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ?", new String[]{String.valueOf(init), String.valueOf(end)}, null, null, "TIMESTAMP DESC");
        //db.close();
        return processCursor(fetch);
	}

	public List<MessangiLog> getMessages(String type){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TYPE = ?", new String[]{type}, null, null, "TIMESTAMP DESC");
        //db.close();
        return processCursor(fetch);
	}

	public List<MessangiLog> getMessages(Nullable notUsed,String content){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "MESSAGE LIKE = ?", new String[]{content}, null, null, "TIMESTAMP DESC");
        //db.close();
        return processCursor(fetch);
	}

	public List<MessangiLog> getMessages(long init,long end, String type){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ? AND TYPE = ?", new String[]{String.valueOf(init), String.valueOf(end), type}, null, null, "TIMESTAMP DESC");
        //db.close();
        return processCursor(fetch);
	}

    public List<MessangiLog> getMessages(Nullable notUsed,long init,long end, String content){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ? AND MESSAGE LIKE = ?", new String[]{String.valueOf(init), String.valueOf(end), content}, null, null, "TIMESTAMP DESC");
        //db.close();
        return processCursor(fetch);
    }

    public List<MessangiLog> getMessages(long init,long end, String type ,String content){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ? AND TYPE = ? AND MESSAGE LIKE = ?", new String[]{String.valueOf(init), String.valueOf(end), type, content}, null, null, "TIMESTAMP DESC");
        //db.close();
        return processCursor(fetch);
    }




    public String getJSONMessages(long init,long end){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ?", new String[]{String.valueOf(init), String.valueOf(end)}, null, null, "TIMESTAMP DESC");
        //db.close();
        Gson gson = new Gson();
        return gson.toJson(processCursor(fetch));
    }

    public String getJSONMessages(String type){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TYPE = ?", new String[]{type}, null, null, "TIMESTAMP DESC");
        //db.close();
        Gson gson = new Gson();
        return gson.toJson(processCursor(fetch));
    }

    public String getJSONMessages(Nullable notUsed,String content){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "MESSAGE LIKE = ?", new String[]{content}, null, null, "TIMESTAMP DESC");
        //db.close();
        Gson gson = new Gson();
        return gson.toJson(processCursor(fetch));
    }

    public String getJSONMessages(long init,long end, String type){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ? AND TYPE = ?", new String[]{String.valueOf(init), String.valueOf(end), type}, null, null, "TIMESTAMP DESC");
        //db.close();
        Gson gson = new Gson();
        return gson.toJson(processCursor(fetch));
    }

    public String getJSONMessages(Nullable notUsed,long init,long end, String content){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ? AND MESSAGE LIKE = ?", new String[]{String.valueOf(init), String.valueOf(end), content}, null, null, "TIMESTAMP DESC");
        //db.close();
        Gson gson = new Gson();
        return gson.toJson(processCursor(fetch));
    }

    public String getJSONMessages(long init,long end, String type ,String content){
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor fetch = db.query(databaseName, null, "TIMESTAMP BETWEEN ? AND ? AND TYPE = ? AND MESSAGE LIKE = ?", new String[]{String.valueOf(init), String.valueOf(end), type, content}, null, null, "TIMESTAMP DESC");
        //db.close();
        Gson gson = new Gson();
        return gson.toJson(processCursor(fetch));
    }




    private List<MessangiLog> processCursor(Cursor fetch){
        List<MessangiLog> logs = new ArrayList<>();
        if (fetch != null) {
            if (fetch.moveToFirst()) {
                do {
                    String message = fetch.getString(fetch.getColumnIndex("MESSAGE"));
                    String messagetType = fetch.getString(fetch.getColumnIndex("TYPE"));
                    long timestamp = fetch.getLong(fetch.getColumnIndex("TIMESTAMP"));
                    logs.add(new MessangiLog(message,messagetType,timestamp));
                } while (fetch.moveToNext());
            }
        }
        return logs;
    }
}
