package com.ogangi.messangi.android.sdk;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.os.Handler;
import android.telephony.TelephonyManager;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;

/**
 * TODO: MEjorar la ventana de Login, preferiblemente utilizar una activity
 * Create and Handler the registration process
 */
@SuppressWarnings("unused")
public class LoginDialog extends Dialog {
	private static final int maxWait = 2 * 60 * 1000;
	private Handler handler;
	private Context ctx;
    private static Preferences prefs = null;
    private Activity parent = null;
    private int actualState;

	private Button btnRegister;
	private Button btnExit;
	private EditText editText;
	private TextView txtHint;
	private TextView txtBadNumber;
    private TextView txtNetworkError;
	private TextView txtEnterValidationCode;
	private ProgressBar pbRegister;
    private EditText validationCode = null;
    private Button btnValidate = null;
    private boolean progressRunning = false;

    /**
     * Instantiates a new Login dialog.
     *
     * @param context the context
     * @param parent the parent
     */
    public LoginDialog(Context context, Activity parent) {
		super(context, R.style.DialogStyle);
		this.ctx = context;
		this.parent = parent;
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.dialog_login);
		setTitle(R.string.app_name);
		prefs = Preferences.getInstance(context);
        loadRegister();
        handler = new Handler();
        watchConnection.start();
	}

    private void loadRegister(){
        actualState = 1;
        btnRegister = (Button) findViewById(R.id.btnDoRegister);
        btnRegister.setOnClickListener(btnRegisterClicked);
        btnExit = (Button) findViewById(R.id.btnExit);
        btnExit.setOnClickListener(btnExitClicked);
        txtHint = (TextView) findViewById(R.id.textHint);
        txtHint.setText(R.string.registerHint);
        editText = (EditText) findViewById(R.id.inputPhone);
        TelephonyManager tMgr = (TelephonyManager) ctx.getSystemService(Context.TELEPHONY_SERVICE);
        String phoneNumber = tMgr.getLine1Number();
        String countryCodeNet = tMgr.getNetworkCountryIso();
        String countryCodeSim = tMgr.getSimCountryIso();
        if (phoneNumber != null) {
            if (!phoneNumber.startsWith("+")) {
                phoneNumber = "+" + phoneNumber;
            }
            editText.setText(phoneNumber);
        }
        txtNetworkError = (TextView) findViewById(R.id.textBadNetwork);
        txtNetworkError.setVisibility(TextView.GONE);
        txtBadNumber = (TextView) findViewById(R.id.textBadNumber);
        txtBadNumber.setVisibility(TextView.GONE);
        txtEnterValidationCode = (TextView) findViewById(R.id.enterValidationCode);
        txtEnterValidationCode.setVisibility(TextView.GONE);
        pbRegister = (ProgressBar) findViewById(R.id.pbRegister);
        pbRegister.setMax(maxWait);
        pbRegister.setVisibility(ProgressBar.GONE);
        validationCode = (EditText) findViewById(R.id.validationCode);
        validationCode.setVisibility(EditText.GONE);
        btnValidate = (Button) findViewById(R.id.btnValidate);
        btnValidate.setOnClickListener(btnValidateClickListener);
        btnValidate.setVisibility(Button.GONE);
    }

	private void activatePhone() {
		boolean activationOk = RemoteAPI.activatePhone(ctx, prefs.getString(Preferences.ACTIVATION_CODE).trim());
		if (activationOk) {
			handler.post(closeDialog);
			prefs.saveValue(Preferences.PHONE_ACTIVATED, "true");

			parent.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Messangi.getInstance().fireInitCompledEvent();
                }
            });
		} else {
            handler.post(showRegister);
            handler.post(badValidation);
		}
	}

    private Thread watchConnection = new Thread(new Runnable() {
        @Override
        public void run() {
            while (true) {
                try {
                    if (RemoteAPI.checkConnection(ctx)) {
                        handler.post(godNetwork);
                    } else {
                        handler.post(badNetwork);
                    }

                    if(Util.isMobileAvailable(ctx)){
                        handler.post(godMobileNetwork);
                    }else{
                        handler.post(badMobileNetwork);
                    }
                    Thread.sleep(30000);

                } catch (Exception e) {
                    RemoteAPI.logEvent(ctx,"ERROR",e.getMessage(),null);
                    e.printStackTrace();
                    handler.post(badNetwork);
                }
            }
        }
    });

    private int current = 0;
    private android.view.View.OnClickListener btnRegisterClicked = new android.view.View.OnClickListener() {
        public void onClick(View v) {
            new Thread(new Runnable() {
                public void run() {
                    String number = editText.getText().toString().trim();
                    if (number.length() == 0 || number.length() < 9) {
                        handler.post(badNumber);
                        return;
                    }
                    if (!number.startsWith("+")) {
                        handler.post(badNumber);
                        return;
                    }
                    RemoteAPI.registerPhone(ctx, editText.getText().toString());
                    handler.post(showActivation);
                    prefs.saveValue(Preferences.ACTIVATION_CODE, "");

                    progressRunning = true;
                    // Start lengthy operation in a background thread

                    while (current < maxWait) {
                        Util.info("Check: " + prefs.getString(Preferences.ACTIVATION_CODE).trim());

                        if(!progressRunning)
                            return;

                        if (!prefs.getString(Preferences.ACTIVATION_CODE).trim().equals("")) {
                            break;
                        }
                        try {
                            Thread.sleep(500);
                            current += 500;
                        } catch (Exception e) {
                            RemoteAPI.logEvent(ctx,"ERROR",e.getMessage(),null);
                            e.printStackTrace();
                        }

                        // Update the progress bar
                        handler.post(new Runnable() {
                            public void run() {
                                pbRegister.setProgress(current);
                            }
                        });
                    }

                    if (current >= maxWait) {
                        handler.post(showRegister);
                        handler.post(badSMS);
                    } else {
                        prefs.saveValue(Preferences.PHONE_NUMBER, editText.getText().toString());
                        LoginDialog.this.activatePhone();
                    }

                }
            }).start();
        }
    };

    private android.view.View.OnClickListener btnValidateClickListener = new android.view.View.OnClickListener() {
		public void onClick(View v) {
			progressRunning = false;
			prefs.saveValue(Preferences.PHONE_NUMBER, editText.getText().toString());
			Preferences.getInstance(LoginDialog.this.getContext()).saveValue(Preferences.ACTIVATION_CODE, validationCode.getText().toString());
			prefs.saveValue(Preferences.ACTIVATION_CODE, validationCode.getText().toString());
            validationCode.setText("");
			new Thread() {
				public void run() {
					LoginDialog.this.activatePhone();
				}
			}.start();
		}
	};

	private android.view.View.OnClickListener btnExitClicked = new android.view.View.OnClickListener() {
		public void onClick(View v) {
            System.runFinalizersOnExit(true);
			System.exit(0);
		}
	};
	private final Runnable badValidation = new Runnable() {
		public void run() {
			txtBadNumber.setText(R.string.badValidation);
			txtBadNumber.setVisibility(TextView.VISIBLE);
		}
	};
	private final Runnable badSMS = new Runnable() {
		public void run() {
            txtBadNumber.setText(R.string.badSMS);
			txtBadNumber.setVisibility(TextView.VISIBLE);
		}
	};
    private final Runnable badMobileNetwork = new Runnable() {
        public void run() {
            txtBadNumber.setText(R.string.badMobileNetwork);
            txtBadNumber.setVisibility(TextView.VISIBLE);
        }
    };

    private final Runnable godMobileNetwork = new Runnable() {
        @Override
        public void run() {
            if(txtBadNumber.getText().equals(ctx.getString(R.string.badMobileNetwork))){
                txtBadNumber.setVisibility(TextView.GONE);
            }
        }
    };

	private final Runnable badNetwork = new Runnable() {
		public void run() {
            txtNetworkError.setVisibility(TextView.VISIBLE);

            btnRegister.setVisibility(View.GONE);
            btnExit.setVisibility(View.GONE);
            btnValidate.setVisibility(Button.GONE);

            pbRegister.setVisibility(View.GONE);
            txtHint.setVisibility(View.GONE);
            editText.setVisibility(View.GONE);

            txtEnterValidationCode.setVisibility(TextView.GONE);
            validationCode.setVisibility(EditText.GONE);

            progressRunning = false;
		}
	};

    private final Runnable godNetwork = new Runnable() {
        public void run() {
            if(actualState == 1) {
                handler.post(showRegister);
            }else{
                handler.post(showActivation);
            }
        }
    };

	private final Runnable badNumber = new Runnable() {
		public void run() {
			btnRegister.setEnabled(true);
			txtBadNumber.setText(R.string.badNumber);
			txtBadNumber.setVisibility(TextView.VISIBLE);
			validationCode.setVisibility(EditText.GONE);
			btnValidate.setVisibility(Button.GONE);
			txtEnterValidationCode.setVisibility(TextView.GONE);
		}
	};
	private final Runnable removeAlert = new Runnable() {
		public void run() {
			txtBadNumber.setVisibility(TextView.GONE);
		}
	};

    private final Runnable showRegister = new Runnable() {
        public void run() {
            actualState = 1;
            txtHint.setText(R.string.registerHint);
            txtHint.setVisibility(View.VISIBLE);

            editText.setVisibility(EditText.VISIBLE);

            txtNetworkError.setVisibility(TextView.GONE);

            btnRegister.setEnabled(true);
            btnRegister.setVisibility(View.VISIBLE);

            btnExit.setVisibility(View.VISIBLE);

            pbRegister.setVisibility(View.GONE);
            txtEnterValidationCode.setVisibility(TextView.GONE);
            validationCode.setVisibility(EditText.GONE);
            btnValidate.setVisibility(Button.GONE);
        }
    };

    private final Runnable showActivation = new Runnable() {
        public void run() {
            actualState = 2;
            txtHint.setText(R.string.registerWaits);
            txtHint.setVisibility(View.VISIBLE);

            pbRegister.setProgress(0);
            pbRegister.setVisibility(View.VISIBLE);

            txtEnterValidationCode.setVisibility(TextView.VISIBLE);

            txtNetworkError.setVisibility(TextView.GONE);

            validationCode.setVisibility(EditText.VISIBLE);
            btnValidate.setVisibility(Button.VISIBLE);

            editText.setVisibility(EditText.GONE);

            btnRegister.setEnabled(false);
            btnRegister.setVisibility(View.GONE);

            btnExit.setVisibility(View.GONE);
        }
    };



	private final Runnable hideButtons = new Runnable() {
		public void run() {
            actualState = 2;
			txtBadNumber.setVisibility(TextView.GONE);
			btnRegister.setEnabled(false);
			pbRegister.setProgress(0);
			pbRegister.setVisibility(View.VISIBLE);
			txtHint.setText(R.string.registerWaits);
			// sCountryCodes.setEnabled(false);
			editText.setVisibility(EditText.GONE);
			btnRegister.setVisibility(View.GONE);
			btnExit.setVisibility(View.GONE);
			txtEnterValidationCode.setVisibility(TextView.VISIBLE);
			validationCode.setVisibility(EditText.VISIBLE);
			btnValidate.setVisibility(Button.VISIBLE);
		}
	};
	private final Runnable showButtons = new Runnable() {
		public void run() {
            actualState = 1;
            btnRegister.setEnabled(true);
            pbRegister.setVisibility(View.GONE);
            txtHint.setText(R.string.registerHint);
            editText.setVisibility(EditText.VISIBLE);
            btnRegister.setVisibility(View.VISIBLE);
            btnExit.setVisibility(View.VISIBLE);
            txtEnterValidationCode.setVisibility(TextView.GONE);
            validationCode.setVisibility(EditText.GONE);
            btnValidate.setVisibility(Button.GONE);
		}
	};
	private final Runnable closeDialog = new Runnable() {
		public void run() {
			dismiss();
		}
	};
	@Override
	public void onBackPressed() {
		System.runFinalizersOnExit(true);
		System.exit(0);
	}
}

