package com.ogangi.messangi.android.sdk;

import android.location.Location;

import com.google.android.gms.location.Geofence;
import com.ogangi.messangi.android.sdk.vo.MessageVO;

import java.util.List;


/**
 * The interface listener to received push notifications.
 */
public interface MessangiListener {

    /**
     * Invoke when a push notification is received.
     *
     * @param message push notification
     */
    void pushReceived(MessageVO message);

    /**
     * Invoke after geoFence is trigger
     *
     * @param geofences the GeoFences trigger
     * @param TriggerAcction the Action trigger geofence
     * @param triggerLocation the location from trigger geofence
     */
    void updateFencesStatus(List<Geofence> geofences,int TriggerAcction, Location triggerLocation);

    /**
     * Invoke after initialization is successful
     */
    void postInit();

    /**
     * Invoke when update location
     * @param lastKnowLocation
     */
    void onLocationChange(Location lastKnowLocation);

    /**
     * Invoke when geofence is created, updated or deleted
     * @param Type
     * @param id the id of geofence to create, update or delete
     */
    void onGeofenceUpdate(String Type, String id);

    /**
     * Invoke when beacon is created, updated or deleted
     * @param Type
     * @param id The id of beacon to create, update or delete
     */
    void onBeaconUpdate(String Type, String id);
}

