package com.ogangi.messangi.android.sdk;

import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.android.gcm.GcmListener;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.common.api.Status;
import com.google.android.gms.location.Geofence;
import com.google.android.gms.location.GeofenceStatusCodes;
import com.google.android.gms.location.GeofencingRequest;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;
import com.ogangi.messangi.android.sdk.vo.BeaconVO;
import com.ogangi.messangi.android.sdk.vo.GeoFenceVO;

import org.altbeacon.beacon.Beacon;
import org.altbeacon.beacon.BeaconConsumer;
import org.altbeacon.beacon.BeaconManager;
import org.altbeacon.beacon.BeaconParser;
import org.altbeacon.beacon.MonitorNotifier;
import org.altbeacon.beacon.Region;
import org.altbeacon.beacon.powersave.BackgroundPowerSaver;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Storage for geofence values, implemented in SharedPreferences.
 */
class SimpleGeofenceStore {
    // The SharedPreferences objects in which geofences are stored.
    private final SharedPreferences mPrefs;
    private final SharedPreferences idPrefs;

    // The name of the SharedPreferences.
    private static final String SHARED_PREFERENCES = "geofences";
    private static final String ID_SHARED_PREFERENCES = "idGeofences";

    /**
     * Create the SharedPreferences storage with private access only.
     */
    public SimpleGeofenceStore(@NonNull Context context) {
        mPrefs = context.getSharedPreferences(SHARED_PREFERENCES, Context.MODE_PRIVATE);
        idPrefs = context.getSharedPreferences(ID_SHARED_PREFERENCES, Context.MODE_PRIVATE);
    }

    /**
     * Save Double value in preference object
     * @param edit preference to edit
     * @param key key value
     * @param value value to store
     * @return The preferences  after add value
     */
    SharedPreferences.Editor putDouble(final SharedPreferences.Editor edit, final String key, final double value) {
        //TODO: En proximas versiones se puede eliminar este try/catch, la actualizacion de 2.6 a 3 lo requeria
        try {
            return edit.putLong(key, Double.doubleToRawLongBits(value));
        }catch (Exception _e){
            edit.remove(key);
            return edit.putLong(key, Double.doubleToRawLongBits(value));
        }

    }

    /**
     * Retrive a double save
     * @param prefs preference object
     * @param key key valu to retrieve
     * @param defaultValue default values if key not are register
     * @return double storage in key
     */
    double getDouble(final SharedPreferences prefs, final String key, final double defaultValue) {
        //TODO: En proximas versiones se puede eliminar este try/catch, la actualizacion de 2.6 a 3 lo requeria
        try {
            return Double.longBitsToDouble(prefs.getLong(key, Double.doubleToLongBits(defaultValue)));
        }catch (Exception _e){
            float tmp = prefs.getFloat(key,Float.MAX_VALUE);
            mPrefs.edit().remove(key).commit();
            putDouble(prefs.edit(), key, tmp);
            return tmp;
        }
    }

    /**
     * Returns a stored geofence by its id, or returns null if it's not found.
     * @param id The ID of a stored geofence.
     * @return A SimpleGeofence defined by its center and radius, or null if the ID is invalid.
     */
    @Nullable
    public GeoFenceVO getGeofence(String id) {
        // Get the latitude for the geofence identified by id, or INVALID_FLOAT_VALUE if it doesn't
        // exist (similarly for the other values that follow).
        double lat = getDouble(mPrefs, getGeofenceFieldKey(id, "LATITUDE"),  Double.NaN);
        double lng = getDouble(mPrefs, getGeofenceFieldKey(id, "LONGITUDE"), Double.NaN);

        //TODO: En proximas versiones se puede eliminar este try/catch, la actualizacion de 2.6 a 3 lo requeria
        float radius;
        try {
            radius = mPrefs.getFloat(getGeofenceFieldKey(id, "RADIUS"), Float.NaN);
        }catch (Exception _e){
            radius = (float)mPrefs.getInt(getGeofenceFieldKey(id, "RADIUS"),0);
        }

        long expirationDuration = mPrefs.getLong(getGeofenceFieldKey(id, "EXPIRATION"), -100);
        int transitionType = mPrefs.getInt(getGeofenceFieldKey(id, "TYPE"), -1);
        long creationTime = mPrefs.getLong(getGeofenceFieldKey(id, "CREATIONTIME"), -1);
        long lastEnter = mPrefs.getLong(getGeofenceFieldKey(id, "LASTENTER"), -1);
        long lastExit  = mPrefs.getLong(getGeofenceFieldKey(id, "LASTEXIT"), -1);
        String name = mPrefs.getString(getGeofenceFieldKey(id, "NAME"), "");
        // If none of the values is incorrect, return the object.
        if (Double.isNaN(lat)
            || Double.isNaN(lng)
            || Float.isNaN(radius)
            || expirationDuration == -100
            || transitionType == -1
            || creationTime == -1) {
            return null;
        }
        return new GeoFenceVO(id, lat, lng, radius, name, expirationDuration, transitionType, creationTime, lastEnter, lastExit);

    }

    /**
     * Return List of all Geofence stored in sharedPreference
     * @return List of all Geofence stored in sharedPreference;
     */
    @NonNull
    public ArrayList<Geofence> getGeofence(){
        ArrayList<Geofence> tmp = new ArrayList<>();
        for(String key: idPrefs.getAll().keySet()){
            String id = idPrefs.getString(key, null);
            if(null != id) {
                GeoFenceVO geofence = getGeofence(id);
                if(null != geofence) {
                    tmp.add(geofence.toGeofence());
                }
            }
        }
        return tmp;
    }

    @NonNull
    public ArrayList<GeoFenceVO> getGeofenceVO(){
        ArrayList<GeoFenceVO> tmp = new ArrayList<>();
        for(String key: idPrefs.getAll().keySet()){
            String id = idPrefs.getString(key, null);
            if(null != id) {
                GeoFenceVO geofence = getGeofence(id);
                if(null != geofence) {
                    tmp.add(geofence);
                }
            }
        }
        return tmp;
    }

    /**
     * Save a geofence.
     * @param geofence The SimpleGeofence with the values you want to save in SharedPreferences.
     */
    public void saveGeofence(String id, @NonNull GeoFenceVO geofence) {
        SharedPreferences.Editor ids = idPrefs.edit();
        ids.putString(getGeofenceFieldKey(id, "ID"), id);
        ids.apply();
        // Get a SharedPreferences editor instance. Among other things, SharedPreferences
        // ensures that updates are atomic and non-concurrent.
        SharedPreferences.Editor prefs = mPrefs.edit();
        // Write the Geofence values to SharedPreferences.


        putDouble(prefs, getGeofenceFieldKey(id, "LATITUDE"), geofence.getLatitude());
        putDouble(prefs, getGeofenceFieldKey(id, "LONGITUDE"), geofence.getLongitude());

        //TODO: En proximas versiones se puede eliminar este try/catch, la actualizacion de 2.6 a 3 lo requeria
        try {
            prefs.putFloat(getGeofenceFieldKey(id, "RADIUS"), geofence.getRadius());
        }catch (Exception _e){
            prefs.remove(getGeofenceFieldKey(id, "RADIUS"));
            prefs.putFloat(getGeofenceFieldKey(id, "RADIUS"), geofence.getRadius());
        }

        prefs.putLong(getGeofenceFieldKey(id, "EXPIRATION"), geofence.getExpirationDuration());
        prefs.putInt(getGeofenceFieldKey(id, "TYPE"), geofence.getTransitionType());

        prefs.putLong(getGeofenceFieldKey(id, "CREATIONTIME"), geofence.getCreationTime());
        prefs.putLong(getGeofenceFieldKey(id, "LASTENTER"), geofence.getLastEnterTime());
        prefs.putLong(getGeofenceFieldKey(id, "LASTEXIT"), geofence.getLastExitTime());
        prefs.putString(getGeofenceFieldKey(id, "NAME"), geofence.getName());

        // Commit the changes.
        prefs.apply();
    }

    @NonNull
    public GeoFenceVO updateGeofence(GeoFenceVO geofence) {

        GeoFenceVO stored = this.getGeofence(geofence.getId());
        if(null == stored){
            this.saveGeofence(geofence.getId(),geofence);
            return geofence;
        }
        return geofence;
    }

    /**
     * Remove a flattened geofence object from storage by removing all of its keys.
     */
    public void clearGeofence(String id) {
        SharedPreferences.Editor ids = idPrefs.edit();
        ids.remove(getGeofenceFieldKey(id, "ID"));
        ids.apply();

        SharedPreferences.Editor prefs = mPrefs.edit();
        prefs.remove(getGeofenceFieldKey(id, "LATITUDE"));
        prefs.remove(getGeofenceFieldKey(id, "LONGITUDE"));
        prefs.remove(getGeofenceFieldKey(id, "RADIUS"));
        prefs.remove(getGeofenceFieldKey(id, "EXPIRATION"));
        prefs.remove(getGeofenceFieldKey(id, "TYPE"));
        prefs.remove(getGeofenceFieldKey(id, "CREATIONTIME"));
        prefs.remove(getGeofenceFieldKey(id, "LASTENTER"));
        prefs.remove(getGeofenceFieldKey(id, "LASTEXIT"));
        prefs.remove(getGeofenceFieldKey(id, "NAME"));
        prefs.apply();
    }

    /**
     * Remove all geofence object from storage.
     */
    public void clearGeofence(){
        SharedPreferences.Editor ids = idPrefs.edit();
        ids.clear().apply();

        SharedPreferences.Editor prefs = mPrefs.edit();
        prefs.clear().apply();
    }

    /**
     * Given a Geofence object's ID and the name of a field (for example, KEY_LATITUDE), return
     * the key name of the object's values in SharedPreferences.
     * @param id The ID of a Geofence object.
     * @param fieldName The field represented by the key.
     * @return The full key name of a value in SharedPreferences.
     */
    @NonNull
    private String getGeofenceFieldKey(String id, String fieldName) {
        return "GEOFENCE" + "_" + id + "_" + fieldName;
    }


}

/**
 * Monitor Location, Beacons and GeoFence.
 */
public class MessangiLocationManager implements BeaconConsumer, MonitorNotifier, GoogleApiClient.ConnectionCallbacks, GoogleApiClient.OnConnectionFailedListener, LocationListener {
	private static final String geoEnterTrigger = "ENTER";
	private static final String geoExitTrigger = "EXIT";

    private static MessangiLocationManager instance = null;
    private Context context = null;
    private GoogleApiClient mGoogleApiClient;
    private boolean initialized = false;

    private SimpleGeofenceStore geoFenceStorage;

	private Location lastKnownLocation = null;

    private ArrayList<Geofence> mGeofenceList;

    private BeaconManager beaconManager = null;
    private boolean beaconsSupported = false;
    private PendingIntent mGeofencePendingIntent;
    private String mLastUpdateTime;
    private boolean mRequestingLocationUpdates;
    private boolean TriggerReady;
    private boolean locationAvalible;

    /**
     *
     * @return
     */
    @Override
    public Context getApplicationContext() {
        return context;
    }

    /**
     * private Constructor
     */
    private MessangiLocationManager() {
		initialized = false;
	}

    /**
     * Get the singleton of Messangi Location Manager
     *
     * @return the singleton of Messangi Location Manager
     */
    public static MessangiLocationManager getInstance() {
		if (instance == null) {
			instance = new MessangiLocationManager();
		}
		return instance;
	}

    /**
     * Invoke init.
     *
     * @param context the context
     */
    public void forceInit(Context context) {
		//initialized = false;
		init(context);
	}

    /**
     * Returns <code>true</code> if Location manager is initialized.
     *
     * @return <code>true</code> if Location manager is initialized; <code>false</code> otherwise
     */
    public boolean isInitialized() {
		return initialized;
	}

    /**
     * Init Location Manager.
     *
     * @param context the context
     */
    public void init(Context context) {
		if (!initialized) {
			Util.debug("MessangiLocationManager init");

			this.context = context;
            mRequestingLocationUpdates = true;
            locationAvalible = false;
            buildGoogleApiClient(context);
            onStart();
            mGeofenceList = new ArrayList<>();
            geoFenceStorage = new SimpleGeofenceStore(context);

            //Support for Bluetooth LE and iBeacons started with Android 4.3 (SDK 18)
            try {
                beaconsSupported = Build.VERSION.SDK_INT >= 18;
                beaconsSupported = beaconsSupported && context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE);
            }catch (Exception _e){
                Messangi.getInstance().reportError(_e);
                _e.printStackTrace();
                beaconsSupported = false;
            }
			if (beaconsSupported && beaconManager == null) {
				beaconManager = BeaconManager.getInstanceForApplication(context);

                // Eddystone

                // Detect the main identifier (UID) frame:
                beaconManager.getBeaconParsers().add(new BeaconParser().setBeaconLayout("s:0-1=feaa,m:2-2=00,p:3-3:-41,i:4-13,i:14-19"));
                // Detect the telemetry (TLM) frame:
                // Este es para telemetria
                //beaconManager.getBeaconParsers().add(new BeaconParser().setBeaconLayout("x,s:0-1=feaa,m:2-2=20,d:3-3,d:4-5,d:6-7,d:8-11,d:12-15"));
                // Detect the URL frame:
                // Este es para el itnernet fisico, nuevos protocolos de google
                //beaconManager.getBeaconParsers().add(new BeaconParser().setBeaconLayout("s:0-1=feaa,m:2-2=10,p:3-3:-41,i:4-20"));


				// Redbear
				beaconManager.getBeaconParsers().add(new BeaconParser().setBeaconLayout("m:2-3=0215,i:4-19,i:20-21,i:22-23,p:24-24,d:25-25"));

				// Estimote
				beaconManager.getBeaconParsers().add(new BeaconParser().setBeaconLayout("m:2-3=0215,i:4-19,i:20-21,i:22-23,p:24-24"));



				beaconManager.bind(this);
			}

			initialized = true;
		}
	}

    /**
     * Get geofence from server and start watching geofences
     */
    public void startGeofences(){
        RemoteAPI.getGeofences(getApplicationContext());
        mGeofenceList = geoFenceStorage.getGeofence();
        if(mGeofenceList.size() > 0 && mGoogleApiClient.isConnected()) {
            watchGeofence();
        }
    }

    /**
     * Gets last known location.
     *
     * @return the last known location
     */
    public Location getLastKnownLocation() {
        if(mGoogleApiClient != null && mGoogleApiClient.isConnected() && isGooglePlayServicesAvailable()) {
            Location tmp = LocationServices.FusedLocationApi.getLastLocation(mGoogleApiClient);
            if(tmp != null){
                lastKnownLocation = tmp;
            }
        }
        return lastKnownLocation;
    }

    /***************************************************************************
     ********************* Errors & Exceptions *********************************
     ***************************************************************************/

    /**
     *
     * @param securityException
     */
    private void logSecurityException(SecurityException securityException) {
        securityException.printStackTrace();
        RemoteAPI.logEvent(context, "SecurityException", securityException.getMessage(), null);
        Util.debug(securityException.getMessage());
    }

    /**
     * Create Update Location Request for monitoring location updates
     */
    protected LocationRequest createLocationRequest() {
        LocationRequest mLocationRequest = new LocationRequest();
        mLocationRequest.setInterval(Long.parseLong(Preferences.getInstance(context).getString(Preferences.LOCATION_UPDATE_TIME, "300000")));
        mLocationRequest.setFastestInterval(5000);
        mLocationRequest.setPriority(Integer.parseInt(Preferences.getInstance(context).getString(Preferences.LOCATION_UPDATE_TYPE, "105")));

        return mLocationRequest;
    }

    /**
     * Init to recording Location updates
     */
    protected void startLocationUpdates() {
        LocationServices.FusedLocationApi.requestLocationUpdates(mGoogleApiClient, createLocationRequest(), this);
    }

    /**
     * Function fired when the current location change
     * @param location the new location
     */
    @Override
    public void onLocationChanged(Location location) {
        if(location != null) {
            if(!locationAvalible){
                Messangi.getInstance().logEvent(context,"Location","Avalible");
                locationAvalible = true;
            }
            lastKnownLocation = location;
            mLastUpdateTime = DateFormat.getTimeInstance().format(new Date());
            Messangi.getInstance().firelocationUpdate(lastKnownLocation);
        }else{
            Messangi.getInstance().logEvent(context,"Location","Not Available");
            locationAvalible = false;
        }
    }


    /*****************************************************************************
     *************************** GEOPUSH *****************************************
     *****************************************************************************/

    /**
     * Invoke when Messangi received a GeoPush,
     *
     * @param context The app context
     * @param service The service was received the push notification
     * @param id The ID of Notification
     * @param lat The Latitude of GeoPush
     * @param lon The Longitude GeoPush
     * @param radius The action Radio of GeoPush
     */
    public void onGeoPush(Context context, GcmListener service, String id, String lat, String lon, String radius) {
        Util.debug("MessangiLocationManager onGeoPush");

        Location location = MessangiLocationManager.getInstance().getLastKnownLocation();

        if (location != null) {

            Location geoPushLoc = new Location(id);
            geoPushLoc.setLatitude(Double.parseDouble(lat));
            geoPushLoc.setLongitude(Double.parseDouble(lon));
            float distance = Util.distanceBetween(location,geoPushLoc);

            if (distance <= Float.parseFloat(radius)) {
                service.pushReceived(context, id);
            }
        }

    }

    /*****************************************************************************
	 ************************* GEOFENCES *****************************************
	 *****************************************************************************/

    /**
     * Return all Geofence storage by MessangiSDK
     * @return
     */
    public ArrayList<GeoFenceVO> getGeofenceList() {
        return geoFenceStorage.getGeofenceVO();
    }

    private ResultCallback<Status> onAddGeofence = new ResultCallback<Status>() {
        @Override
        public void onResult(Status status) {
            if(!status.isSuccess())
                switch (status.getStatusCode()){
                    case GeofenceStatusCodes.GEOFENCE_NOT_AVAILABLE:
                        Util.debug("Geofence service is not available now. Typically this is because the user turned off location access in settings > location access.");
                        break;
                    case GeofenceStatusCodes.GEOFENCE_TOO_MANY_GEOFENCES:
                        Util.debug("Your app has registered more than 100 geofences. Remove unused ones before adding new geofences.");
                        break;
                    case GeofenceStatusCodes.GEOFENCE_TOO_MANY_PENDING_INTENTS:
                        Util.debug("You have provided more than 5 different PendingIntents to the addGeofences(GoogleApiClient, GeofencingRequest, PendingIntent) call.");
                        break;
                    default:
                        Util.debug("Unknown error adding geofences");
                        break;
                }
            else
                Util.debug("Geofences Ready");
        }
    };

    private ResultCallback<Status> onRemoveGeofence = new ResultCallback<Status>() {
        @Override
        public void onResult(Status status) {
            if(!status.isSuccess())
                Util.debug(status.getStatusMessage());
        }
    };

    /**
     * Gets a PendingIntent to send with the request to add or remove Geofences. Location Services
     * issues the Intent inside this PendingIntent whenever a geofence transition occurs for the
     * current list of geofences.
     *
     * @return A PendingIntent for the IntentService that handles geofence transitions.
     */
    private PendingIntent getGeofencePendingIntent() {
        // Reuse the PendingIntent if we already have it.
        if (mGeofencePendingIntent != null) {
            return mGeofencePendingIntent;
        }
        Intent intent = new Intent(context, GeofenceTransitionsIntentService.class);
                // We use FLAG_UPDATE_CURRENT so that we get the same pending intent back when calling
                // addGeofences() and removeGeofences().
        return PendingIntent.getService(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
        //mGeofencePendingIntent = PendingIntent.getService(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
        //return mGeofencePendingIntent;
    }


    private PendingIntent createRequestPendingIntent() {

        // If the PendingIntent already exists
        if (null != mGeofencePendingIntent) {
            // Return the existing intent
            return mGeofencePendingIntent;
            // If no PendingIntent exists
        } else {

            // Create an Intent pointing to the IntentService HERE use GEOFENCERECEIVER OR GEOFENCE INTENTSERVICE
            Intent intent = new Intent("com.ogangi.messangi.android.sdk.ACTION_RECEIVE_GEOFENCE");
            //Intent intent = new Intent(context, ReceiveTransitionsIntentService.class);
            /*
             * Return a PendingIntent to start the IntentService.
             * Always create a PendingIntent sent to Location Services
             * with FLAG_UPDATE_CURRENT, so that sending the PendingIntent
             * again updates the original. Otherwise, Location Services
             * can't match the PendingIntent to requests made with it.
             */
            return PendingIntent.getBroadcast(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
        }
    }

    /**
     * Builds and returns a GeofencingRequest. Specifies the list of geofences to be monitored.
     * Also specifies how the geofence notifications are initially triggered.
     */
    private GeofencingRequest getGeofencingRequest() {
        GeofencingRequest.Builder builder = new GeofencingRequest.Builder();
        builder.setInitialTrigger(GeofencingRequest.INITIAL_TRIGGER_ENTER);
        builder.addGeofences(mGeofenceList);
        return builder.build();
    }

    /**
     * Delete GeoFence.
     *
     * @param id the id of GeoFence to delete
     */
    public void deleteGeofence(String id) {
        if (!isGooglePlayServicesAvailable()){
            //LOG ERROR
            return;
        }
        try {
            //TODO: Mover geoFenceStorage al ResultCallback
            geoFenceStorage.clearGeofence(id);
            ArrayList<String> tmp = new ArrayList<>(1);
            tmp.add(id);
            if (isGooglePlayServicesAvailable() && initialized){
                LocationServices.GeofencingApi.removeGeofences(mGoogleApiClient, tmp).setResultCallback(onRemoveGeofence);
            }
        } catch (SecurityException securityException) {
            // Catch exception generated if the app does not use ACCESS_FINE_LOCATION permission.
            logSecurityException(securityException);
        }
	}

    /**
     * Remove all GeoFences.
     */
    public void removeAllGeofences() {
        if (!isGooglePlayServicesAvailable()){
            //LOG ERROR
            return;
        }
        try {
            // Remove geofences.
            //TODO: Mover geoFenceStorage al ResultCallback
            geoFenceStorage.clearGeofence();
        } catch (SecurityException securityException) {
            // Catch exception generated if the app does not use ACCESS_FINE_LOCATION permission.
            logSecurityException(securityException);
        }
	}

    /**
     * Add GeoFence.
     *
     * @param id The id of GeoFence
     * @param geofenceData the geofence data
     */
    public void addGeofence(String id, String geofenceData) {
        if(initialized) {
            // geofenceData: 97e449d0-8691-4994-b67e-342062f43826:25.684538:-80.315442:250.0:ENTER:NAME
            String str[] = geofenceData.split(":");
            double lat = Double.parseDouble(str[1]);
            double lon = Double.parseDouble(str[2]);
            float radius = Float.parseFloat(str[3]);

            int type;
            switch (str[4]) {
                case geoEnterTrigger:
                    type = Geofence.GEOFENCE_TRANSITION_ENTER;
                    break;
                case geoExitTrigger:
                    type = Geofence.GEOFENCE_TRANSITION_EXIT;
                    break;
                default:
                    type = Geofence.GEOFENCE_TRANSITION_ENTER | Geofence.GEOFENCE_TRANSITION_EXIT;
            }
            GeoFenceVO geoFence = geoFenceStorage.getGeofence(id);
            if (geoFence == null) {
                geoFence = new GeoFenceVO(id, lat, lon, radius, str[5],Geofence.NEVER_EXPIRE, type);
            } else {
                geoFence = new GeoFenceVO(id, lat, lon, radius, str[5],Geofence.NEVER_EXPIRE, type, geoFence.getCreationTime(), geoFence.getLastEnterTime(), geoFence.getLastExitTime());
            }

            //TODO: Mover geoFenceStorage al ResultCallback
            geoFenceStorage.saveGeofence(id, geoFence);
            mGeofenceList.add(geoFence.toGeofence());
        }
	}

    /**
     * Add or update a GeoFence.
     *
     * @param geofence A geofencevo object provided by the SDK
     */
    public void addGeofence(GeoFenceVO geofence){
        if(initialized) {

            GeoFenceVO storageGeofence = geoFenceStorage.getGeofence(geofence.getId());
            if (null != storageGeofence) {
                geofence.setCreationTime(storageGeofence.getCreationTime());
                geofence.setLastEnterTime(storageGeofence.getCreationTime());
                geofence.setLastExitTime(storageGeofence.getLastExitTime());
            }
            //TODO: Mover geoFenceStorage al ResultCallback
            geoFenceStorage.saveGeofence(geofence.getId(), geofence);
            mGeofenceList.add(geofence.toGeofence());

        }
    }

    /**
     * Adds geofences, which sets alerts to be notified when the device enters or exits one of the
     * specified geofences. Handles the success or failure results returned by addGeofences().
     */
    public void watchGeofence(){
        Util.debug("Watch Geofence");
        if (!isGooglePlayServicesAvailable() || !initialized){
            //LOG ERROR
            Util.debug("ERROR EN WATCH GEOFENCE");
            return;
        }
        try {
            Util.debug("Geofence init triggers");
            LocationServices.GeofencingApi.addGeofences(mGoogleApiClient, getGeofencingRequest(), createRequestPendingIntent()).setResultCallback(onAddGeofence);
        } catch (SecurityException securityException) {
            // Catch exception generated if the app does not use ACCESS_FINE_LOCATION permission.
            logSecurityException(securityException);
        }
    }


	/*****************************************************************************
	 ************************* BEACONS *******************************************
	 *****************************************************************************/

    /**
     *
     * @return
     */
    public List<BeaconVO> getBeaconList(){
        SharedPreferences beaconPref = context.getSharedPreferences("beacons", 0);
        Map<String,String> keys = (Map<String, String>) beaconPref.getAll();
        List<BeaconVO> beacons = new ArrayList<>();
        for(Map.Entry<String,String> entry : keys.entrySet()){
            // 2472f171-d26f-45d4-bff6-1df0f1d9b5e5:B9407F30-F5F8-466E-AFF9-25556B57FE6D:1010:9090:ESTIMOTE:IMMEDIATE:ENTER:Fruits
            String beaconStr[] = entry.getValue().split(":");
            BeaconVO beacon = new BeaconVO(entry.getKey(),beaconStr[1],beaconStr[2],beaconStr[3],beaconStr[4],beaconStr[5],beaconStr[6],beaconStr[7]);
            beacons.add(beacon);
        }
        return beacons;
    }

    /**
     * Remove all beacons.
     */
    public void removeAllBeacons() {
		SharedPreferences beacons = context.getSharedPreferences("beacons", 0);
		SharedPreferences.Editor editor = beacons.edit();
		editor.clear();
		editor.apply();
	}

    /**
     * Delete beacon.
     *
     * @param id the id of Beacon to Delete
     */
    public void deleteBeacon(String id) {
		SharedPreferences beacons = context.getSharedPreferences("beacons", 0);
		SharedPreferences.Editor editor = beacons.edit();
		editor.remove(id + ".EXIT");
		editor.remove(id + ".ENTER");
		editor.apply();

		SharedPreferences beaconsStatus = context.getSharedPreferences("beaconsstatus", 0);
		editor = beaconsStatus.edit();
		editor.remove(id + ".timestamp.EXIT");
        editor.remove(id + ".timestamp.ENTER");
		editor.apply();
	}

    /**
     * Add beacon.
     *
     * @param id the id of Beacon to Add
     * @param beaconData the beacon data
     */
    public void addBeacon(String id, String beaconData) {
		if (beaconsSupported) {
			String str[] = beaconData.split(":");
			String type = str[6];
			
			SharedPreferences beacons = context.getSharedPreferences("beacons", 0);
			SharedPreferences.Editor editor = beacons.edit();
			editor.putString(id + "." + type, beaconData);
			editor.apply();

			try {
				if ("-1".equals(str[2])) {
					str[2] = null;
				}
				
				if ("-1".equals(str[3])) {
					str[3] = null;
				}
				
				Beacon beacon = new Beacon.Builder().setId1(str[1]).setId2(str[2]).setId3(str[3]).build();
				Region region = new Region(id, beacon.getId1(), beacon.getId2(), beacon.getId3());
				beaconManager.startMonitoringBeaconsInRegion(region);
			} catch (Exception _e) {
				_e.printStackTrace();
                RemoteAPI.logEvent(context, "ERROR", "Error while registering beacon '" + beaconData + "': " + _e.getMessage(),null);
				Util.debug("Error while registering beacon '" + beaconData + "': " + _e.getMessage());
			}
		}
	}

    /**
     * Returns <code>true</code> if beacons are supported.
     *
     * @return <code>true</code> if beacons are supported; <code>false</code> otherwise
     */
    public boolean areBeaconsSupported() {
		return beaconsSupported;
	}

	private synchronized void onBeaconEvent(final Region region, final String event) {
		if (!beaconsSupported) {
			return;
		}

		SharedPreferences beaconsPreferences = context.getSharedPreferences("beacons", 0);
		SharedPreferences beaconsStatus = context.getSharedPreferences("beaconsstatus", 0);
		
		final String id = region.getUniqueId();
		String beaconData = beaconsPreferences.getString(id + "." + event, null);
		if (beaconData != null) {
			String str[] = beaconData.split(":");
            Messangi.getInstance().logEvent(context,"BEACON_"+event, id,str[7]); // LOG BEACON EVENT in BigData


			String type = str[6];
            Date timestamp = new Date(beaconsStatus.getLong(id + ".timestamp." + event, 0));
			Calendar now = Calendar.getInstance();
			double minutes = (now.getTimeInMillis() - timestamp.getTime())/1000.0/60.0;
			
			if (!type.contains(event)) {
				return;
			}
			
			SharedPreferences.Editor editor = beaconsStatus.edit();
			editor.putLong(str[0] + ".timestamp." + event, new Date().getTime());
			editor.apply();
			
			// The user must spend 5 CONTINUOUS minutes outside the beacon for the event to be triggered again
			
			if (minutes >= 5) {
				Location location = getLastKnownLocation();
				final double latitude = (location != null) ? location.getLatitude() : -1;
				final double longitude = (location != null) ? location.getLongitude() : -1;

				new Thread() {
					public void run() {
						RemoteAPI.geoFenceEvent(context, id, event, latitude, longitude, "BEACON");
					}
				}.start();
			}
		}
	}

	@Override
	public void didDetermineStateForRegion(int state, Region region) {
		// TODO Auto-generated method stub
	}

	@Override
	public void didEnterRegion(Region region) {
		onBeaconEvent(region, "ENTER");
	}

	@Override
	public void didExitRegion(Region region) {
        onBeaconEvent(region, "EXIT");
	}

	@Override
	public boolean bindService(Intent intent, ServiceConnection serviceConnection, int val) {
		return context.bindService(intent, serviceConnection, val);
	}

	@Override
	public void onBeaconServiceConnect() {
		beaconManager.setMonitorNotifier(this);
		BackgroundPowerSaver backgroundPowerSaver = new BackgroundPowerSaver(context.getApplicationContext());
		beaconManager.setBackgroundBetweenScanPeriod(60000l);
		beaconManager.setForegroundBetweenScanPeriod(10000l);

		new Thread() {
			public void run() {
				RemoteAPI.getBeacons(context);
			}
		}.start();
	}

	@Override
	public void unbindService(ServiceConnection arg0) {
	}


    /*****************************************************************************
     ****************************GoogleApiClient**********************************
     *****************************************************************************/

    /**
     * Function to build a GoogleApiClient
     * @param context the contex of activity
     */
    protected synchronized void buildGoogleApiClient(Context context) {
        mGoogleApiClient = new GoogleApiClient.Builder(context)
                .addConnectionCallbacks(this)
                .addOnConnectionFailedListener(this)
                .addApi(LocationServices.API)
                .build();
    }

    /**
     * Checks if Google Play services is available.
     * @return true if it is.
     */
    private boolean isGooglePlayServicesAvailable() {
        int resultCode = GooglePlayServicesUtil.isGooglePlayServicesAvailable(context);
        if (resultCode != ConnectionResult.SUCCESS) {
            if (GooglePlayServicesUtil.isUserRecoverableError(resultCode)) {
                Util.debug("Error recuperable "+resultCode);
            }else{
                Util.debug("This device is not supported.");
            }
            return false;
        }
        return true;
    }

    /**********************************
     *** ConnectionCallbacks **********
     **********************************/

    /**
     * Invoke when Google play service are bond successfully
     * @param bundle
     */
	@Override
	public void onConnected(Bundle bundle) {
        if (mRequestingLocationUpdates) {
            LocationServices.FusedLocationApi.requestLocationUpdates(mGoogleApiClient, createLocationRequest(), this);
        }
        lastKnownLocation = LocationServices.FusedLocationApi.getLastLocation(mGoogleApiClient);

        new Thread() {
            public void run() {
                startGeofences();
            }
        }.start();

	}

    /**
     * Invoke Google play service is temporally suspended
     * @param i
     */
	@Override
	public void onConnectionSuspended(int i) {
        Util.debug("Connection Suspended Status: " + i);
	}

    /*************************************
     *** OnConnectionFailedListener ******
     *************************************/

    /**
     * Call if Google Play Service can't connect
     * @param connectionResult Error ConnectionResult
     */
	@Override
	public void onConnectionFailed(ConnectionResult connectionResult) {
        Util.debug("Google API Client ERROR");
        Util.debug(connectionResult.toString());
	}

    /**
     * Function to connect Google Api Client
     */
    public void onStart() {
        if(!mGoogleApiClient.isConnected())
            mGoogleApiClient.connect();
    }


    /**
     * Function to Resume Location Updates
     */
    public void onResume(){
        if (mGoogleApiClient.isConnected() && mRequestingLocationUpdates) {
            startLocationUpdates();
        }
    }

    /**
     * Function to Pause Location Updates
     */
    public void onPause(){
        if(mGoogleApiClient.isConnected())
            LocationServices.FusedLocationApi.removeLocationUpdates(mGoogleApiClient, this);
    }

}