package com.ogangi.messangi.android.sdk;

import android.content.Context;
import android.content.SharedPreferences;
import android.location.Location;
import android.os.Environment;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.ogangi.messangi.android.sdk.vo.GeoPointVO;
import com.ogangi.messangi.android.sdk.vo.MessageVO;

import org.apache.commons.codec.binary.Base64;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

/**
 * Multiple utility functions for the Library
 */
public class Util {
	private static final String LOG_TAG = "MessangiPush";
	private static final boolean LOG_TOGGLE = true;
	// private static final String API_CLIENT_PUBLIC_KEY =
	// "wzuuzVeVA4hR9Xc8kWQB";
	private static final String ALGORITHM = "HMACSHA256";
	private static final String CHARSET_NAME = "UTF-8";

	public static Boolean isMobileAvailable(Context appcontext) {
		TelephonyManager tel = (TelephonyManager) appcontext.getSystemService(Context.TELEPHONY_SERVICE);
		return (tel.getNetworkOperator() != null && tel.getNetworkOperator().length()>0);
	}


    /**
     * Function for log debug data
     *
     * @param text the text to add log
     */
    public static void debug(String text) {
		if (LOG_TOGGLE) {
			Log.d(LOG_TAG, text);
		}
	}

    /**
     * Function for log info data
     *
     * @param text the text to add log
     */
    public static void info(String text) {
		if (LOG_TOGGLE) {
			Log.i(LOG_TAG, text);
		}
	}

    /**
     * Remove especial characters from string
     *
     * @param aux String to replace
     * @return The string cleared
     */
	@SuppressWarnings("unused")
    public static String cleanString(String aux) {
		return aux.replaceAll("\\?", "").replaceAll("!", "").replaceAll(":", "").replaceAll(",", "")
				.replaceAll("\\*", "x");
	}

    /**
     * Gets Disk root.
     *
     * @return the Disk root
     */
	@SuppressWarnings("unused")
    public static String getSDRoot() {
		return Environment.getExternalStorageDirectory().getAbsolutePath();
	}

    /**
     * Read file.
     *
     * @param fileName the filename to read
     * @return The String read
     * @throws IOException the iO exception
     */
    public static byte[] readFile(String fileName) throws IOException {
		File srcFile = new File(fileName);
		byte[] fileBytes = new byte[(int) srcFile.length()];
		FileInputStream fis = new FileInputStream(srcFile);
		int total = 0;
		do {
			int read = fis.read(fileBytes);
			if (read > 0) {
				total += read;
			}
		} while (total < srcFile.length());
		fis.close();
		return fileBytes;
	}

    /**
     * Copy file from directory to another
     *
     * @param srFile the source file
     * @param dtFile the destination file
     */
	@SuppressWarnings("unused")
    public static void copyfile(String srFile, String dtFile) {
		try {
			File f1 = new File(srFile);
			File f2 = new File(dtFile);
			InputStream in = new FileInputStream(f1);
			OutputStream out = new FileOutputStream(f2);
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0) {
				out.write(buf, 0, len);
			}
			in.close();
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
     * Encode url
     *
     * @param url to encode
     * @return the URL encoded
     */
    public static String escapeUrl(String url) {
		url = url.replace(",", "%2C");
		url = url.replace(" ", "%20");
		url = url.replace("-", "%2D");
		url = url.replace("_", "%5F");
		return url;
	}

    /**
     * Generate time stamp.
     *
     * @return the string with timeStamp
     */
    public static String generateTimeStamp() {
        Calendar now = Calendar.getInstance();
        long millis = now.getTimeInMillis();
        Util.debug("Time Stamp: " + millis + "");
		return (millis + "");
	}

    /**
     * Generate signature for REST API
     *
	 * @param url the input to make signature
     * @param privateKey
	 * @return The signature generated
     */
    public static String generateSignature(String url, String privateKey) {
		try {
			Util.debug("UNSIGNED: " + url);
			SecretKey signingKey = new SecretKeySpec(privateKey.getBytes(), ALGORITHM);
			Mac mac = Mac.getInstance(ALGORITHM);
			mac.init(signingKey);
			byte[] digest = mac.doFinal(url.getBytes(CHARSET_NAME));
			char[] hexadecimals = new char[digest.length * 2];
			for (int i = 0; i < digest.length; ++i) {
				for (int j = 0; j < 2; ++j) {
					int value = (digest[i] >> (4 - 4 * j)) & 0xf;
					char base = (value < 10) ? ('0') : ('a' - 10);
					hexadecimals[i * 2 + j] = (char) (base + value);
				}
			}
			return new String(hexadecimals);
		} catch (Exception e) {
			e.printStackTrace();
			return url;
		}
	}

    /**
	 * @deprecated Use distanceBetween(Location locationA, Location locationB)
	 *
     * Calc the lateral distance from two points
     *
     * @param p1 the first point
     * @param p2 the second point
     * @return The lateral distance between p1 and p2
     */
    public static float distanceBetween(GeoPointVO p1, GeoPointVO p2) {
		double earthRadius = 3958.75;
		double dLat = Math.toRadians(p2.getLatitude() - p1.getLatitude());
		double dLng = Math.toRadians(p2.getLongitude() - p1.getLongitude());
		double a = Math.sin(dLat / 2) * Math.sin(dLat / 2) + Math.cos(Math.toRadians(p1.getLatitude()))
				* Math.cos(Math.toRadians(p2.getLatitude())) * Math.sin(dLng / 2) * Math.sin(dLng / 2);
		double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
		double dist = earthRadius * c;
		int meterConversion = 1609;
		return (float)(dist * meterConversion);
	}

	public static float distanceBetween(Location locationA, Location locationB){
		return locationA.distanceTo(locationB);
	}

    /**
     * Safe base 64 encode.
     *
     * @param value the value
     * @return the string
     */
    public static String safeBase64Encode(String value){
		String b64String = new String(Base64.encodeBase64(value.getBytes())); 
		return b64String.replace("+", "-").replace("/", "_").replace("=", "*");
	}

    /**
     * Safe base 64 encode.
     *
     * @param value the value
     * @param encoding the encoding
     * @return the string
     */
    @SuppressWarnings("unused")
    public static String safeBase64Encode(String value, String encoding){
		String b64String = "";
		try {
			b64String = new String(Base64.encodeBase64(value.getBytes(encoding)));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} 
		return b64String.replace("+", "-").replace("/", "_").replace("=", "*");
		
	}

    /**
     * Safe base 64 decode.
     *
     * @param value the value
     * @return the string
     */
    @SuppressWarnings("unused")
    public static String safeBase64Decode(String value){
		
		String b64String = value.replace("-", "+").replace("_", "/").replace("*", "=");
		if(Base64.isArrayByteBase64(b64String.getBytes())){
			return new String(Base64.decodeBase64(b64String.getBytes()));
		}
		
		return null;		
		
	}

    /**
     * Safe base 64 decode.
     *
     * @param value the value
     * @param encoding the encoding
     * @return the string
     */
    @SuppressWarnings("unused")
    public static String safeBase64Decode(String value, String encoding){
		
		String b64String = value.replace("-", "+").replace("_", "/").replace("*", "=");
		try {
			if(Base64.isArrayByteBase64(b64String.getBytes(encoding))){
				return new String(Base64.decodeBase64(b64String.getBytes(encoding)));
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		
		return null;		
		
	}




	/**
	 * Return the directory where store push Notifications
	 *
	 * @param ctx context of the APP
	 * @return String with the Path where is storing all push notifications received
	 */
	private static String getInboxDirectory(Context ctx) {
		String inboxDirectory = "";
		try{
			File rootDir = new File(ctx.getFilesDir(), Constants.NOTIFICATION_INBOX);
			if (!rootDir.exists() && !rootDir.mkdir()) {
				throw new Exception();
			}
			inboxDirectory = rootDir.getAbsolutePath();
		}catch (Exception e){
			e.printStackTrace();
		}

		return inboxDirectory;
	}

    /**
     * Return All Notification Stored
     *
     * @param ctx context of the APP
     * @return Array of files with all Push Notification stored
     */
    private static List getAllInboxFiles(Context ctx) {

        File rootDir = new File(ctx.getFilesDir(), Constants.NOTIFICATION_INBOX);
        if (!rootDir.exists() && !rootDir.mkdir()) {
            return new ArrayList();
        }
        return Arrays.asList(rootDir.listFiles());
    }




	/**
	 * Invoked by firePushNotificationEvent, when the Push Notification is arrive by first Time and don't have a MessangiListener Register yet
	 * @param ctx Context of the APP
	 * @param name Filename with push notification was saved ****
	 */
	public static synchronized void storeUndeliveredMessage(Context ctx, String name) {
		Util.debug("Storing undelivered message '" + name + "'");

		SharedPreferences undelivered = ctx.getSharedPreferences("undelivered", 0);
		String tmp = undelivered.getString("undelivered.name", null);

		if (tmp == null) {
			tmp = name;
		} else {
			tmp += ";;" + name;
		}

		SharedPreferences.Editor editor = undelivered.edit();
		editor.putString("undelivered.name", tmp);
		editor.apply();

		Util.debug("storeUndeliveredMessage - Undelivered messages is '"
				+ undelivered.getString("undelivered.name", "") + "'");
	}

	/**
	 *
	 * @param ctx Context of the APP
	 * @return Array of String with filename of all Push Notification stored
	 */
	public static synchronized List<String> getUndeliveredMessages(Context ctx) {
		SharedPreferences undelivered = ctx.getSharedPreferences("undelivered", 0);
		String messages = undelivered.getString("undelivered.name", null);

		Util.debug("getUndeliveredMessages - Undelivered messages is '" + undelivered.getString("undelivered.name", "")+ "'");

		SharedPreferences.Editor editor = undelivered.edit();
		editor.remove("undelivered.name");
		editor.apply();

		if (messages != null) {
			return Arrays.asList(messages.split(";;"));
		}
		return new ArrayList<>();

	}





	/**
	 * Internal function for store Push Notification when don't have MessangiListenet Register
	 * @param ctx context of the APP
	 * @param message MessageVO object Received
	 * @return The FileName that was Stored
	 */
	public static String storeMessage(Context ctx, MessageVO message){
		try {
			File rootDir = new File(getInboxDirectory(ctx));

			if (!rootDir.exists()) {
				throw new FileNotFoundException("Can't create inbox dir");
			}

			File file = new File(rootDir.getAbsolutePath() + "/" + message.getMessageIDAux());
			FileOutputStream fos = new FileOutputStream(file);
			fos.write(message.toJson().getBytes());
			fos.close();

			return file.getName();
		} catch (Exception _e) {
			_e.printStackTrace();
			return "";
		}
	}

	/**
	 * Transform all Messages from Files to MessageVO
	 * @param ctx context of the APP
	 * @return Array of MessageVO Objects with all Push Received
	 */
	public static List getMessages(Context ctx) {
		List<File> files = getAllInboxFiles(ctx);
		ArrayList<MessageVO> messages = new ArrayList<>(files.size());

		for(File file : files){
			try {
				String json = new String(Util.readFile(file.getAbsolutePath()));
				if(json.length() > 0) {
					MessageVO message = MessageVO.fromJson(json);
					messages.add(message);
				}else{
					file.delete();
				}
			} catch (Exception _e) {
				_e.printStackTrace();
			}
		}

		return messages;
	}

	/**
	 *
	 * @param ctx context of the APP
	 * @param fileName String with filename where is storing a push notification
	 * @return a MessageVO with the info stored in fileName
	 */
	public static MessageVO getMessage(Context ctx, String fileName) {
		try {
			File file = new File(getInboxDirectory(ctx) + "/" + fileName);
			String json = new String(Util.readFile(file.getAbsolutePath()));
			return MessageVO.fromJson(json);
		} catch (Exception _e) {
			_e.printStackTrace();
		}
		return null;
	}

	/**
	 * Delete push notification stored
	 * @param ctx context of the APP
	 * @param id The ID of push notification to Delete
	 */
	public static void deleteMessage(Context ctx, String id) {
		List<File> files = getAllInboxFiles(ctx);
		try {
			for (File file : files) {
				if (file.getName().startsWith(id)) {
					if (!file.delete()) {
						throw new Exception();
					}
					break;
				}
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}

	public static void deleteMessages(Context ctx) {
		List<File> files = getAllInboxFiles(ctx);
		try {
			for (File file : files) {
				if (!file.delete()) {
					throw new Exception();
				}
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}
}

