package com.ogangi.messangi.android.sdk.vo;

import android.util.Xml;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringReader;

/**
 * Represents a generic response from Messangi REST API,
 * providing access to response and message
 */
@SuppressWarnings("unused")
public class AnswerVO implements Serializable{
	private static final String ns = null;
	private static final long serialVersionUID = -6950205064269499534L;
	private String response;
	private String message;

	public AnswerVO(String response, String message) {
		this.response = response;
		this.message = message;
	}

	/**
     * Returns the response of AnswerVO Object
     * @return String with response from Messangi REST API
     */
	public String getResponse() {
		return response;
	}

    public String getStatus(){
        return response;
    }

    /**
     * Sets the response of AnswerVO Object
     * @param response String with response to store
     */
	public void setResponse(String response) {
		this.response = response;
	}

    /**
     * Returns the message of AnswerVO Object
     * @return String with the message of notification
     */
	public String getMessage() {
		return message;
	}

    /**
     * Sets the message of AnswerVO Object
     * @param message String with message to store
     */
	public void setMessage(String message) {
		this.message = message;
	}



    //TODO: Considerar mover esto a un padre comun de todas las clases VO y extender

    public static AnswerVO parse(String in) throws XmlPullParserException, IOException {
        XmlPullParser parser = Xml.newPullParser();
        parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
        parser.setInput(new StringReader(in));
        parser.nextTag();
        return parseAnswerVO(parser);
    }

	// Parses the contents of an XML entry. If it encounters a successful or message tag, hands them off
	// to their respective "read" methods for processing. Otherwise, skips the tag.
	public static AnswerVO parseAnswerVO(XmlPullParser parser) throws XmlPullParserException, IOException {
		parser.require(XmlPullParser.START_TAG, ns, "transactionResult");
		String response = null;
		String message = null;
		while (parser.next() != XmlPullParser.END_TAG) {
			if (parser.getEventType() != XmlPullParser.START_TAG) {
				continue;
			}
			String name = parser.getName();
			if (name.equals("successful")) {
				response = readStatus(parser);
			} else if (name.equals("message")) {
				message = readMessage(parser);
			} else {
				skip(parser);
			}
		}
		return new AnswerVO(response, message);
	}

	// Processes successful tags in the feed.
	private static String readStatus(XmlPullParser parser) throws IOException, XmlPullParserException {
		parser.require(XmlPullParser.START_TAG, ns, "successful");
		String status = readText(parser);
		parser.require(XmlPullParser.END_TAG, ns, "successful");
		return status;
	}

	// Processes message tags in the feed.
	private static String readMessage(XmlPullParser parser) throws IOException, XmlPullParserException {
		parser.require(XmlPullParser.START_TAG, ns, "message");
		String message = readText(parser);
		parser.require(XmlPullParser.END_TAG, ns, "message");
		return message;
	}

	// For the tags, extracts text values.
	private static String readText(XmlPullParser parser) throws IOException, XmlPullParserException {
		String result = "";
		if (parser.next() == XmlPullParser.TEXT) {
			result = parser.getText();
			parser.nextTag();
		}
		return result;
	}

	private static void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
		if (parser.getEventType() != XmlPullParser.START_TAG) {
			throw new IllegalStateException();
		}
		int depth = 1;
		while (depth != 0) {
			switch (parser.next()) {
				case XmlPullParser.END_TAG:
					depth--;
					break;
				case XmlPullParser.START_TAG:
					depth++;
					break;
			}
		}
	}
}

