package com.ogangi.messangi.android.sdk.vo;

import android.util.Xml;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a Beacon Object retrieved from Messangi
 */
public class BeaconVO {
    //Esto es lo que dice el API
    private String regionId;
    private String name;
    private String type;
    private String eventType;
    private String msgTitle;
    private String msgContent;
    private String appName;
    private String timezone;
    private String update;
    private String macAddress;
    private String manufacturer;
    private String distance;

    //Esto es lo que dice el codigo
    //private String regionId;
    private String uuid;
    private String major;
    private String minor;
    //private String manufacturer;
    private String distanceString;
    //private String eventType;
    //private String name;


    public BeaconVO(String regionId, String uuid, String major, String minor, String manufacturer, String distanceString, String eventType, String name) {
        this.regionId = regionId;
        this.eventType = eventType;
        this.name = name;
        this.uuid = uuid;
        this.manufacturer = manufacturer;
        this.distanceString = distanceString;
        this.minor = minor;
        this.major = major;
    }

    public BeaconVO(String regionId, String name, String type, String eventType, String msgTitle, String msgContent, String appName, String timezone, String update, String macAddress, String manufacturer, String distance, String uuid, String major, String minor, String distanceString) {
        this.regionId = regionId;
        this.name = name;
        this.type = type;
        this.eventType = eventType;
        this.msgTitle = msgTitle;
        this.msgContent = msgContent;
        this.appName = appName;
        this.timezone = timezone;
        this.update = update;
        this.macAddress = macAddress;
        this.manufacturer = manufacturer;
        this.distance = distance;
        this.uuid = uuid;
        this.major = major;
        this.minor = minor;
        this.distanceString = distanceString;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getEventType() {
        return eventType;
    }

    public void setEventType(String eventType) {
        this.eventType = eventType;
    }

    public String getDistanceString() {
        return distanceString;
    }

    public void setDistanceString(String distanceString) {
        this.distanceString = distanceString;
    }

    public String getMajor() {
        return major;
    }

    public void setMajor(String major) {
        this.major = major;
    }

    public String getMinor() {
        return minor;
    }

    public void setMinor(String minor) {
        this.minor = minor;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getManufacturer() {
        return manufacturer;
    }

    public void setManufacturer(String manufacturer) {
        this.manufacturer = manufacturer;
    }

    public String toString(){
        String message = regionId;
        message += ":" + uuid;
        message += ":" + major;
        message += ":" + minor;
        message += ":" + manufacturer;
        message += ":" + distanceString;
        message += ":" + eventType;
        message += ":" + name;
        return message;
    }

    //TODO: Considerar mover esto a un padre comun de todas las clases VO y extender
    private static final String ns = null;

    public static List parse(String in) throws XmlPullParserException, IOException {
        XmlPullParser parser = Xml.newPullParser();
        parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
        parser.setInput(new StringReader(in));
        parser.nextTag();
        return readFeed(parser);
    }


    private static List readFeed(XmlPullParser parser) throws XmlPullParserException, IOException {
        List<BeaconVO> entries = new ArrayList<>();

        parser.require(XmlPullParser.START_TAG, ns, "BeaconRegions");
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG) {
                continue;
            }
            String name = parser.getName();
            // Starts by looking for the entry tag
            if (name.equals("beacon")) {
                entries.add(parseBeaconVO(parser));
            } else {
                skip(parser);
            }
        }
        return entries;
    }

    // Parses the contents of an XML entry. If it encounters a successful or message tag, hands them off
    // to their respective "read" methods for processing. Otherwise, skips the tag.
    public static BeaconVO parseBeaconVO(XmlPullParser parser) throws XmlPullParserException, IOException {
        parser.require(XmlPullParser.START_TAG, ns, "beacon");

        String regionId = null;
        String name = null;
        String type = null;
        String eventType = null;
        String msgTitle = null;
        String msgContent = null;
        String appName = null;
        String timezone = null;
        String update = null;
        String macAddress = null;
        String manufacturer = null;
        String distance = null;
        String uuid = null;
        String major = null;
        String minor = null;
        String distanceString = null;

        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG) {
                continue;
            }
            String nameTag = parser.getName();

            switch (nameTag){
                case "regionId":
                    regionId = read(parser,nameTag);
                    break;
                case "name":
                    name = read(parser,nameTag);
                    break;
                case "type":
                    type = read(parser,nameTag);
                    break;
                case "eventType":
                    eventType = read(parser,nameTag);
                    break;
                case "msgTitle":
                    msgTitle = read(parser,nameTag);
                    break;
                case "msgContent":
                    msgContent = read(parser,nameTag);
                    break;
                case "appName":
                    appName = read(parser,nameTag);
                    break;
                case "timezone":
                    timezone = read(parser,nameTag);
                    break;
                case "update":
                    update = read(parser,nameTag);
                    break;
                case "macAddress":
                    macAddress = read(parser,nameTag);
                    break;
                case "manufacturer":
                    manufacturer = read(parser,nameTag);
                    break;
                case "distance":
                    distance = read(parser,nameTag);
                    break;
                case "uuid":
                    uuid = read(parser,nameTag);
                    break;
                case "major":
                    major = read(parser,nameTag);
                    break;
                case "minor":
                    minor = read(parser,nameTag);
                    break;
                case "distanceString":
                    distanceString = read(parser,nameTag);
                    break;
                default:
                    skip(parser);
            }
        }
        return new BeaconVO(regionId,name,type,eventType,msgTitle,msgContent,appName,timezone,update,macAddress,manufacturer,distance,uuid,major,minor,distanceString);
    }

    // Processes id tags in the feed.
    private static String read(XmlPullParser parser,String tag) throws IOException, XmlPullParserException {
        parser.require(XmlPullParser.START_TAG, ns, tag);
        String tagContent = readText(parser);
        parser.require(XmlPullParser.END_TAG, ns, tag);
        return tagContent;
    }

    // For the tags, extracts text values.
    private static String readText(XmlPullParser parser) throws IOException, XmlPullParserException {
        String result = "";
        if (parser.next() == XmlPullParser.TEXT) {
            result = parser.getText();
            parser.nextTag();
        }
        return result;
    }

    private static void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
        if (parser.getEventType() != XmlPullParser.START_TAG) {
            throw new IllegalStateException();
        }
        int depth = 1;
        while (depth != 0) {
            switch (parser.next()) {
                case XmlPullParser.END_TAG:
                    depth--;
                    break;
                case XmlPullParser.START_TAG:
                    depth++;
                    break;
            }
        }
    }


}

