package com.ogangi.messangi.android.sdk.vo;

import android.util.Xml;

import com.google.gson.Gson;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Represents a Notification retrieved from Messangi
 */
@SuppressWarnings("unused")
public class MessageVO implements Serializable {
	private static final long serialVersionUID = 7463612628372914221L;

	private String blastID;
	private String id;
	private String type;
	private String from;
	private String to;
	private String subject;
	private String text;
	private String status;
	private String statusDescription;
	private String date;
	private String timezone;
	private String clientId;
	private String appName;
	private String platform;

    public MessageVO() {
    }

    public MessageVO(String blastID, String id, String type, String from, String to, String subject, String text, String status, String statusDescription, String date, String timezone, String clientId, String appName, String platform) {
		this.blastID = blastID;
		this.id = id;
		this.type = type;
		this.from = from;
		this.to = to;
		this.subject = subject;
		this.text = text;
		this.status = status;
		this.statusDescription = statusDescription;
		this.date = date;
		this.timezone = timezone;
		this.clientId = clientId;
		this.appName = appName;
		this.platform = platform;
	}


	/**
     * Gets messageID.
     *
     * @return the messageID
     */
    public String getMessageIDAux() {
		return id;
	}

    /**
     * Sets messageID
     *
     * @param messageIDAux the messageID to set
     */
    public void setMessageIDAux(String messageIDAux) {
		this.id = messageIDAux;
	}

    /**
     * Gets message type.
     *
     * @return the message type
     */
    public String getMessageType() {
		return type;
	}

    /**
     * Sets message type.
     *
     * @param messageType the message type
     */
    public void setMessageType(String messageType) {
		this.type = messageType;
	}

    /**
     * Gets message title.
     *
     * @return the message title
     */
    public String getMessageTitle() {
		return subject;
	}

    /**
     * Sets message title.
     *
     * @param messageTitle the message title
     */
    public void setMessageTitle(String messageTitle) {
		this.subject = messageTitle;
	}

    /**
     * Gets message content.
     *
     * @return the message content
     */
    public String getMessageContent() {
		return text;
	}

    /**
     * Sets message content.
     *
     * @param messageContent the message content
     */
    public void setMessageContent(String messageContent) {
		this.text = messageContent;
	}

	public Date getMessageDate(){
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZZZZZ");
		try {
			return sdf.parse(this.date);
		} catch (ParseException e) {
			e.printStackTrace();
			return null;
		}
	}

	//TODO: Considerar mover esto a un padre comun de todas las clases VO y extender
	private static final String ns = null;

	public static List parseMultiple(String in) throws XmlPullParserException, IOException {
		XmlPullParser parser = Xml.newPullParser();
		parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
		parser.setInput(new StringReader(in));
		parser.nextTag();
		return readFeed(parser);
	}

	public static MessageVO parse(String in) throws XmlPullParserException, IOException {
		XmlPullParser parser = Xml.newPullParser();
		parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
		parser.setInput(new StringReader(in));
		parser.nextTag();
		return parseMessageVO(parser);
	}

	private static List readFeed(XmlPullParser parser) throws XmlPullParserException, IOException {
		List<MessageVO> entries = new ArrayList<>();

		parser.require(XmlPullParser.START_TAG, ns, "messages");
		while (parser.next() != XmlPullParser.END_TAG) {
			if (parser.getEventType() != XmlPullParser.START_TAG) {
				continue;
			}
			String name = parser.getName();
			// Starts by looking for the entry tag
			if (name.equals("message")) {
				entries.add(parseMessageVO(parser));
			} else {
				skip(parser);
			}
		}
		return entries;
	}

	// Parses the contents of an XML entry. If it encounters a successful or message tag, hands them off
	// to their respective "read" methods for processing. Otherwise, skips the tag.
	public static MessageVO parseMessageVO(XmlPullParser parser) throws XmlPullParserException, IOException {
		parser.require(XmlPullParser.START_TAG, ns, "message");

		String blastId = null;
		String id = null;
		String type = null;
		String from = null;
		String to = null;
		String subject = null;
		String text = null;
		String status = null;
		String statusDescription = null;
		String date = null;
		String timezone = null;
		String clientId = null;
		String appName = null;
		String platform = null;

		while (parser.next() != XmlPullParser.END_TAG) {
			if (parser.getEventType() != XmlPullParser.START_TAG) {
				continue;
			}
			String nameTag = parser.getName();

			switch (nameTag){
				case "blastId":
					blastId = read(parser,nameTag);
					break;
				case "id":
					id = read(parser,nameTag);
					break;
				case "type":
					type = read(parser,nameTag);
					break;
				case "from":
					from = read(parser,nameTag);
					break;
				case "to":
					to = read(parser,nameTag);
					break;
				case "subject":
					subject = read(parser, nameTag);
					break;
				case "text":
					text = read(parser, nameTag).replaceAll("(\\A|\\s)((http|https|ftp|mailto):\\S+)(\\s|\\z)", "$1<a href=\"$2\">$2</a>$4");
					break;
				case "status":
					status = read(parser,nameTag);
					break;
				case "statusDescription":
					statusDescription = read(parser,nameTag);
					break;
				case "date":
					date = read(parser,nameTag);
					break;
				case "timezone":
					timezone = read(parser,nameTag);
					break;
				case "clientId":
					clientId = read(parser,nameTag);
					break;
				case "appName":
					appName = read(parser,nameTag);
					break;
				case "platform":
					platform = read(parser,nameTag);
					break;
				default:
					skip(parser);
			}
		}
		return new MessageVO(blastId,id,type,from,to,subject,text,status,statusDescription,date,timezone,clientId,appName,platform);
	}

	// Processes id tags in the feed.
	private static String read(XmlPullParser parser,String tag) throws IOException, XmlPullParserException {
		parser.require(XmlPullParser.START_TAG, ns, tag);
		String tagContent = readText(parser);
		parser.require(XmlPullParser.END_TAG, ns, tag);
		return tagContent;
	}

	// For the tags, extracts text values.
	private static String readText(XmlPullParser parser) throws IOException, XmlPullParserException {
		String result = "";
		if (parser.next() == XmlPullParser.TEXT) {
			result = parser.getText();
			parser.nextTag();
		}
		return result;
	}

	private static void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
		if (parser.getEventType() != XmlPullParser.START_TAG) {
			throw new IllegalStateException();
		}
		int depth = 1;
		while (depth != 0) {
			switch (parser.next()) {
				case XmlPullParser.END_TAG:
					depth--;
					break;
				case XmlPullParser.START_TAG:
					depth++;
					break;
			}
		}
	}

	//Serialization
	static Gson gson = new Gson();

	public String toJson(){
		return gson.toJson(this);
	}

	public static MessageVO fromJson(String json){
		return gson.fromJson(json, MessageVO.class);
	}

}